import { __decorate, __param } from '../../../../../../external/tslib/tslib.es6.js';
import '../../../base/common/arrays.js';
import { Disposable } from '../../../base/common/lifecycle.js';
import { joinPath } from '../../../base/common/resources.js';
import '../../../base/common/path.js';
import '../../../base/common/platform.js';
import { IFileService } from '../../files/common/files.js';
import { IProductService } from '../../product/common/productService.js';
import '../../../base/common/async.js';
import '../../../base/common/event.js';
import '../../../base/common/process.js';
import '../../../base/common/strings.js';
import './extensionManagement.js';
import '../../instantiation/common/instantiation.js';
import '../../../base/common/network.js';
let ExtensionTipsService = class ExtensionTipsService extends Disposable {
    constructor(fileService, productService) {
        super();
        this.fileService = fileService;
        this.productService = productService;
        this.allConfigBasedTips = ( new Map());
        if (this.productService.configBasedExtensionTips) {
            Object.entries(this.productService.configBasedExtensionTips).forEach(([, value]) => this.allConfigBasedTips.set(value.configPath, value));
        }
    }
    getConfigBasedTips(folder) {
        return this.getValidConfigBasedTips(folder);
    }
    async getImportantExecutableBasedTips() {
        return [];
    }
    async getOtherExecutableBasedTips() {
        return [];
    }
    async getValidConfigBasedTips(folder) {
        const result = [];
        for (const [configPath, tip] of this.allConfigBasedTips) {
            if (tip.configScheme && tip.configScheme !== folder.scheme) {
                continue;
            }
            try {
                const content = ( (await this.fileService.readFile(joinPath(folder, configPath))).value.toString());
                for (const [key, value] of Object.entries(tip.recommendations)) {
                    if (!value.contentPattern || ( new RegExp(value.contentPattern, 'mig')).test(content)) {
                        result.push({
                            extensionId: key,
                            extensionName: value.name,
                            configName: tip.configName,
                            important: !!value.important,
                            isExtensionPack: !!value.isExtensionPack,
                            whenNotInstalled: value.whenNotInstalled
                        });
                    }
                }
            }
            catch (error) {  }
        }
        return result;
    }
};
ExtensionTipsService = ( __decorate([
    ( __param(0, IFileService)),
    ( __param(1, IProductService))
], ExtensionTipsService));
export { ExtensionTipsService };
