import { isWindows, isWeb } from '../../../base/common/platform.js';
import { localizeWithPath } from '../../../nls.js';
import { Extensions } from '../../configuration/common/configurationRegistry.js';
import { Registry } from '../../registry/common/platform.js';
const configurationRegistry = ( Registry.as(Extensions.Configuration));
configurationRegistry.registerConfiguration({
    id: 'update',
    order: 15,
    title: ( localizeWithPath(
        'vs/platform/update/common/update.config.contribution',
        'updateConfigurationTitle',
        "Update"
    )),
    type: 'object',
    properties: {
        'update.mode': {
            type: 'string',
            enum: ['none', 'manual', 'start', 'default'],
            default: 'default',
            scope: 1 ,
            description: ( localizeWithPath(
                'vs/platform/update/common/update.config.contribution',
                'updateMode',
                "Configure whether you receive automatic updates. Requires a restart after change. The updates are fetched from a Microsoft online service."
            )),
            tags: ['usesOnlineServices'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/platform/update/common/update.config.contribution',
                    'none',
                    "Disable updates."
                )),
                ( localizeWithPath(
                    'vs/platform/update/common/update.config.contribution',
                    'manual',
                    "Disable automatic background update checks. Updates will be available if you manually check for updates."
                )),
                ( localizeWithPath(
                    'vs/platform/update/common/update.config.contribution',
                    'start',
                    "Check for updates only on startup. Disable automatic background update checks."
                )),
                ( localizeWithPath(
                    'vs/platform/update/common/update.config.contribution',
                    'default',
                    "Enable automatic update checks. Code will check for updates automatically and periodically."
                ))
            ],
            policy: {
                name: 'UpdateMode',
                minimumVersion: '1.67',
            }
        },
        'update.channel': {
            type: 'string',
            default: 'default',
            scope: 1 ,
            description: ( localizeWithPath(
                'vs/platform/update/common/update.config.contribution',
                'updateMode',
                "Configure whether you receive automatic updates. Requires a restart after change. The updates are fetched from a Microsoft online service."
            )),
            deprecationMessage: ( localizeWithPath(
                'vs/platform/update/common/update.config.contribution',
                'deprecated',
                "This setting is deprecated, please use '{0}' instead.",
                'update.mode'
            ))
        },
        'update.enableWindowsBackgroundUpdates': {
            type: 'boolean',
            default: true,
            scope: 1 ,
            title: ( localizeWithPath(
                'vs/platform/update/common/update.config.contribution',
                'enableWindowsBackgroundUpdatesTitle',
                "Enable Background Updates on Windows"
            )),
            description: ( localizeWithPath(
                'vs/platform/update/common/update.config.contribution',
                'enableWindowsBackgroundUpdates',
                "Enable to download and install new VS Code versions in the background on Windows."
            )),
            included: isWindows && !isWeb
        },
        'update.showReleaseNotes': {
            type: 'boolean',
            default: true,
            scope: 1 ,
            description: ( localizeWithPath(
                'vs/platform/update/common/update.config.contribution',
                'showReleaseNotes',
                "Show Release Notes after an update. The Release Notes are fetched from a Microsoft online service."
            )),
            tags: ['usesOnlineServices']
        }
    }
});
