import { __decorate, __param } from '../../../../../../external/tslib/tslib.es6.js';
import { DisposableMap } from '../../../base/common/lifecycle.js';
import { revive } from '../../../base/common/marshalling.js';
import { ExtHostContext, MainContext } from '../common/extHost.protocol.js';
import { IChatVariablesService } from '../../contrib/chat/common/chatVariables.js';
import { extHostNamedCustomer } from '../../services/extensions/common/extHostCustomers.js';
let MainThreadChatVariables = class MainThreadChatVariables {
    constructor(extHostContext, _chatVariablesService) {
        this._chatVariablesService = _chatVariablesService;
        this._variables = ( new DisposableMap());
        this._pendingProgress = ( new Map());
        this._proxy = ( extHostContext.getProxy(ExtHostContext.ExtHostChatVariables));
    }
    dispose() {
        this._variables.clearAndDisposeAll();
    }
    $registerVariable(handle, data) {
        const registration = this._chatVariablesService.registerVariable(data, async (messageText, _arg, model, progress, token) => {
            const varRequestId = `${model.sessionId}-${handle}`;
            this._pendingProgress.set(varRequestId, progress);
            const result = revive(await this._proxy.$resolveVariable(handle, varRequestId, messageText, token));
            this._pendingProgress.delete(varRequestId);
            return result;
        });
        this._variables.set(handle, registration);
    }
    async $handleProgressChunk(requestId, progress) {
        const revivedProgress = revive(progress);
        this._pendingProgress.get(requestId)?.(revivedProgress);
    }
    $unregisterVariable(handle) {
        this._variables.deleteAndDispose(handle);
    }
};
MainThreadChatVariables = __decorate([
    extHostNamedCustomer(MainContext.MainThreadChatVariables),
    ( __param(1, IChatVariablesService))
], MainThreadChatVariables);
export { MainThreadChatVariables };
