import { __decorate, __param } from '../../../../../../external/tslib/tslib.es6.js';
import { shouldSynchronizeModel } from '../../../editor/common/model.js';
import { localizeWithPath } from '../../../nls.js';
import { IInstantiationService } from '../../../platform/instantiation/common/instantiation.js';
import { extHostCustomer } from '../../services/extensions/common/extHostCustomers.js';
import { ITextFileService } from '../../services/textfile/common/textfiles.js';
import { ExtHostContext } from '../common/extHost.protocol.js';
import { raceCancellationError } from '../../../base/common/async.js';
class ExtHostSaveParticipant {
    constructor(extHostContext) {
        this._proxy = ( extHostContext.getProxy(ExtHostContext.ExtHostDocumentSaveParticipant));
    }
    async participate(editorModel, context, _progress, token) {
        if (!editorModel.textEditorModel || !shouldSynchronizeModel(editorModel.textEditorModel)) {
            return undefined;
        }
        const p = ( new Promise((resolve, reject) => {
            setTimeout(() => reject(( new Error(( localizeWithPath(
                'vs/workbench/api/browser/mainThreadSaveParticipant',
                'timeout.onWillSave',
                "Aborted onWillSaveTextDocument-event after 1750ms"
            ))))), 1750);
            this._proxy.$participateInSave(editorModel.resource, context.reason).then(values => {
                if (!values.every(success => success)) {
                    return Promise.reject(( new Error('listener failed')));
                }
                return undefined;
            }).then(resolve, reject);
        }));
        return raceCancellationError(p, token);
    }
}
let SaveParticipant = class SaveParticipant {
    constructor(extHostContext, instantiationService, _textFileService) {
        this._textFileService = _textFileService;
        this._saveParticipantDisposable = this._textFileService.files.addSaveParticipant(instantiationService.createInstance(ExtHostSaveParticipant, extHostContext));
    }
    dispose() {
        this._saveParticipantDisposable.dispose();
    }
};
SaveParticipant = __decorate([
    extHostCustomer,
    ( __param(1, IInstantiationService)),
    ( __param(2, ITextFileService))
], SaveParticipant);
export { SaveParticipant };
