import { Codicon } from '../../../../../base/common/codicons.js';
import { localize2WithPath } from '../../../../../nls.js';
import { registerAction2, Action2, MenuId } from '../../../../../platform/actions/common/actions.js';
import { IAccessibilitySignalService, AccessibilitySignal } from '../../../../../platform/accessibilitySignal/browser/accessibilitySignalService.js';
import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
import { ViewAction } from '../../../../browser/parts/views/viewPane.js';
import { ActiveEditorContext } from '../../../../common/contextkeys.js';
import { CHAT_CATEGORY } from './chatActions.js';
import { clearChatEditor } from './chatClear.js';
import { IChatWidgetService } from '../chat.js';
import { ChatEditorInput } from '../chatEditorInput.js';
import { CONTEXT_PROVIDER_EXISTS, CONTEXT_IN_CHAT_SESSION } from '../../common/chatContextKeys.js';
const ACTION_ID_NEW_CHAT = `workbench.action.chat.newChat`;
function registerNewChatActions() {
    registerAction2(class NewChatEditorAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chatEditor.newChat',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatClearActions',
                    'chat.newChat.label',
                    "New Chat"
                )),
                icon: Codicon.plus,
                f1: false,
                precondition: CONTEXT_PROVIDER_EXISTS,
                menu: [{
                        id: MenuId.EditorTitle,
                        group: 'navigation',
                        order: 0,
                        when: ( ActiveEditorContext.isEqualTo(ChatEditorInput.EditorID)),
                    }]
            });
        }
        async run(accessor, ...args) {
            announceChatCleared(accessor);
            await clearChatEditor(accessor);
        }
    });
    registerAction2(class GlobalClearChatAction extends Action2 {
        constructor() {
            super({
                id: ACTION_ID_NEW_CHAT,
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatClearActions',
                    'chat.newChat.label',
                    "New Chat"
                )),
                category: CHAT_CATEGORY,
                icon: Codicon.plus,
                precondition: CONTEXT_PROVIDER_EXISTS,
                f1: true,
                keybinding: {
                    weight: 200 ,
                    primary: 2048  | 42 ,
                    mac: {
                        primary: 256  | 42
                    },
                    when: CONTEXT_IN_CHAT_SESSION
                },
                menu: {
                    id: MenuId.ChatContext,
                    group: 'z_clear'
                }
            });
        }
        run(accessor, ...args) {
            const widgetService = accessor.get(IChatWidgetService);
            const widget = widgetService.lastFocusedWidget;
            if (!widget) {
                return;
            }
            announceChatCleared(accessor);
            widget.clear();
            widget.focusInput();
        }
    });
}
const getNewChatActionDescriptorForViewTitle = (viewId, providerId) => ({
    viewId,
    id: `workbench.action.chat.${providerId}.newChat`,
    title: ( localize2WithPath(
        'vs/workbench/contrib/chat/browser/actions/chatClearActions',
        'chat.newChat.label',
        "New Chat"
    )),
    menu: {
        id: MenuId.ViewTitle,
        when: ( ContextKeyExpr.equals('view', viewId)),
        group: 'navigation',
        order: -1
    },
    precondition: CONTEXT_PROVIDER_EXISTS,
    category: CHAT_CATEGORY,
    icon: Codicon.plus,
    f1: false
});
function getNewChatAction(viewId, providerId) {
    return class NewChatAction extends ViewAction {
        constructor() {
            super(getNewChatActionDescriptorForViewTitle(viewId, providerId));
        }
        async runInView(accessor, view) {
            announceChatCleared(accessor);
            await view.clear();
            view.widget.focusInput();
        }
    };
}
function announceChatCleared(accessor) {
    accessor.get(IAccessibilitySignalService).playSignal(AccessibilitySignal.clear);
}
export { ACTION_ID_NEW_CHAT, getNewChatAction, registerNewChatActions };
