import { localize2WithPath } from '../../../../../nls.js';
import { registerAction2, Action2, MenuId } from '../../../../../platform/actions/common/actions.js';
import { IClipboardService } from '../../../../../platform/clipboard/common/clipboardService.js';
import { CHAT_CATEGORY } from './chatActions.js';
import { IChatWidgetService } from '../chat.js';
import { CONTEXT_RESPONSE_FILTERED } from '../../common/chatContextKeys.js';
import { isRequestVM, isResponseVM } from '../../common/chatViewModel.js';
function registerChatCopyActions() {
    registerAction2(class CopyAllAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.copyAll',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatCopyActions',
                    'interactive.copyAll.label',
                    "Copy All"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                menu: {
                    id: MenuId.ChatContext,
                    when: ( CONTEXT_RESPONSE_FILTERED.toNegated()),
                    group: 'copy',
                }
            });
        }
        run(accessor, ...args) {
            const clipboardService = accessor.get(IClipboardService);
            const chatWidgetService = accessor.get(IChatWidgetService);
            const widget = chatWidgetService.lastFocusedWidget;
            if (widget) {
                const viewModel = widget.viewModel;
                const sessionAsText = viewModel?.getItems()
                    .filter((item) => isRequestVM(item) || (isResponseVM(item) && !item.errorDetails?.responseIsFiltered))
                    .map(item => stringifyItem(item))
                    .join('\n\n');
                if (sessionAsText) {
                    clipboardService.writeText(sessionAsText);
                }
            }
        }
    });
    registerAction2(class CopyItemAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.copyItem',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatCopyActions',
                    'interactive.copyItem.label',
                    "Copy"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                menu: {
                    id: MenuId.ChatContext,
                    when: ( CONTEXT_RESPONSE_FILTERED.toNegated()),
                    group: 'copy',
                }
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isRequestVM(item) && !isResponseVM(item)) {
                return;
            }
            const clipboardService = accessor.get(IClipboardService);
            const text = stringifyItem(item, false);
            clipboardService.writeText(text);
        }
    });
}
function stringifyItem(item, includeName = true) {
    if (isRequestVM(item)) {
        return (includeName ? `${item.username}: ` : '') + item.messageText;
    }
    else {
        return (includeName ? `${item.username}: ` : '') + item.response.asString();
    }
}
export { registerChatCopyActions };
