import { localize2WithPath } from '../../../../../nls.js';
import { registerAction2, Action2 } from '../../../../../platform/actions/common/actions.js';
import { CHAT_CATEGORY } from './chatActions.js';
import { IChatWidgetService } from '../chat.js';
import { CONTEXT_IN_CHAT_SESSION, CONTEXT_PROVIDER_EXISTS } from '../../common/chatContextKeys.js';
import { isResponseVM } from '../../common/chatViewModel.js';
function registerChatFileTreeActions() {
    registerAction2(class NextFileTreeAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.nextFileTree',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatFileTreeActions',
                    'interactive.nextFileTree.label',
                    "Next File Tree"
                )),
                keybinding: {
                    primary: 2048  | 67 ,
                    weight: 200 ,
                    when: CONTEXT_IN_CHAT_SESSION,
                },
                precondition: CONTEXT_PROVIDER_EXISTS,
                f1: true,
                category: CHAT_CATEGORY,
            });
        }
        run(accessor, ...args) {
            navigateTrees(accessor, false);
        }
    });
    registerAction2(class PreviousFileTreeAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.previousFileTree',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatFileTreeActions',
                    'interactive.previousFileTree.label',
                    "Previous File Tree"
                )),
                keybinding: {
                    primary: 2048  | 1024  | 67 ,
                    weight: 200 ,
                    when: CONTEXT_IN_CHAT_SESSION,
                },
                precondition: CONTEXT_PROVIDER_EXISTS,
                f1: true,
                category: CHAT_CATEGORY,
            });
        }
        run(accessor, ...args) {
            navigateTrees(accessor, true);
        }
    });
}
function navigateTrees(accessor, reverse) {
    const chatWidgetService = accessor.get(IChatWidgetService);
    const widget = chatWidgetService.lastFocusedWidget;
    if (!widget) {
        return;
    }
    const focused = !widget.inputEditor.hasWidgetFocus() && widget.getFocus();
    const focusedResponse = isResponseVM(focused) ? focused : undefined;
    const currentResponse = focusedResponse ?? widget.viewModel?.getItems().reverse().find((item) => isResponseVM(item));
    if (!currentResponse) {
        return;
    }
    widget.reveal(currentResponse);
    const responseFileTrees = widget.getFileTreeInfosForResponse(currentResponse);
    const lastFocusedFileTree = widget.getLastFocusedFileTreeForResponse(currentResponse);
    const focusIdx = lastFocusedFileTree ?
        (lastFocusedFileTree.treeIndex + (reverse ? -1 : 1) + responseFileTrees.length) % responseFileTrees.length :
        reverse ? responseFileTrees.length - 1 : 0;
    responseFileTrees[focusIdx]?.focus();
}
export { registerChatFileTreeActions };
