import { localize2WithPath, localizeWithPath } from '../../../../../nls.js';
import { registerAction2, Action2, MenuId } from '../../../../../platform/actions/common/actions.js';
import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
import { ViewAction } from '../../../../browser/parts/views/viewPane.js';
import { ActiveEditorContext } from '../../../../common/contextkeys.js';
import { IViewsService } from '../../../../services/views/common/viewsService.js';
import { CHAT_CATEGORY } from './chatActions.js';
import { IChatWidgetService } from '../chat.js';
import { ChatEditorInput } from '../chatEditorInput.js';
import { CONTEXT_PROVIDER_EXISTS } from '../../common/chatContextKeys.js';
import { IChatContributionService } from '../../common/chatContributionService.js';
import { IChatService } from '../../common/chatService.js';
import { IEditorGroupsService } from '../../../../services/editor/common/editorGroupsService.js';
import { IEditorService, AUX_WINDOW_GROUP, ACTIVE_GROUP } from '../../../../services/editor/common/editorService.js';
var MoveToNewLocation;
( (function(MoveToNewLocation) {
    MoveToNewLocation["Editor"] = "Editor";
    MoveToNewLocation["Window"] = "Window";
})(MoveToNewLocation || (MoveToNewLocation = {})));
const getMoveToChatActionDescriptorForViewTitle = (viewId, providerId, moveTo) => ({
    id: `workbench.action.chat.${providerId}.openIn${moveTo}`,
    title: {
        value: moveTo === MoveToNewLocation.Editor ? ( localizeWithPath(
            'vs/workbench/contrib/chat/browser/actions/chatMoveActions',
            'chat.openInEditor.label',
            "Open Chat in Editor"
        )) : ( localizeWithPath(
            'vs/workbench/contrib/chat/browser/actions/chatMoveActions',
            'chat.openInNewWindow.label',
            "Open Chat in New Window"
        )),
        original: moveTo === MoveToNewLocation.Editor ? 'Open Chat in Editor' : 'Open Chat in New Window',
    },
    category: CHAT_CATEGORY,
    precondition: CONTEXT_PROVIDER_EXISTS,
    f1: false,
    viewId,
    menu: {
        id: MenuId.ViewTitle,
        when: ( ContextKeyExpr.equals('view', viewId)),
        order: 0
    },
});
function getMoveToEditorAction(viewId, providerId) {
    return getMoveToAction(viewId, providerId, MoveToNewLocation.Editor);
}
function getMoveToNewWindowAction(viewId, providerId) {
    return getMoveToAction(viewId, providerId, MoveToNewLocation.Window);
}
function getMoveToAction(viewId, providerId, moveTo) {
    return class MoveToAction extends ViewAction {
        constructor() {
            super(getMoveToChatActionDescriptorForViewTitle(viewId, providerId, moveTo));
        }
        async runInView(accessor, view) {
            const viewModel = view.widget.viewModel;
            if (!viewModel) {
                return;
            }
            const editorService = accessor.get(IEditorService);
            const sessionId = viewModel.sessionId;
            const viewState = view.widget.getViewState();
            view.clear();
            await editorService.openEditor({ resource: ChatEditorInput.getNewEditorUri(), options: { target: { sessionId }, pinned: true, viewState: viewState } }, moveTo === MoveToNewLocation.Window ? AUX_WINDOW_GROUP : ACTIVE_GROUP);
        }
    };
}
function registerMoveActions() {
    registerAction2(class GlobalMoveToEditorAction extends Action2 {
        constructor() {
            super({
                id: `workbench.action.chat.openInEditor`,
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatMoveActions',
                    'interactiveSession.openInEditor.label',
                    "Open Chat in Editor"
                )),
                category: CHAT_CATEGORY,
                precondition: CONTEXT_PROVIDER_EXISTS,
                f1: true
            });
        }
        async run(accessor, ...args) {
            executeMoveToAction(accessor, MoveToNewLocation.Editor);
        }
    });
    registerAction2(class GlobalMoveToNewWindowAction extends Action2 {
        constructor() {
            super({
                id: `workbench.action.chat.openInNewWindow`,
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatMoveActions',
                    'interactiveSession.openInNewWindow.label',
                    "Open Chat in New Window"
                )),
                category: CHAT_CATEGORY,
                precondition: CONTEXT_PROVIDER_EXISTS,
                f1: true
            });
        }
        async run(accessor, ...args) {
            executeMoveToAction(accessor, MoveToNewLocation.Window);
        }
    });
    registerAction2(class GlobalMoveToSidebarAction extends Action2 {
        constructor() {
            super({
                id: `workbench.action.chat.openInSidebar`,
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatMoveActions',
                    'interactiveSession.openInSidebar.label',
                    "Open Chat in Side Bar"
                )),
                category: CHAT_CATEGORY,
                precondition: CONTEXT_PROVIDER_EXISTS,
                f1: true,
                menu: [{
                        id: MenuId.EditorTitle,
                        order: 0,
                        when: ( ActiveEditorContext.isEqualTo(ChatEditorInput.EditorID)),
                    }]
            });
        }
        async run(accessor, ...args) {
            return moveToSidebar(accessor);
        }
    });
}
async function executeMoveToAction(accessor, moveTo) {
    const widgetService = accessor.get(IChatWidgetService);
    const viewService = accessor.get(IViewsService);
    const chatService = accessor.get(IChatService);
    const editorService = accessor.get(IEditorService);
    const widget = widgetService.lastFocusedWidget;
    if (!widget || !('viewId' in widget.viewContext)) {
        const providerId = chatService.getProviderInfos()[0].id;
        await editorService.openEditor({ resource: ChatEditorInput.getNewEditorUri(), options: { target: { providerId }, pinned: true } }, moveTo === MoveToNewLocation.Window ? AUX_WINDOW_GROUP : ACTIVE_GROUP);
        return;
    }
    const viewModel = widget.viewModel;
    if (!viewModel) {
        return;
    }
    const sessionId = viewModel.sessionId;
    const view = await viewService.openView(widget.viewContext.viewId);
    const viewState = view.widget.getViewState();
    view.clear();
    await editorService.openEditor({ resource: ChatEditorInput.getNewEditorUri(), options: { target: { sessionId }, pinned: true, viewState: viewState } }, moveTo === MoveToNewLocation.Window ? AUX_WINDOW_GROUP : ACTIVE_GROUP);
}
async function moveToSidebar(accessor) {
    const viewsService = accessor.get(IViewsService);
    const editorService = accessor.get(IEditorService);
    const chatContribService = accessor.get(IChatContributionService);
    const editorGroupService = accessor.get(IEditorGroupsService);
    const chatEditorInput = editorService.activeEditor;
    if (chatEditorInput instanceof ChatEditorInput && chatEditorInput.sessionId && chatEditorInput.providerId) {
        await editorService.closeEditor({ editor: chatEditorInput, groupId: editorGroupService.activeGroup.id });
        const viewId = chatContribService.getViewIdForProvider(chatEditorInput.providerId);
        const view = await viewsService.openView(viewId);
        view.loadSession(chatEditorInput.sessionId);
    }
    else {
        const chatService = accessor.get(IChatService);
        const providerId = chatService.getProviderInfos()[0].id;
        const viewId = chatContribService.getViewIdForProvider(providerId);
        await viewsService.openView(viewId);
    }
}
export { getMoveToAction, getMoveToEditorAction, getMoveToNewWindowAction, registerMoveActions };
