import { Codicon } from '../../../../../base/common/codicons.js';
import marked_1 from '../../../../../../../../external/vscode-marked/lib/marked.esm.js';
import { IBulkEditService } from '../../../../../editor/browser/services/bulkEditService.js';
import { localize2WithPath } from '../../../../../nls.js';
import { registerAction2, Action2, MenuId } from '../../../../../platform/actions/common/actions.js';
import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
import { ResourceNotebookCellEdit } from '../../../bulkEdit/browser/bulkCellEdits.js';
import { CHAT_CATEGORY } from './chatActions.js';
import { IChatWidgetService } from '../chat.js';
import { CONTEXT_RESPONSE_VOTE, CONTEXT_RESPONSE, CONTEXT_CHAT_RESPONSE_SUPPORT_ISSUE_REPORTING, CONTEXT_RESPONSE_FILTERED, CONTEXT_IN_CHAT_SESSION, CONTEXT_IN_CHAT_INPUT, CONTEXT_REQUEST } from '../../common/chatContextKeys.js';
import { IChatService, InteractiveSessionVoteDirection } from '../../common/chatService.js';
import { isResponseVM, isRequestVM } from '../../common/chatViewModel.js';
import { NOTEBOOK_EDITOR_ID, CellKind } from '../../../notebook/common/notebookCommon.js';
import { NOTEBOOK_IS_ACTIVE_EDITOR } from '../../../notebook/common/notebookContextKeys.js';
import { IEditorService } from '../../../../services/editor/common/editorService.js';
function registerChatTitleActions() {
    registerAction2(class MarkHelpfulAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.markHelpful',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatTitleActions',
                    'interactive.helpful.label',
                    "Helpful"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.thumbsup,
                toggled: ( CONTEXT_RESPONSE_VOTE.isEqualTo('up')),
                menu: {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    order: 1,
                    when: CONTEXT_RESPONSE
                }
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const chatService = accessor.get(IChatService);
            chatService.notifyUserAction({
                providerId: item.providerId,
                agentId: item.agent?.id,
                sessionId: item.sessionId,
                requestId: item.requestId,
                result: item.result,
                action: {
                    kind: 'vote',
                    direction: InteractiveSessionVoteDirection.Up,
                }
            });
            item.setVote(InteractiveSessionVoteDirection.Up);
        }
    });
    registerAction2(class MarkUnhelpfulAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.markUnhelpful',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatTitleActions',
                    'interactive.unhelpful.label',
                    "Unhelpful"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.thumbsdown,
                toggled: ( CONTEXT_RESPONSE_VOTE.isEqualTo('down')),
                menu: {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    order: 2,
                    when: CONTEXT_RESPONSE
                }
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const chatService = accessor.get(IChatService);
            chatService.notifyUserAction({
                providerId: item.providerId,
                agentId: item.agent?.id,
                sessionId: item.sessionId,
                requestId: item.requestId,
                result: item.result,
                action: {
                    kind: 'vote',
                    direction: InteractiveSessionVoteDirection.Down,
                }
            });
            item.setVote(InteractiveSessionVoteDirection.Down);
        }
    });
    registerAction2(class ReportIssueForBugAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.reportIssueForBug',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatTitleActions',
                    'interactive.reportIssueForBug.label',
                    "Report Issue"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.report,
                menu: {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    order: 3,
                    when: ( ContextKeyExpr.and(CONTEXT_CHAT_RESPONSE_SUPPORT_ISSUE_REPORTING, CONTEXT_RESPONSE))
                }
            });
        }
        run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const chatService = accessor.get(IChatService);
            chatService.notifyUserAction({
                providerId: item.providerId,
                agentId: item.agent?.id,
                sessionId: item.sessionId,
                requestId: item.requestId,
                result: item.result,
                action: {
                    kind: 'bug'
                }
            });
        }
    });
    registerAction2(class InsertToNotebookAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.insertIntoNotebook',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatTitleActions',
                    'interactive.insertIntoNotebook.label',
                    "Insert into Notebook"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.insert,
                menu: {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    isHiddenByDefault: true,
                    when: ( ContextKeyExpr.and(NOTEBOOK_IS_ACTIVE_EDITOR, CONTEXT_RESPONSE, ( CONTEXT_RESPONSE_FILTERED.negate())))
                }
            });
        }
        async run(accessor, ...args) {
            const item = args[0];
            if (!isResponseVM(item)) {
                return;
            }
            const editorService = accessor.get(IEditorService);
            if (editorService.activeEditorPane?.getId() === NOTEBOOK_EDITOR_ID) {
                const notebookEditor = editorService.activeEditorPane.getControl();
                if (!notebookEditor.hasModel()) {
                    return;
                }
                if (notebookEditor.isReadOnly) {
                    return;
                }
                const value = item.response.asString();
                const splitContents = splitMarkdownAndCodeBlocks(value);
                const focusRange = notebookEditor.getFocus();
                const index = Math.max(focusRange.end, 0);
                const bulkEditService = accessor.get(IBulkEditService);
                await bulkEditService.apply([
                    ( new ResourceNotebookCellEdit(notebookEditor.textModel.uri, {
                        editType: 1 ,
                        index: index,
                        count: 0,
                        cells: ( splitContents.map(content => {
                            const kind = content.type === 'markdown' ? CellKind.Markup : CellKind.Code;
                            const language = content.type === 'markdown' ? 'markdown' : content.language;
                            const mime = content.type === 'markdown' ? 'text/markdown' : `text/x-${content.language}`;
                            return {
                                cellKind: kind,
                                language,
                                mime,
                                source: content.content,
                                outputs: [],
                                metadata: {}
                            };
                        }))
                    }))
                ], { quotableLabel: 'Insert into Notebook' });
            }
        }
    });
    registerAction2(class RemoveAction extends Action2 {
        constructor() {
            super({
                id: 'workbench.action.chat.remove',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/chat/browser/actions/chatTitleActions',
                    'chat.remove.label',
                    "Remove Request and Response"
                )),
                f1: false,
                category: CHAT_CATEGORY,
                icon: Codicon.x,
                keybinding: {
                    primary: 20 ,
                    mac: {
                        primary: 2048  | 1 ,
                    },
                    when: ( ContextKeyExpr.and(CONTEXT_IN_CHAT_SESSION, ( CONTEXT_IN_CHAT_INPUT.negate()))),
                    weight: 200 ,
                },
                menu: {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    order: 2,
                    when: CONTEXT_REQUEST
                }
            });
        }
        run(accessor, ...args) {
            let item = args[0];
            if (!isRequestVM(item)) {
                const chatWidgetService = accessor.get(IChatWidgetService);
                const widget = chatWidgetService.lastFocusedWidget;
                item = widget?.getFocus();
            }
            const requestId = isRequestVM(item) ? item.id :
                isResponseVM(item) ? item.requestId : undefined;
            if (requestId) {
                const chatService = accessor.get(IChatService);
                chatService.removeRequest(item.sessionId, requestId);
            }
        }
    });
}
function splitMarkdownAndCodeBlocks(markdown) {
    const lexer = new marked_1.Lexer();
    const tokens = lexer.lex(markdown);
    const splitContent = [];
    let markdownPart = '';
    tokens.forEach((token) => {
        if (token.type === 'code') {
            if (markdownPart.trim()) {
                splitContent.push({ type: 'markdown', content: markdownPart });
                markdownPart = '';
            }
            splitContent.push({
                type: 'code',
                language: token.lang || '',
                content: token.text,
            });
        }
        else {
            markdownPart += token.raw;
        }
    });
    if (markdownPart.trim()) {
        splitContent.push({ type: 'markdown', content: markdownPart });
    }
    return splitContent;
}
export { registerChatTitleActions };
