import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { isMarkdownString, MarkdownString } from '../../../../base/common/htmlContent.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { Schemas } from '../../../../base/common/network.js';
import { isMacintosh } from '../../../../base/common/platform.js';
import { ICodeEditorService } from '../../../../editor/browser/services/codeEditorService.js';
import { localizeWithPath } from '../../../../nls.js';
import { ICommandService } from '../../../../platform/commands/common/commands.js';
import { Extensions } from '../../../../platform/configuration/common/configurationRegistry.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { EditorPaneDescriptor } from '../../../browser/editor.js';
import { registerWorkbenchContribution2, Extensions as Extensions$1 } from '../../../common/contributions.js';
import { EditorExtensions } from '../../../common/editor.js';
import { alertFocusChange } from '../../accessibility/browser/accessibilityContributions.js';
import { IAccessibleViewService } from '../../accessibility/browser/accessibleView.js';
import { AccessibleViewAction } from '../../accessibility/browser/accessibleViewActions.js';
import { registerChatActions } from './actions/chatActions.js';
import { registerNewChatActions, ACTION_ID_NEW_CHAT } from './actions/chatClearActions.js';
import { registerChatCodeBlockActions } from './actions/chatCodeblockActions.js';
import { registerChatCopyActions } from './actions/chatCopyActions.js';
import { registerChatExecuteActions, SubmitAction } from './actions/chatExecuteActions.js';
import { registerChatFileTreeActions } from './actions/chatFileTreeActions.js';
import { registerChatExportActions } from './actions/chatImportExport.js';
import { registerMoveActions } from './actions/chatMoveActions.js';
import { registerQuickChatActions } from './actions/chatQuickInputActions.js';
import { registerChatTitleActions } from './actions/chatTitleActions.js';
import { IChatWidgetService } from './chat.js';
import './chatAccessibilityService.js';
import './chatContributionServiceImpl.js';
import { ChatEditor } from './chatEditor.js';
import { ChatEditorInput, ChatEditorInputSerializer } from './chatEditorInput.js';
import './chatQuick.js';
import './chatVariables.js';
import './chatWidget.js';
import './contrib/chatHistoryVariables.js';
import './contrib/chatInputEditorContrib.js';
import { ChatAgentLocation, IChatAgentService } from '../common/chatAgents.js';
import { CONTEXT_IN_CHAT_SESSION } from '../common/chatContextKeys.js';
import { ChatWelcomeMessageModel } from '../common/chatModel.js';
import { chatSubcommandLeader, chatVariableLeader, chatAgentLeader } from '../common/chatParserTypes.js';
import '../common/chatService.js';
import '../common/chatServiceImpl.js';
import { IChatSlashCommandService } from '../common/chatSlashCommands.js';
import { IChatVariablesService } from '../common/chatVariables.js';
import { isResponseVM } from '../common/chatViewModel.js';
import '../common/chatWidgetHistoryService.js';
import '../../../../base/common/event.js';
import '../common/voiceChat.js';
import { RegisteredEditorPriority, IEditorResolverService } from '../../../services/editor/common/editorResolverService.js';
import '../common/chatColors.js';
const configurationRegistry = ( Registry.as(Extensions.Configuration));
configurationRegistry.registerConfiguration({
    id: 'chatSidebar',
    title: ( localizeWithPath(
        'vs/workbench/contrib/chat/browser/chat.contribution',
        'interactiveSessionConfigurationTitle',
        "Chat"
    )),
    type: 'object',
    properties: {
        'chat.editor.fontSize': {
            type: 'number',
            description: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'interactiveSession.editor.fontSize',
                "Controls the font size in pixels in chat codeblocks."
            )),
            default: isMacintosh ? 12 : 14,
        },
        'chat.editor.fontFamily': {
            type: 'string',
            description: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'interactiveSession.editor.fontFamily',
                "Controls the font family in chat codeblocks."
            )),
            default: 'default'
        },
        'chat.editor.fontWeight': {
            type: 'string',
            description: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'interactiveSession.editor.fontWeight',
                "Controls the font weight in chat codeblocks."
            )),
            default: 'default'
        },
        'chat.editor.wordWrap': {
            type: 'string',
            description: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'interactiveSession.editor.wordWrap',
                "Controls whether lines should wrap in chat codeblocks."
            )),
            default: 'off',
            enum: ['on', 'off']
        },
        'chat.editor.lineHeight': {
            type: 'number',
            description: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'interactiveSession.editor.lineHeight',
                "Controls the line height in pixels in chat codeblocks. Use 0 to compute the line height from the font size."
            )),
            default: 0
        },
        'chat.experimental.implicitContext': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'chat.experimental.implicitContext',
                "Controls whether a checkbox is shown to allow the user to determine which implicit context is included with a chat participant's prompt."
            )),
            default: false
        },
    }
});
( Registry.as(EditorExtensions.EditorPane)).registerEditorPane(EditorPaneDescriptor.create(ChatEditor, ChatEditorInput.EditorID, ( localizeWithPath('vs/workbench/contrib/chat/browser/chat.contribution', 'chat', "Chat"))), [
    ( new SyncDescriptor(ChatEditorInput))
]);
let ChatResolverContribution = class ChatResolverContribution extends Disposable {
    static { this.ID = 'workbench.contrib.chatResolver'; }
    constructor(editorResolverService, instantiationService) {
        super();
        this._register(editorResolverService.registerEditor(`${Schemas.vscodeChatSesssion}:**/**`, {
            id: ChatEditorInput.EditorID,
            label: ( localizeWithPath('vs/workbench/contrib/chat/browser/chat.contribution', 'chat', "Chat")),
            priority: RegisteredEditorPriority.builtin
        }, {
            singlePerResource: true,
            canSupportResource: resource => resource.scheme === Schemas.vscodeChatSesssion
        }, {
            createEditorInput: ({ resource, options }) => {
                return { editor: instantiationService.createInstance(ChatEditorInput, resource, options), options };
            }
        }));
    }
};
ChatResolverContribution = ( __decorate([
    ( __param(0, IEditorResolverService)),
    ( __param(1, IInstantiationService))
], ChatResolverContribution));
class ChatAccessibleViewContribution extends Disposable {
    constructor() {
        super();
        this._register(AccessibleViewAction.addImplementation(100, 'panelChat', accessor => {
            const accessibleViewService = accessor.get(IAccessibleViewService);
            const widgetService = accessor.get(IChatWidgetService);
            const codeEditorService = accessor.get(ICodeEditorService);
            return renderAccessibleView(accessibleViewService, widgetService, codeEditorService, true);
            function renderAccessibleView(accessibleViewService, widgetService, codeEditorService, initialRender) {
                const widget = widgetService.lastFocusedWidget;
                if (!widget) {
                    return false;
                }
                const chatInputFocused = initialRender && !!codeEditorService.getFocusedCodeEditor();
                if (initialRender && chatInputFocused) {
                    widget.focusLastMessage();
                }
                if (!widget) {
                    return false;
                }
                const verifiedWidget = widget;
                const focusedItem = verifiedWidget.getFocus();
                if (!focusedItem) {
                    return false;
                }
                widget.focus(focusedItem);
                const isWelcome = focusedItem instanceof ChatWelcomeMessageModel;
                let responseContent = isResponseVM(focusedItem) ? focusedItem.response.asString() : undefined;
                if (isWelcome) {
                    const welcomeReplyContents = [];
                    for (const content of focusedItem.content) {
                        if (Array.isArray(content)) {
                            welcomeReplyContents.push(...( content.map(m => m.message)));
                        }
                        else {
                            welcomeReplyContents.push(content.value);
                        }
                    }
                    responseContent = welcomeReplyContents.join('\n');
                }
                if (!responseContent && 'errorDetails' in focusedItem && focusedItem.errorDetails) {
                    responseContent = focusedItem.errorDetails.message;
                }
                if (!responseContent) {
                    return false;
                }
                const responses = verifiedWidget.viewModel?.getItems().filter(i => isResponseVM(i));
                const length = responses?.length;
                const responseIndex = responses?.findIndex(i => i === focusedItem);
                accessibleViewService.show({
                    id: "panelChat" ,
                    verbositySettingKey: "accessibility.verbosity.panelChat" ,
                    provideContent() { return responseContent; },
                    onClose() {
                        verifiedWidget.reveal(focusedItem);
                        if (chatInputFocused) {
                            verifiedWidget.focusInput();
                        }
                        else {
                            verifiedWidget.focus(focusedItem);
                        }
                    },
                    next() {
                        verifiedWidget.moveFocus(focusedItem, 'next');
                        alertFocusChange(responseIndex, length, 'next');
                        renderAccessibleView(accessibleViewService, widgetService, codeEditorService);
                    },
                    previous() {
                        verifiedWidget.moveFocus(focusedItem, 'previous');
                        alertFocusChange(responseIndex, length, 'previous');
                        renderAccessibleView(accessibleViewService, widgetService, codeEditorService);
                    },
                    options: { type: "view"  }
                });
                return true;
            }
        }, CONTEXT_IN_CHAT_SESSION));
    }
}
let ChatSlashStaticSlashCommandsContribution = class ChatSlashStaticSlashCommandsContribution extends Disposable {
    constructor(slashCommandService, commandService, chatAgentService, chatVariablesService) {
        super();
        this._store.add(slashCommandService.registerSlashCommand({
            command: 'clear',
            detail: ( localizeWithPath(
                'vs/workbench/contrib/chat/browser/chat.contribution',
                'clear',
                "Start a new chat"
            )),
            sortText: 'z2_clear',
            executeImmediately: true
        }, async () => {
            commandService.executeCommand(ACTION_ID_NEW_CHAT);
        }));
        this._store.add(slashCommandService.registerSlashCommand({
            command: 'help',
            detail: '',
            sortText: 'z1_help',
            executeImmediately: true
        }, async (prompt, progress) => {
            const defaultAgent = chatAgentService.getDefaultAgent(ChatAgentLocation.Panel);
            const agents = chatAgentService.getAgents();
            if (defaultAgent?.metadata.helpTextPrefix) {
                if (isMarkdownString(defaultAgent.metadata.helpTextPrefix)) {
                    progress.report({ content: defaultAgent.metadata.helpTextPrefix, kind: 'markdownContent' });
                }
                else {
                    progress.report({ content: defaultAgent.metadata.helpTextPrefix, kind: 'content' });
                }
                progress.report({ content: '\n\n', kind: 'content' });
            }
            const agentText = (await Promise.all(( agents
                .filter(a => a.id !== defaultAgent?.id)
                .map(async (a) => {
                const agentWithLeader = `${chatAgentLeader}${a.name}`;
                const actionArg = { inputValue: `${agentWithLeader} ${a.metadata.sampleRequest}` };
                const urlSafeArg = encodeURIComponent(JSON.stringify(actionArg));
                const agentLine = `* [\`${agentWithLeader}\`](command:${SubmitAction.ID}?${urlSafeArg}) - ${a.description}`;
                const commandText = ( a.slashCommands.map(c => {
                    const actionArg = { inputValue: `${agentWithLeader} ${chatSubcommandLeader}${c.name} ${c.sampleRequest ?? ''}` };
                    const urlSafeArg = encodeURIComponent(JSON.stringify(actionArg));
                    return `\t* [\`${chatSubcommandLeader}${c.name}\`](command:${SubmitAction.ID}?${urlSafeArg}) - ${c.description}`;
                })).join('\n');
                return (agentLine + '\n' + commandText).trim();
            })))).join('\n');
            progress.report({ content: ( new MarkdownString(agentText, { isTrusted: { enabledCommands: [SubmitAction.ID] } })), kind: 'markdownContent' });
            if (defaultAgent?.metadata.helpTextVariablesPrefix) {
                progress.report({ content: '\n\n', kind: 'content' });
                if (isMarkdownString(defaultAgent.metadata.helpTextVariablesPrefix)) {
                    progress.report({ content: defaultAgent.metadata.helpTextVariablesPrefix, kind: 'markdownContent' });
                }
                else {
                    progress.report({ content: defaultAgent.metadata.helpTextVariablesPrefix, kind: 'content' });
                }
                const variableText = ( Array.from(chatVariablesService.getVariables())
                    .map(v => `* \`${chatVariableLeader}${v.name}\` - ${v.description}`))
                    .join('\n');
                progress.report({ content: '\n' + variableText, kind: 'content' });
            }
            if (defaultAgent?.metadata.helpTextPostfix) {
                progress.report({ content: '\n\n', kind: 'content' });
                if (isMarkdownString(defaultAgent.metadata.helpTextPostfix)) {
                    progress.report({ content: defaultAgent.metadata.helpTextPostfix, kind: 'markdownContent' });
                }
                else {
                    progress.report({ content: defaultAgent.metadata.helpTextPostfix, kind: 'content' });
                }
            }
        }));
    }
};
ChatSlashStaticSlashCommandsContribution = ( __decorate([
    ( __param(0, IChatSlashCommandService)),
    ( __param(1, ICommandService)),
    ( __param(2, IChatAgentService)),
    ( __param(3, IChatVariablesService))
], ChatSlashStaticSlashCommandsContribution));
const workbenchContributionsRegistry = ( Registry.as(Extensions$1.Workbench));
registerWorkbenchContribution2(ChatResolverContribution.ID, ChatResolverContribution, 1 );
workbenchContributionsRegistry.registerWorkbenchContribution(ChatAccessibleViewContribution, 4 );
workbenchContributionsRegistry.registerWorkbenchContribution(ChatSlashStaticSlashCommandsContribution, 4 );
( Registry.as(EditorExtensions.EditorFactory)).registerEditorSerializer(ChatEditorInput.TypeID, ChatEditorInputSerializer);
registerChatActions();
registerChatCopyActions();
registerChatCodeBlockActions();
registerChatFileTreeActions();
registerChatTitleActions();
registerChatExecuteActions();
registerQuickChatActions();
registerChatExportActions();
registerMoveActions();
registerNewChatActions();
