import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { isNonEmptyArray } from '../../../../base/common/arrays.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { DisposableStore, DisposableMap } from '../../../../base/common/lifecycle.js';
import { localizeWithPath, localize2WithPath } from '../../../../nls.js';
import { registerAction2 } from '../../../../platform/actions/common/actions.js';
import { ContextKeyExpr, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { ILogService } from '../../../../platform/log/common/log.js';
import { IProductService } from '../../../../platform/product/common/productService.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { ViewPaneContainer } from '../../../browser/parts/views/viewPaneContainer.js';
import { registerWorkbenchContribution2 } from '../../../common/contributions.js';
import { Extensions } from '../../../common/views.js';
import { getHistoryAction, getOpenChatEditorAction } from './actions/chatActions.js';
import { getNewChatAction } from './actions/chatClearActions.js';
import { getMoveToEditorAction, getMoveToNewWindowAction } from './actions/chatMoveActions.js';
import { getQuickChatActionForProvider } from './actions/chatQuickInputActions.js';
import { ChatViewPane, CHAT_SIDEBAR_PANEL_ID } from './chatViewPane.js';
import { ChatAgentLocation, IChatAgentService } from '../common/chatAgents.js';
import { IChatContributionService } from '../common/chatContributionService.js';
import { isProposedApiEnabled } from '../../../services/extensions/common/extensions.js';
import { ExtensionsRegistry } from '../../../services/extensions/common/extensionsRegistry.js';
const chatExtensionPoint = ExtensionsRegistry.registerExtensionPoint({
    extensionPoint: 'interactiveSession',
    jsonSchema: {
        description: ( localizeWithPath(
            'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
            'vscode.extension.contributes.interactiveSession',
            'Contributes an Interactive Session provider'
        )),
        type: 'array',
        items: {
            additionalProperties: false,
            type: 'object',
            defaultSnippets: [{ body: { id: '', program: '', runtime: '' } }],
            required: ['id', 'label'],
            properties: {
                id: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'vscode.extension.contributes.interactiveSession.id',
                        "Unique identifier for this Interactive Session provider."
                    )),
                    type: 'string'
                },
                label: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'vscode.extension.contributes.interactiveSession.label',
                        "Display name for this Interactive Session provider."
                    )),
                    type: 'string'
                },
                icon: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'vscode.extension.contributes.interactiveSession.icon',
                        "An icon for this Interactive Session provider."
                    )),
                    type: 'string'
                },
                when: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'vscode.extension.contributes.interactiveSession.when',
                        "A condition which must be true to enable this Interactive Session provider."
                    )),
                    type: 'string'
                },
            }
        }
    },
    activationEventsGenerator: (contributions, result) => {
        for (const contrib of contributions) {
            result.push(`onInteractiveSession:${contrib.id}`);
        }
    },
});
const chatParticipantExtensionPoint = ExtensionsRegistry.registerExtensionPoint({
    extensionPoint: 'chatParticipants',
    jsonSchema: {
        description: ( localizeWithPath(
            'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
            'vscode.extension.contributes.chatParticipant',
            'Contributes a chat participant'
        )),
        type: 'array',
        items: {
            additionalProperties: false,
            type: 'object',
            defaultSnippets: [{ body: { name: '', description: '' } }],
            required: ['name', 'id'],
            properties: {
                id: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatParticipantId',
                        "A unique id for this chat participant."
                    )),
                    type: 'string'
                },
                name: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatParticipantName',
                        "User-facing display name for this chat participant. The user will use '@' with this name to invoke the participant."
                    )),
                    type: 'string'
                },
                description: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatParticipantDescription',
                        "A description of this chat participant, shown in the UI."
                    )),
                    type: 'string'
                },
                isDefault: {
                    markdownDescription: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatParticipantIsDefaultDescription',
                        "**Only** allowed for extensions that have the `defaultChatParticipant` proposal."
                    )),
                    type: 'boolean',
                },
                isSticky: {
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatCommandSticky',
                        "Whether invoking the command puts the chat into a persistent mode, where the command is automatically added to the chat input for the next message."
                    )),
                    type: 'boolean'
                },
                defaultImplicitVariables: {
                    markdownDescription: '**Only** allowed for extensions that have the `chatParticipantAdditions` proposal. The names of the variables that are invoked by default',
                    type: 'array',
                    items: {
                        type: 'string'
                    }
                },
                commands: {
                    markdownDescription: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatCommandsDescription',
                        "Commands available for this chat participant, which the user can invoke with a `/`."
                    )),
                    type: 'array',
                    items: {
                        additionalProperties: false,
                        type: 'object',
                        defaultSnippets: [{ body: { name: '', description: '' } }],
                        required: ['name'],
                        properties: {
                            name: {
                                description: ( localizeWithPath(
                                    'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                                    'chatCommand',
                                    "A short name by which this command is referred to in the UI, e.g. `fix` or * `explain` for commands that fix an issue or explain code. The name should be unique among the commands provided by this participant."
                                )),
                                type: 'string'
                            },
                            description: {
                                description: ( localizeWithPath(
                                    'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                                    'chatCommandDescription',
                                    "A description of this command."
                                )),
                                type: 'string'
                            },
                            when: {
                                description: ( localizeWithPath(
                                    'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                                    'chatCommandWhen',
                                    "A condition which must be true to enable this command."
                                )),
                                type: 'string'
                            },
                            sampleRequest: {
                                description: ( localizeWithPath(
                                    'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                                    'chatCommandSampleRequest',
                                    "When the user clicks this command in `/help`, this text will be submitted to this participant."
                                )),
                                type: 'string'
                            },
                            isSticky: {
                                description: ( localizeWithPath(
                                    'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                                    'chatCommandSticky',
                                    "Whether invoking the command puts the chat into a persistent mode, where the command is automatically added to the chat input for the next message."
                                )),
                                type: 'boolean'
                            },
                            defaultImplicitVariables: {
                                markdownDescription: ( localizeWithPath(
                                    'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                                    'defaultImplicitVariables',
                                    "**Only** allowed for extensions that have the `chatParticipantAdditions` proposal. The names of the variables that are invoked by default"
                                )),
                                type: 'array',
                                items: {
                                    type: 'string'
                                }
                            },
                        }
                    }
                },
                locations: {
                    markdownDescription: ( localizeWithPath(
                        'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
                        'chatLocationsDescription',
                        "Locations in which this chat participant is available."
                    )),
                    type: 'array',
                    default: ['panel'],
                    items: {
                        type: 'string',
                        enum: ['panel', 'terminal', 'notebook']
                    }
                }
            }
        }
    },
    activationEventsGenerator: (contributions, result) => {
        for (const contrib of contributions) {
            result.push(`onChatParticipant:${contrib.id}`);
        }
    },
});
let ChatExtensionPointHandler = class ChatExtensionPointHandler {
    static { this.ID = 'workbench.contrib.chatExtensionPointHandler'; }
    constructor(_chatContributionService, _chatAgentService, productService, contextService, logService) {
        this._chatContributionService = _chatContributionService;
        this._chatAgentService = _chatAgentService;
        this.productService = productService;
        this.contextService = contextService;
        this.logService = logService;
        this.disposables = ( new DisposableStore());
        this._registrationDisposables = ( new Map());
        this._participantRegistrationDisposables = ( new DisposableMap());
        this._viewContainer = this.registerViewContainer();
        this.registerListeners();
        this.handleAndRegisterChatExtensions();
    }
    registerListeners() {
        this.contextService.onDidChangeContext(e => {
            if (!this.productService.chatWelcomeView) {
                return;
            }
            const showWelcomeViewConfigKey = 'workbench.chat.experimental.showWelcomeView';
            const keys = ( new Set([showWelcomeViewConfigKey]));
            if (e.affectsSome(keys)) {
                const contextKeyExpr = ( ContextKeyExpr.equals(showWelcomeViewConfigKey, true));
                const viewsRegistry = ( Registry.as(Extensions.ViewsRegistry));
                if (this.contextService.contextMatchesRules(contextKeyExpr)) {
                    const viewId = this._chatContributionService.getViewIdForProvider(this.productService.chatWelcomeView.welcomeViewId);
                    this._welcomeViewDescriptor = {
                        id: viewId,
                        name: { original: this.productService.chatWelcomeView.welcomeViewTitle, value: this.productService.chatWelcomeView.welcomeViewTitle },
                        containerIcon: this._viewContainer.icon,
                        ctorDescriptor: ( new SyncDescriptor(
                            ChatViewPane,
                            [{ providerId: this.productService.chatWelcomeView.welcomeViewId }]
                        )),
                        canToggleVisibility: false,
                        canMoveView: true,
                        order: 100
                    };
                    viewsRegistry.registerViews([this._welcomeViewDescriptor], this._viewContainer);
                    viewsRegistry.registerViewWelcomeContent(viewId, {
                        content: this.productService.chatWelcomeView.welcomeViewContent,
                    });
                }
                else if (this._welcomeViewDescriptor) {
                    viewsRegistry.deregisterViews([this._welcomeViewDescriptor], this._viewContainer);
                }
            }
        }, null, this.disposables);
    }
    handleAndRegisterChatExtensions() {
        chatExtensionPoint.setHandler((extensions, delta) => {
            for (const extension of delta.added) {
                const extensionDisposable = ( new DisposableStore());
                for (const providerDescriptor of extension.value) {
                    this.registerChatProvider(providerDescriptor);
                    this._chatContributionService.registerChatProvider(providerDescriptor);
                }
                this._registrationDisposables.set(extension.description.identifier.value, extensionDisposable);
            }
            for (const extension of delta.removed) {
                const registration = this._registrationDisposables.get(extension.description.identifier.value);
                if (registration) {
                    registration.dispose();
                    this._registrationDisposables.delete(extension.description.identifier.value);
                }
                for (const providerDescriptor of extension.value) {
                    this._chatContributionService.deregisterChatProvider(providerDescriptor.id);
                }
            }
        });
        chatParticipantExtensionPoint.setHandler((extensions, delta) => {
            for (const extension of delta.added) {
                for (const providerDescriptor of extension.value) {
                    if (providerDescriptor.isDefault && !isProposedApiEnabled(extension.description, 'defaultChatParticipant')) {
                        this.logService.error(`Extension '${extension.description.identifier.value}' CANNOT use API proposal: defaultChatParticipant.`);
                        continue;
                    }
                    if (providerDescriptor.defaultImplicitVariables && !isProposedApiEnabled(extension.description, 'chatParticipantAdditions')) {
                        this.logService.error(`Extension '${extension.description.identifier.value}' CANNOT use API proposal: chatParticipantAdditions.`);
                        continue;
                    }
                    if (!providerDescriptor.id || !providerDescriptor.name) {
                        this.logService.error(`Extension '${extension.description.identifier.value}' CANNOT register participant without both id and name.`);
                        continue;
                    }
                    this._participantRegistrationDisposables.set(getParticipantKey(extension.description.identifier, providerDescriptor.name), this._chatAgentService.registerAgent(providerDescriptor.id, {
                        extensionId: extension.description.identifier,
                        id: providerDescriptor.id,
                        description: providerDescriptor.description,
                        metadata: {
                            isSticky: providerDescriptor.isSticky,
                        },
                        name: providerDescriptor.name,
                        isDefault: providerDescriptor.isDefault,
                        defaultImplicitVariables: providerDescriptor.defaultImplicitVariables,
                        locations: isNonEmptyArray(providerDescriptor.locations) ?
                            ( providerDescriptor.locations.map(ChatAgentLocation.fromRaw)) :
                            [ChatAgentLocation.Panel],
                        slashCommands: providerDescriptor.commands ?? []
                    }));
                }
            }
            for (const extension of delta.removed) {
                for (const providerDescriptor of extension.value) {
                    this._participantRegistrationDisposables.deleteAndDispose(getParticipantKey(extension.description.identifier, providerDescriptor.name));
                }
            }
        });
    }
    registerViewContainer() {
        const title = ( localize2WithPath(
            'vs/workbench/contrib/chat/browser/chatContributionServiceImpl',
            'chat.viewContainer.label',
            "Chat"
        ));
        const icon = Codicon.commentDiscussion;
        const viewContainerId = CHAT_SIDEBAR_PANEL_ID;
        const viewContainer = ( Registry.as(Extensions.ViewContainersRegistry)).registerViewContainer({
            id: viewContainerId,
            title,
            icon,
            ctorDescriptor: ( new SyncDescriptor(
                ViewPaneContainer,
                [viewContainerId, { mergeViewWithContainerWhenSingleView: true }]
            )),
            storageId: viewContainerId,
            hideIfEmpty: true,
            order: 100,
        }, 0 );
        return viewContainer;
    }
    registerChatProvider(providerDescriptor) {
        const viewId = this._chatContributionService.getViewIdForProvider(providerDescriptor.id);
        const viewDescriptor = [{
                id: viewId,
                containerIcon: this._viewContainer.icon,
                containerTitle: this._viewContainer.title.value,
                singleViewPaneContainerTitle: this._viewContainer.title.value,
                name: { value: providerDescriptor.label, original: providerDescriptor.label },
                canToggleVisibility: false,
                canMoveView: true,
                ctorDescriptor: ( new SyncDescriptor(ChatViewPane, [{ providerId: providerDescriptor.id }])),
                when: ContextKeyExpr.deserialize(providerDescriptor.when)
            }];
        ( Registry.as(Extensions.ViewsRegistry)).registerViews(viewDescriptor, this._viewContainer);
        const disposables = ( new DisposableStore());
        disposables.add(registerAction2(getHistoryAction(viewId, providerDescriptor.id)));
        disposables.add(registerAction2(getNewChatAction(viewId, providerDescriptor.id)));
        disposables.add(registerAction2(getMoveToEditorAction(viewId, providerDescriptor.id)));
        disposables.add(registerAction2(getMoveToNewWindowAction(viewId, providerDescriptor.id)));
        disposables.add(registerAction2(getOpenChatEditorAction(providerDescriptor.id, providerDescriptor.label, providerDescriptor.when)));
        disposables.add(registerAction2(getQuickChatActionForProvider(providerDescriptor.id, providerDescriptor.label)));
        return {
            dispose: () => {
                ( Registry.as(Extensions.ViewsRegistry)).deregisterViews(viewDescriptor, this._viewContainer);
                ( Registry.as(Extensions.ViewContainersRegistry)).deregisterViewContainer(this._viewContainer);
                disposables.dispose();
            }
        };
    }
};
ChatExtensionPointHandler = ( __decorate([
    ( __param(0, IChatContributionService)),
    ( __param(1, IChatAgentService)),
    ( __param(2, IProductService)),
    ( __param(3, IContextKeyService)),
    ( __param(4, ILogService))
], ChatExtensionPointHandler));
registerWorkbenchContribution2(ChatExtensionPointHandler.ID, ChatExtensionPointHandler, 1 );
function getParticipantKey(extensionId, participantName) {
    return `${extensionId.value}_${participantName}`;
}
class ChatContributionService {
    constructor() {
        this._registeredProviders = ( new Map());
    }
    getViewIdForProvider(providerId) {
        return ChatViewPane.ID + '.' + providerId;
    }
    registerChatProvider(provider) {
        this._registeredProviders.set(provider.id, provider);
    }
    deregisterChatProvider(providerId) {
        this._registeredProviders.delete(providerId);
    }
    get registeredProviders() {
        return Array.from(( this._registeredProviders.values()));
    }
}
export { ChatContributionService, ChatExtensionPointHandler };
