import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { CancellationToken } from '../../../../base/common/cancellation.js';
import { DisposableStore } from '../../../../base/common/lifecycle.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { IContextMenuService } from '../../../../platform/contextview/browser/contextView.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { ServiceCollection } from '../../../../platform/instantiation/common/serviceCollection.js';
import { IKeybindingService } from '../../../../platform/keybinding/common/keybinding.js';
import { ILogService } from '../../../../platform/log/common/log.js';
import { IOpenerService } from '../../../../platform/opener/common/opener.js';
import { IStorageService } from '../../../../platform/storage/common/storage.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.js';
import '../../../../platform/theme/common/colorUtils.js';
import '../../../../platform/theme/common/colors/baseColors.js';
import '../../../../platform/theme/common/colors/chartsColors.js';
import { editorBackground } from '../../../../platform/theme/common/colors/editorColors.js';
import '../../../../platform/theme/common/colors/inputColors.js';
import '../../../../platform/theme/common/colors/listColors.js';
import '../../../../platform/theme/common/colors/menuColors.js';
import '../../../../platform/theme/common/colors/minimapColors.js';
import '../../../../platform/theme/common/colors/miscColors.js';
import '../../../../platform/theme/common/colors/quickpickColors.js';
import '../../../../platform/theme/common/colors/searchColors.js';
import { IThemeService } from '../../../../platform/theme/common/themeService.js';
import { ViewPane } from '../../../browser/parts/views/viewPane.js';
import { Memento } from '../../../common/memento.js';
import { SIDE_BAR_FOREGROUND } from '../../../common/theme.js';
import { IViewDescriptorService } from '../../../common/views.js';
import { ChatWidget } from './chatWidget.js';
import { ChatAgentLocation } from '../common/chatAgents.js';
import { IChatService } from '../common/chatService.js';
const CHAT_SIDEBAR_PANEL_ID = 'workbench.panel.chatSidebar';
let ChatViewPane = class ChatViewPane extends ViewPane {
    static { this.ID = 'workbench.panel.chat.view'; }
    get widget() { return this._widget; }
    constructor(chatViewOptions, options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService, storageService, chatService, logService) {
        super(options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService);
        this.chatViewOptions = chatViewOptions;
        this.storageService = storageService;
        this.chatService = chatService;
        this.logService = logService;
        this.modelDisposables = this._register(( new DisposableStore()));
        this.didProviderRegistrationFail = false;
        this.didUnregisterProvider = false;
        this.memento = ( new Memento(
            'interactive-session-view-' + this.chatViewOptions.providerId,
            this.storageService
        ));
        this.viewState = this.memento.getMemento(1 , 1 );
        this._register(this.chatService.onDidRegisterProvider(({ providerId }) => {
            if (providerId === this.chatViewOptions.providerId && !this._widget?.viewModel) {
                const sessionId = this.getSessionId();
                const model = sessionId ? this.chatService.getOrRestoreSession(sessionId) : undefined;
                try {
                    this._widget.setVisible(false);
                    this.updateModel(model);
                    this.didProviderRegistrationFail = false;
                    this.didUnregisterProvider = false;
                    this._onDidChangeViewWelcomeState.fire();
                }
                finally {
                    this.widget.setVisible(true);
                }
            }
        }));
        this._register(this.chatService.onDidUnregisterProvider(({ providerId }) => {
            if (providerId === this.chatViewOptions.providerId) {
                this.didUnregisterProvider = true;
                this._onDidChangeViewWelcomeState.fire();
            }
        }));
    }
    updateModel(model, viewState) {
        this.modelDisposables.clear();
        model = model ?? (this.chatService.transferredSessionData?.sessionId
            ? this.chatService.getOrRestoreSession(this.chatService.transferredSessionData.sessionId)
            : this.chatService.startSession(this.chatViewOptions.providerId, CancellationToken.None));
        if (!model) {
            throw new Error('Could not start chat session');
        }
        this._widget.setModel(model, { ...(viewState ?? this.viewState) });
        this.viewState.sessionId = model.sessionId;
    }
    shouldShowWelcome() {
        const noPersistedSessions = !this.chatService.hasSessions(this.chatViewOptions.providerId);
        return this.didUnregisterProvider || !this._widget?.viewModel && (noPersistedSessions || this.didProviderRegistrationFail);
    }
    getSessionId() {
        let sessionId;
        if (this.chatService.transferredSessionData) {
            sessionId = this.chatService.transferredSessionData.sessionId;
            this.viewState.inputValue = this.chatService.transferredSessionData.inputValue;
        }
        else {
            sessionId = this.viewState.sessionId;
        }
        return sessionId;
    }
    renderBody(parent) {
        try {
            super.renderBody(parent);
            const scopedInstantiationService = this.instantiationService.createChild(( new ServiceCollection([IContextKeyService, this.scopedContextKeyService])));
            this._widget = this._register(scopedInstantiationService.createInstance(ChatWidget, ChatAgentLocation.Panel, { viewId: this.id }, { supportsFileReferences: true }, {
                listForeground: SIDE_BAR_FOREGROUND,
                listBackground: this.getBackgroundColor(),
                inputEditorBackground: this.getBackgroundColor(),
                resultEditorBackground: editorBackground
            }));
            this._register(this.onDidChangeBodyVisibility(visible => {
                this._widget.setVisible(visible);
            }));
            this._register(this._widget.onDidClear(() => this.clear()));
            this._widget.render(parent);
            const sessionId = this.getSessionId();
            const disposeListener = sessionId ? this._register(this.chatService.onDidDisposeSession((e) => {
                if (e.reason === 'initializationFailed' && e.providerId === this.chatViewOptions.providerId) {
                    this.didProviderRegistrationFail = true;
                    disposeListener?.dispose();
                    this._onDidChangeViewWelcomeState.fire();
                }
            })) : undefined;
            const model = sessionId ? this.chatService.getOrRestoreSession(sessionId) : undefined;
            this.updateModel(model);
        }
        catch (e) {
            this.logService.error(e);
            throw e;
        }
    }
    acceptInput(query) {
        this._widget.acceptInput(query);
    }
    async clear() {
        if (this.widget.viewModel) {
            this.chatService.clearSession(this.widget.viewModel.sessionId);
        }
        this.updateModel(undefined, { ...this.viewState, inputValue: undefined });
    }
    loadSession(sessionId) {
        if (this.widget.viewModel) {
            this.chatService.clearSession(this.widget.viewModel.sessionId);
        }
        const newModel = this.chatService.getOrRestoreSession(sessionId);
        this.updateModel(newModel);
    }
    focusInput() {
        this._widget.focusInput();
    }
    focus() {
        super.focus();
        this._widget.focusInput();
    }
    layoutBody(height, width) {
        super.layoutBody(height, width);
        this._widget.layout(height, width);
    }
    saveState() {
        if (this._widget) {
            this._widget.saveState();
            const widgetViewState = this._widget.getViewState();
            this.viewState.inputValue = widgetViewState.inputValue;
            this.viewState.inputState = widgetViewState.inputState;
            this.memento.saveMemento();
        }
        super.saveState();
    }
};
ChatViewPane = ( __decorate([
    ( __param(2, IKeybindingService)),
    ( __param(3, IContextMenuService)),
    ( __param(4, IConfigurationService)),
    ( __param(5, IContextKeyService)),
    ( __param(6, IViewDescriptorService)),
    ( __param(7, IInstantiationService)),
    ( __param(8, IOpenerService)),
    ( __param(9, IThemeService)),
    ( __param(10, ITelemetryService)),
    ( __param(11, IStorageService)),
    ( __param(12, IChatService)),
    ( __param(13, ILogService))
], ChatViewPane));
export { CHAT_SIDEBAR_PANEL_ID, ChatViewPane };
