import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../nls.js';
import { basename } from '../../../../base/common/path.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { registerEditorContribution } from '../../../../editor/browser/editorExtensions.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { Severity, INotificationService } from '../../../../platform/notification/common/notification.js';
let LargeFileOptimizationsWarner = class LargeFileOptimizationsWarner extends Disposable {
    static { this.ID = 'editor.contrib.largeFileOptimizationsWarner'; }
    constructor(_editor, _notificationService, _configurationService) {
        super();
        this._editor = _editor;
        this._notificationService = _notificationService;
        this._configurationService = _configurationService;
        this._register(this._editor.onDidChangeModel((e) => this._update()));
        this._update();
    }
    _update() {
        const model = this._editor.getModel();
        if (!model) {
            return;
        }
        if (model.isTooLargeForTokenization()) {
            const message = ( localizeWithPath('vs/workbench/contrib/codeEditor/browser/largeFileOptimizations', {
                key: 'largeFile',
                comment: [
                    'Variable 0 will be a file name.'
                ]
            }, "{0}: tokenization, wrapping, folding, codelens, word highlighting and sticky scroll have been turned off for this large file in order to reduce memory usage and avoid freezing or crashing.", basename(model.uri.path)));
            this._notificationService.prompt(Severity.Info, message, [
                {
                    label: ( localizeWithPath(
                        'vs/workbench/contrib/codeEditor/browser/largeFileOptimizations',
                        'removeOptimizations',
                        "Forcefully Enable Features"
                    )),
                    run: () => {
                        this._configurationService.updateValue(`editor.largeFileOptimizations`, false).then(() => {
                            this._notificationService.info(( localizeWithPath(
                                'vs/workbench/contrib/codeEditor/browser/largeFileOptimizations',
                                'reopenFilePrompt',
                                "Please reopen file in order for this setting to take effect."
                            )));
                        }, (err) => {
                            this._notificationService.error(err);
                        });
                    }
                }
            ], { neverShowAgain: { id: 'editor.contrib.largeFileOptimizationsWarner' } });
        }
    }
};
LargeFileOptimizationsWarner = ( __decorate([
    ( __param(1, INotificationService)),
    ( __param(2, IConfigurationService))
], LargeFileOptimizationsWarner));
registerEditorContribution(LargeFileOptimizationsWarner.ID, LargeFileOptimizationsWarner, 1 );
export { LargeFileOptimizationsWarner };
