import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../nls.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import './commentsEditorContribution.js';
import { ICommentService } from './commentService.js';
import { Extensions } from '../../../../platform/configuration/common/configurationRegistry.js';
import { Disposable, MutableDisposable } from '../../../../base/common/lifecycle.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { Extensions as Extensions$1 } from '../../../common/contributions.js';
import { NumberBadge, IActivityService } from '../../../services/activity/common/activity.js';
import { COMMENTS_VIEW_ID } from './commentsTreeViewer.js';
import { CommentThreadState } from '../../../../editor/common/languages.js';
import { registerAction2, MenuId, Action2 } from '../../../../platform/actions/common/actions.js';
import { CONTEXT_KEY_HAS_COMMENTS, CONTEXT_KEY_SOME_COMMENTS_EXPANDED } from './commentsView.js';
import { ViewAction } from '../../../browser/parts/views/viewPane.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { IUriIdentityService } from '../../../../platform/uriIdentity/common/uriIdentity.js';
import { revealCommentThread } from './commentsController.js';
registerAction2(class Collapse extends ViewAction {
    constructor() {
        super({
            viewId: COMMENTS_VIEW_ID,
            id: 'comments.collapse',
            title: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'collapseAll',
                "Collapse All"
            )),
            f1: false,
            icon: Codicon.collapseAll,
            menu: {
                id: MenuId.ViewTitle,
                group: 'navigation',
                when: ( ContextKeyExpr.and(( ContextKeyExpr.and(( ContextKeyExpr.equals('view', COMMENTS_VIEW_ID)), CONTEXT_KEY_HAS_COMMENTS)), CONTEXT_KEY_SOME_COMMENTS_EXPANDED)),
                order: 100
            }
        });
    }
    runInView(_accessor, view) {
        view.collapseAll();
    }
});
registerAction2(class Expand extends ViewAction {
    constructor() {
        super({
            viewId: COMMENTS_VIEW_ID,
            id: 'comments.expand',
            title: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'expandAll',
                "Expand All"
            )),
            f1: false,
            icon: Codicon.expandAll,
            menu: {
                id: MenuId.ViewTitle,
                group: 'navigation',
                when: ( ContextKeyExpr.and(( ContextKeyExpr.and(( ContextKeyExpr.equals('view', COMMENTS_VIEW_ID)), CONTEXT_KEY_HAS_COMMENTS)), ContextKeyExpr.not(CONTEXT_KEY_SOME_COMMENTS_EXPANDED.key))),
                order: 100
            }
        });
    }
    runInView(_accessor, view) {
        view.expandAll();
    }
});
registerAction2(class Reply extends Action2 {
    constructor() {
        super({
            id: 'comments.reply',
            title: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'reply',
                "Reply"
            )),
            icon: Codicon.reply,
            menu: {
                id: MenuId.CommentsViewThreadActions,
                order: 100,
                when: ( ContextKeyExpr.equals('canReply', true))
            },
        });
    }
    run(accessor, marshalledCommentThread) {
        const commentService = accessor.get(ICommentService);
        const editorService = accessor.get(IEditorService);
        const uriIdentityService = accessor.get(IUriIdentityService);
        revealCommentThread(commentService, editorService, uriIdentityService, marshalledCommentThread.thread, marshalledCommentThread.thread.comments[marshalledCommentThread.thread.comments.length - 1], true);
    }
});
( Registry.as(Extensions.Configuration)).registerConfiguration({
    id: 'comments',
    order: 20,
    title: ( localizeWithPath(
        'vs/workbench/contrib/comments/browser/comments.contribution',
        'commentsConfigurationTitle',
        "Comments"
    )),
    type: 'object',
    properties: {
        'comments.openPanel': {
            enum: ['neverOpen', 'openOnSessionStart', 'openOnSessionStartWithComments'],
            default: 'openOnSessionStartWithComments',
            description: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'openComments',
                "Controls when the comments panel should open."
            )),
            restricted: false,
            markdownDeprecationMessage: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.openPanel.deprecated',
                "This setting is deprecated in favor of `comments.openView`."
            ))
        },
        'comments.openView': {
            enum: ['never', 'file', 'firstFile', 'firstFileUnresolved'],
            enumDescriptions: [( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.openView.never',
                "The comments view will never be opened."
            )), ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.openView.file',
                "The comments view will open when a file with comments is active."
            )), ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.openView.firstFile',
                "If the comments view has not been opened yet during this session it will open the first time during a session that a file with comments is active."
            )), ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.openView.firstFileUnresolved',
                "If the comments view has not been opened yet during this session and the comment is not resolved, it will open the first time during a session that a file with comments is active."
            ))],
            default: 'firstFile',
            description: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.openView',
                "Controls when the comments view should open."
            )),
            restricted: false
        },
        'comments.useRelativeTime': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'useRelativeTime',
                "Determines if relative time will be used in comment timestamps (ex. '1 day ago')."
            ))
        },
        'comments.visible': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.visible',
                "Controls the visibility of the comments bar and comment threads in editors that have commenting ranges and comments. Comments are still accessible via the Comments view and will cause commenting to be toggled on in the same way running the command \"Comments: Toggle Editor Commenting\" toggles comments."
            ))
        },
        'comments.maxHeight': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'comments.maxHeight',
                "Controls whether the comments widget scrolls or expands."
            ))
        },
        'comments.collapseOnResolve': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/comments/browser/comments.contribution',
                'collapseOnResolve',
                "Controls whether the comment thread should collapse when the thread is resolved."
            ))
        }
    }
});
let UnresolvedCommentsBadge = class UnresolvedCommentsBadge extends Disposable {
    constructor(_commentService, activityService) {
        super();
        this._commentService = _commentService;
        this.activityService = activityService;
        this.activity = this._register(( new MutableDisposable()));
        this.totalUnresolved = 0;
        this._register(this._commentService.onDidSetAllCommentThreads(this.onAllCommentsChanged, this));
        this._register(this._commentService.onDidUpdateCommentThreads(this.onCommentsUpdated, this));
    }
    onAllCommentsChanged(e) {
        let unresolved = 0;
        for (const thread of e.commentThreads) {
            if (thread.state === CommentThreadState.Unresolved) {
                unresolved++;
            }
        }
        this.updateBadge(unresolved);
    }
    onCommentsUpdated() {
        let unresolved = 0;
        for (const resource of this._commentService.commentsModel.resourceCommentThreads) {
            for (const thread of resource.commentThreads) {
                if (thread.threadState === CommentThreadState.Unresolved) {
                    unresolved++;
                }
            }
        }
        this.updateBadge(unresolved);
    }
    updateBadge(unresolved) {
        if (unresolved === this.totalUnresolved) {
            return;
        }
        this.totalUnresolved = unresolved;
        const message = ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/comments.contribution',
            'totalUnresolvedComments',
            '{0} Unresolved Comments',
            this.totalUnresolved
        ));
        this.activity.value = this.activityService.showViewActivity(COMMENTS_VIEW_ID, { badge: ( new NumberBadge(this.totalUnresolved, () => message)) });
    }
};
UnresolvedCommentsBadge = ( __decorate([
    ( __param(0, ICommentService)),
    ( __param(1, IActivityService))
], UnresolvedCommentsBadge));
( Registry.as(Extensions$1.Workbench)).registerWorkbenchContribution(UnresolvedCommentsBadge, 4 );
export { UnresolvedCommentsBadge };
