import { KeyChord } from '../../../../base/common/keyCodes.js';
import './media/review.css.js';
import { isDiffEditor, isCodeEditor } from '../../../../editor/browser/editorBrowser.js';
import { registerEditorContribution } from '../../../../editor/browser/editorExtensions.js';
import { ICodeEditorService } from '../../../../editor/browser/services/codeEditorService.js';
import { localizeWithPath } from '../../../../nls.js';
import { CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { KeybindingsRegistry } from '../../../../platform/keybinding/common/keybindingsRegistry.js';
import { ICommentService } from './commentService.js';
import { ctxCommentEditorFocused, SimpleCommentEditor } from './simpleCommentEditor.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { MenuRegistry, MenuId } from '../../../../platform/actions/common/actions.js';
import { EditorContextKeys } from '../../../../editor/common/editorContextKeys.js';
import { CommentController, ID } from './commentsController.js';
import { Range } from '../../../../editor/common/core/range.js';
import { INotificationService } from '../../../../platform/notification/common/notification.js';
import { CommentContextKeys } from '../common/commentContextKeys.js';
import { CONTEXT_ACCESSIBILITY_MODE_ENABLED } from '../../../../platform/accessibility/common/accessibility.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { accessibilityHelpIsShown, accessibleViewCurrentProviderId } from '../../accessibility/browser/accessibilityConfiguration.js';
registerEditorContribution(ID, CommentController, 1 );
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "editor.action.nextCommentThreadAction" ,
    handler: async (accessor, args) => {
        const activeEditor = getActiveEditor(accessor);
        if (!activeEditor) {
            return Promise.resolve();
        }
        const controller = CommentController.get(activeEditor);
        if (!controller) {
            return Promise.resolve();
        }
        controller.nextCommentThread();
    },
    weight: 100 ,
    primary: 512  | 67 ,
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "editor.action.previousCommentThreadAction" ,
    handler: async (accessor, args) => {
        const activeEditor = getActiveEditor(accessor);
        if (!activeEditor) {
            return Promise.resolve();
        }
        const controller = CommentController.get(activeEditor);
        if (!controller) {
            return Promise.resolve();
        }
        controller.previousCommentThread();
    },
    weight: 100 ,
    primary: 1024  | 512  | 67
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "editor.action.nextCommentingRange" ,
    handler: async (accessor, args) => {
        const activeEditor = getActiveEditor(accessor);
        if (!activeEditor) {
            return Promise.resolve();
        }
        const controller = CommentController.get(activeEditor);
        if (!controller) {
            return Promise.resolve();
        }
        controller.nextCommentingRange();
    },
    when: ( ContextKeyExpr.and(CONTEXT_ACCESSIBILITY_MODE_ENABLED, ( ContextKeyExpr.or(EditorContextKeys.focus, CommentContextKeys.commentFocused, ( ContextKeyExpr.and(accessibilityHelpIsShown, ( accessibleViewCurrentProviderId.isEqualTo("comments" )))))))),
    primary: KeyChord(2048  | 41 , 2048  | 512  | 18 ),
    weight: 100
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "editor.action.nextCommentingRange" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.nextCommentingRange',
            "Go to Next Commenting Range"
        )),
        category: 'Comments',
    },
    when: CommentContextKeys.activeEditorHasCommentingRange
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "editor.action.previousCommentingRange" ,
    handler: async (accessor, args) => {
        const activeEditor = getActiveEditor(accessor);
        if (!activeEditor) {
            return Promise.resolve();
        }
        const controller = CommentController.get(activeEditor);
        if (!controller) {
            return Promise.resolve();
        }
        controller.previousCommentingRange();
    },
    when: ( ContextKeyExpr.and(CONTEXT_ACCESSIBILITY_MODE_ENABLED, ( ContextKeyExpr.or(EditorContextKeys.focus, CommentContextKeys.commentFocused, ( ContextKeyExpr.and(accessibilityHelpIsShown, ( accessibleViewCurrentProviderId.isEqualTo("comments" )))))))),
    primary: KeyChord(2048  | 41 , 2048  | 512  | 16 ),
    weight: 100
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "editor.action.previousCommentingRange" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.previousCommentingRange',
            "Go to Previous Commenting Range"
        )),
        category: 'Comments',
    },
    when: CommentContextKeys.activeEditorHasCommentingRange
});
CommandsRegistry.registerCommand({
    id: "workbench.action.toggleCommenting" ,
    handler: (accessor) => {
        const commentService = accessor.get(ICommentService);
        const enable = commentService.isCommentingEnabled;
        commentService.enableCommenting(!enable);
    }
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "workbench.action.toggleCommenting" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.toggleCommenting',
            "Toggle Editor Commenting"
        )),
        category: 'Comments',
    },
    when: CommentContextKeys.WorkspaceHasCommenting
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "workbench.action.addComment" ,
    handler: async (accessor, args) => {
        const activeEditor = getActiveEditor(accessor);
        if (!activeEditor) {
            return Promise.resolve();
        }
        const controller = CommentController.get(activeEditor);
        if (!controller) {
            return Promise.resolve();
        }
        const position = args?.range ? ( new Range(
            args.range.startLineNumber,
            args.range.startLineNumber,
            args.range.endLineNumber,
            args.range.endColumn
        ))
            : (args?.fileComment ? undefined : activeEditor.getSelection());
        const notificationService = accessor.get(INotificationService);
        try {
            await controller.addOrToggleCommentAtLine(position, undefined);
        }
        catch (e) {
            notificationService.error(( localizeWithPath(
                'vs/workbench/contrib/comments/browser/commentsEditorContribution',
                'comments.addCommand.error',
                "The cursor must be within a commenting range to add a comment"
            )));
        }
    },
    weight: 100 ,
    primary: KeyChord(2048  | 41 , 2048  | 512  | 33 ),
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "workbench.action.addComment" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.addCommand',
            "Add Comment on Current Selection"
        )),
        category: 'Comments'
    },
    when: CommentContextKeys.activeCursorHasCommentingRange
});
CommandsRegistry.registerCommand({
    id: "workbench.action.collapseAllComments" ,
    handler: (accessor) => {
        return getActiveController(accessor)?.collapseAll();
    }
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "workbench.action.collapseAllComments" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.collapseAll',
            "Collapse All Comments"
        )),
        category: 'Comments'
    },
    when: CommentContextKeys.WorkspaceHasCommenting
});
CommandsRegistry.registerCommand({
    id: "workbench.action.expandAllComments" ,
    handler: (accessor) => {
        return getActiveController(accessor)?.expandAll();
    }
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "workbench.action.expandAllComments" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.expandAll',
            "Expand All Comments"
        )),
        category: 'Comments'
    },
    when: CommentContextKeys.WorkspaceHasCommenting
});
CommandsRegistry.registerCommand({
    id: "workbench.action.expandUnresolvedComments" ,
    handler: (accessor) => {
        return getActiveController(accessor)?.expandUnresolved();
    }
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: "workbench.action.expandUnresolvedComments" ,
        title: ( localizeWithPath(
            'vs/workbench/contrib/comments/browser/commentsEditorContribution',
            'comments.expandUnresolved',
            "Expand Unresolved Comments"
        )),
        category: 'Comments'
    },
    when: CommentContextKeys.WorkspaceHasCommenting
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "editor.action.submitComment" ,
    weight: 100 ,
    primary: 2048  | 3 ,
    when: ctxCommentEditorFocused,
    handler: (accessor, args) => {
        const activeCodeEditor = accessor.get(ICodeEditorService).getFocusedCodeEditor();
        if (activeCodeEditor instanceof SimpleCommentEditor) {
            activeCodeEditor.getParentThread().submitComment();
        }
    }
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: "workbench.action.hideComment" ,
    weight: 100 ,
    primary: 9 ,
    secondary: [1024  | 9 ],
    when: ctxCommentEditorFocused,
    handler: (accessor, args) => {
        const activeCodeEditor = accessor.get(ICodeEditorService).getFocusedCodeEditor();
        if (activeCodeEditor instanceof SimpleCommentEditor) {
            activeCodeEditor.getParentThread().collapse();
        }
    }
});
function getActiveEditor(accessor) {
    let activeTextEditorControl = accessor.get(IEditorService).activeTextEditorControl;
    if (isDiffEditor(activeTextEditorControl)) {
        if (activeTextEditorControl.getOriginalEditor().hasTextFocus()) {
            activeTextEditorControl = activeTextEditorControl.getOriginalEditor();
        }
        else {
            activeTextEditorControl = activeTextEditorControl.getModifiedEditor();
        }
    }
    if (!isCodeEditor(activeTextEditorControl) || !activeTextEditorControl.hasModel()) {
        return null;
    }
    return activeTextEditorControl;
}
function getActiveController(accessor) {
    const activeEditor = getActiveEditor(accessor);
    if (!activeEditor) {
        return undefined;
    }
    const controller = CommentController.get(activeEditor);
    if (!controller) {
        return undefined;
    }
    return controller;
}
export { getActiveEditor };
