import { InlineChatController } from './inlineChatController.js';
import { CTX_INLINE_CHAT_FOCUSED, CTX_INLINE_CHAT_RESPONSE_FOCUSED } from '../common/inlineChat.js';
import { IAccessibleViewService } from '../../accessibility/browser/accessibleView.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { ICodeEditorService } from '../../../../editor/browser/services/codeEditorService.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { AccessibleViewAction } from '../../accessibility/browser/accessibleViewActions.js';
class InlineChatAccessibleViewContribution extends Disposable {
    constructor() {
        super();
        this._register(AccessibleViewAction.addImplementation(100, 'inlineChat', accessor => {
            const accessibleViewService = accessor.get(IAccessibleViewService);
            const codeEditorService = accessor.get(ICodeEditorService);
            const editor = (codeEditorService.getActiveCodeEditor() || codeEditorService.getFocusedCodeEditor());
            if (!editor) {
                return false;
            }
            const controller = InlineChatController.get(editor);
            if (!controller) {
                return false;
            }
            const responseContent = controller?.getMessage();
            if (!responseContent) {
                return false;
            }
            accessibleViewService.show({
                id: "inlineChat" ,
                verbositySettingKey: "accessibility.verbosity.inlineChat" ,
                provideContent() { return responseContent; },
                onClose() {
                    controller.focus();
                },
                options: { type: "view"  }
            });
            return true;
        }, ( ContextKeyExpr.or(CTX_INLINE_CHAT_FOCUSED, CTX_INLINE_CHAT_RESPONSE_FOCUSED))));
    }
}
export { InlineChatAccessibleViewContribution };
