import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import './markersFileDecorations.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { Extensions } from '../../../../platform/configuration/common/configurationRegistry.js';
import { Categories } from '../../../../platform/action/common/actionCommonCategories.js';
import { KeybindingsRegistry } from '../../../../platform/keybinding/common/keybindingsRegistry.js';
import { localizeWithPath, localize2WithPath } from '../../../../nls.js';
import { Marker, ResourceMarkers, RelatedInformation } from './markersModel.js';
import { MarkersView } from './markersView.js';
import { registerAction2, MenuId, Action2 } from '../../../../platform/actions/common/actions.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { Markers, MarkersContextKeys } from '../common/markers.js';
import Messages from './messages.js';
import { Extensions as Extensions$2 } from '../../../common/contributions.js';
import { IClipboardService } from '../../../../platform/clipboard/common/clipboardService.js';
import { Disposable, MutableDisposable } from '../../../../base/common/lifecycle.js';
import { IStatusbarService } from '../../../services/statusbar/browser/statusbar.js';
import { IMarkerService } from '../../../../platform/markers/common/markers.js';
import { Extensions as Extensions$1 } from '../../../common/views.js';
import { IViewsService } from '../../../services/views/common/viewsService.js';
import { FocusedViewContext, getVisbileViewContextKey } from '../../../common/contextkeys.js';
import { ViewPaneContainer } from '../../../browser/parts/views/viewPaneContainer.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { registerIcon } from '../../../../platform/theme/common/iconRegistry.js';
import { ViewAction } from '../../../browser/parts/views/viewPane.js';
import { NumberBadge, IActivityService } from '../../../services/activity/common/activity.js';
import { viewFilterSubmenu } from '../../../browser/parts/views/viewFilter.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { problemsConfigurationNodeBase } from '../../../common/configuration.js';
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: Markers.MARKER_OPEN_ACTION_ID,
    weight: 200 ,
    when: ( ContextKeyExpr.and(MarkersContextKeys.MarkerFocusContextKey)),
    primary: 3 ,
    mac: {
        primary: 3 ,
        secondary: [2048  | 18 ]
    },
    handler: (accessor, args) => {
        const markersView = accessor.get(IViewsService).getActiveViewWithId(Markers.MARKERS_VIEW_ID);
        markersView.openFileAtElement(markersView.getFocusElement(), false, false, true);
    }
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: Markers.MARKER_OPEN_SIDE_ACTION_ID,
    weight: 200 ,
    when: ( ContextKeyExpr.and(MarkersContextKeys.MarkerFocusContextKey)),
    primary: 2048  | 3 ,
    mac: {
        primary: 256  | 3
    },
    handler: (accessor, args) => {
        const markersView = accessor.get(IViewsService).getActiveViewWithId(Markers.MARKERS_VIEW_ID);
        markersView.openFileAtElement(markersView.getFocusElement(), false, true, true);
    }
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: Markers.MARKER_SHOW_PANEL_ID,
    weight: 200 ,
    when: undefined,
    primary: undefined,
    handler: async (accessor, args) => {
        await accessor.get(IViewsService).openView(Markers.MARKERS_VIEW_ID);
    }
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: Markers.MARKER_SHOW_QUICK_FIX,
    weight: 200 ,
    when: MarkersContextKeys.MarkerFocusContextKey,
    primary: 2048  | 89 ,
    handler: (accessor, args) => {
        const markersView = accessor.get(IViewsService).getActiveViewWithId(Markers.MARKERS_VIEW_ID);
        const focusedElement = markersView.getFocusElement();
        if (focusedElement instanceof Marker) {
            markersView.showQuickFixes(focusedElement);
        }
    }
});
( Registry.as(Extensions.Configuration)).registerConfiguration({
    ...problemsConfigurationNodeBase,
    'properties': {
        'problems.autoReveal': {
            'description': Messages.PROBLEMS_PANEL_CONFIGURATION_AUTO_REVEAL,
            'type': 'boolean',
            'default': true
        },
        'problems.defaultViewMode': {
            'description': Messages.PROBLEMS_PANEL_CONFIGURATION_VIEW_MODE,
            'type': 'string',
            'default': 'tree',
            'enum': ['table', 'tree'],
        },
        'problems.showCurrentInStatus': {
            'description': Messages.PROBLEMS_PANEL_CONFIGURATION_SHOW_CURRENT_STATUS,
            'type': 'boolean',
            'default': false
        },
        'problems.sortOrder': {
            'description': Messages.PROBLEMS_PANEL_CONFIGURATION_COMPARE_ORDER,
            'type': 'string',
            'default': 'severity',
            'enum': ['severity', 'position'],
            'enumDescriptions': [
                Messages.PROBLEMS_PANEL_CONFIGURATION_COMPARE_ORDER_SEVERITY,
                Messages.PROBLEMS_PANEL_CONFIGURATION_COMPARE_ORDER_POSITION,
            ],
        },
    }
});
const markersViewIcon = registerIcon('markers-view-icon', Codicon.warning, ( localizeWithPath(
    'vs/workbench/contrib/markers/browser/markers.contribution',
    'markersViewIcon',
    'View icon of the markers view.'
)));
const VIEW_CONTAINER = ( Registry.as(Extensions$1.ViewContainersRegistry)).registerViewContainer({
    id: Markers.MARKERS_CONTAINER_ID,
    title: Messages.MARKERS_PANEL_TITLE_PROBLEMS,
    icon: markersViewIcon,
    hideIfEmpty: true,
    order: 0,
    ctorDescriptor: ( new SyncDescriptor(
        ViewPaneContainer,
        [Markers.MARKERS_CONTAINER_ID, { mergeViewWithContainerWhenSingleView: true }]
    )),
    storageId: Markers.MARKERS_VIEW_STORAGE_ID,
}, 1 , { doNotRegisterOpenCommand: true });
( Registry.as(Extensions$1.ViewsRegistry)).registerViews([{
        id: Markers.MARKERS_VIEW_ID,
        containerIcon: markersViewIcon,
        name: Messages.MARKERS_PANEL_TITLE_PROBLEMS,
        canToggleVisibility: false,
        canMoveView: true,
        ctorDescriptor: ( new SyncDescriptor(MarkersView)),
        openCommandActionDescriptor: {
            id: 'workbench.actions.view.problems',
            mnemonicTitle: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                { key: 'miMarker', comment: ['&& denotes a mnemonic'] },
                "&&Problems"
            )),
            keybindings: { primary: 2048  | 1024  | 43  },
            order: 0,
        }
    }], VIEW_CONTAINER);
const workbenchRegistry = ( Registry.as(Extensions$2.Workbench));
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.table.${Markers.MARKERS_VIEW_ID}.viewAsTree`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'viewAsTree',
                "View as Tree"
            )),
            menu: {
                id: MenuId.ViewTitle,
                when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)), ( MarkersContextKeys.MarkersViewModeContextKey.isEqualTo("table" )))),
                group: 'navigation',
                order: 3
            },
            icon: Codicon.listTree,
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.setViewMode("tree" );
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.table.${Markers.MARKERS_VIEW_ID}.viewAsTable`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'viewAsTable',
                "View as Table"
            )),
            menu: {
                id: MenuId.ViewTitle,
                when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)), ( MarkersContextKeys.MarkersViewModeContextKey.isEqualTo("tree" )))),
                group: 'navigation',
                order: 3
            },
            icon: Codicon.listFlat,
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.setViewMode("table" );
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.${Markers.MARKERS_VIEW_ID}.toggleErrors`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show errors',
                "Show Errors"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            toggled: MarkersContextKeys.ShowErrorsFilterContextKey,
            menu: {
                id: viewFilterSubmenu,
                group: '1_filter',
                when: ( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)),
                order: 1
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.filters.showErrors = !view.filters.showErrors;
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.${Markers.MARKERS_VIEW_ID}.toggleWarnings`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show warnings',
                "Show Warnings"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            toggled: MarkersContextKeys.ShowWarningsFilterContextKey,
            menu: {
                id: viewFilterSubmenu,
                group: '1_filter',
                when: ( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)),
                order: 2
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.filters.showWarnings = !view.filters.showWarnings;
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.${Markers.MARKERS_VIEW_ID}.toggleInfos`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show infos',
                "Show Infos"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            toggled: MarkersContextKeys.ShowInfoFilterContextKey,
            menu: {
                id: viewFilterSubmenu,
                group: '1_filter',
                when: ( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)),
                order: 3
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.filters.showInfos = !view.filters.showInfos;
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.${Markers.MARKERS_VIEW_ID}.toggleActiveFile`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show active file',
                "Show Active File Only"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            toggled: MarkersContextKeys.ShowActiveFileFilterContextKey,
            menu: {
                id: viewFilterSubmenu,
                group: '2_filter',
                when: ( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)),
                order: 1
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.filters.activeFile = !view.filters.activeFile;
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.${Markers.MARKERS_VIEW_ID}.toggleExcludedFiles`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show excluded files',
                "Show Excluded Files"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            toggled: ( MarkersContextKeys.ShowExcludedFilesFilterContextKey.negate()),
            menu: {
                id: viewFilterSubmenu,
                group: '2_filter',
                when: ( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)),
                order: 2
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        view.filters.excludedFiles = !view.filters.excludedFiles;
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.problems.focus',
            title: Messages.MARKERS_PANEL_SHOW_LABEL,
            category: Categories.View,
            f1: true,
        });
    }
    async run(accessor) {
        accessor.get(IViewsService).openView(Markers.MARKERS_VIEW_ID, true);
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        const when = ( ContextKeyExpr.and(( FocusedViewContext.isEqualTo(Markers.MARKERS_VIEW_ID)), MarkersContextKeys.MarkersTreeVisibilityContextKey, ( MarkersContextKeys.RelatedInformationFocusContextKey.toNegated())));
        super({
            id: Markers.MARKER_COPY_ACTION_ID,
            title: ( localize2WithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'copyMarker',
                'Copy'
            )),
            menu: {
                id: MenuId.ProblemsPanelContext,
                when,
                group: 'navigation'
            },
            keybinding: {
                weight: 200 ,
                primary: 2048  | 33 ,
                when
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        const clipboardService = serviceAccessor.get(IClipboardService);
        const selection = markersView.getFocusedSelectedElements() || markersView.getAllResourceMarkers();
        const markers = [];
        const addMarker = (marker) => {
            if (!markers.includes(marker)) {
                markers.push(marker);
            }
        };
        for (const selected of selection) {
            if (selected instanceof ResourceMarkers) {
                selected.markers.forEach(addMarker);
            }
            else if (selected instanceof Marker) {
                addMarker(selected);
            }
        }
        if (markers.length) {
            await clipboardService.writeText(`[${markers}]`);
        }
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.MARKER_COPY_MESSAGE_ACTION_ID,
            title: ( localize2WithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'copyMessage',
                'Copy Message'
            )),
            menu: {
                id: MenuId.ProblemsPanelContext,
                when: MarkersContextKeys.MarkerFocusContextKey,
                group: 'navigation'
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        const clipboardService = serviceAccessor.get(IClipboardService);
        const element = markersView.getFocusElement();
        if (element instanceof Marker) {
            await clipboardService.writeText(element.marker.message);
        }
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.RELATED_INFORMATION_COPY_MESSAGE_ACTION_ID,
            title: ( localize2WithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'copyMessage',
                'Copy Message'
            )),
            menu: {
                id: MenuId.ProblemsPanelContext,
                when: MarkersContextKeys.RelatedInformationFocusContextKey,
                group: 'navigation'
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        const clipboardService = serviceAccessor.get(IClipboardService);
        const element = markersView.getFocusElement();
        if (element instanceof RelatedInformation) {
            await clipboardService.writeText(element.raw.message);
        }
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.FOCUS_PROBLEMS_FROM_FILTER,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'focusProblemsList',
                "Focus problems view"
            )),
            keybinding: {
                when: MarkersContextKeys.MarkerViewFilterFocusContextKey,
                weight: 200 ,
                primary: 2048  | 18
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        markersView.focus();
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.MARKERS_VIEW_FOCUS_FILTER,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'focusProblemsFilter',
                "Focus problems filter"
            )),
            keybinding: {
                when: ( FocusedViewContext.isEqualTo(Markers.MARKERS_VIEW_ID)),
                weight: 200 ,
                primary: 2048  | 36
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        markersView.focusFilter();
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.MARKERS_VIEW_SHOW_MULTILINE_MESSAGE,
            title: ( localize2WithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show multiline',
                "Show message in multiple lines"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            menu: {
                id: MenuId.CommandPalette,
                when: ( ContextKeyExpr.has(getVisbileViewContextKey(Markers.MARKERS_VIEW_ID)))
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        markersView.setMultiline(true);
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.MARKERS_VIEW_SHOW_SINGLELINE_MESSAGE,
            title: ( localize2WithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'show singleline',
                "Show message in single line"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            menu: {
                id: MenuId.CommandPalette,
                when: ( ContextKeyExpr.has(getVisbileViewContextKey(Markers.MARKERS_VIEW_ID)))
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        markersView.setMultiline(false);
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: Markers.MARKERS_VIEW_CLEAR_FILTER_TEXT,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'clearFiltersText',
                "Clear filters text"
            )),
            category: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'problems',
                "Problems"
            )),
            keybinding: {
                when: MarkersContextKeys.MarkerViewFilterFocusContextKey,
                weight: 200 ,
                primary: 9
            },
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, markersView) {
        markersView.clearFilterText();
    }
});
registerAction2(class extends ViewAction {
    constructor() {
        super({
            id: `workbench.actions.treeView.${Markers.MARKERS_VIEW_ID}.collapseAll`,
            title: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'collapseAll',
                "Collapse All"
            )),
            menu: {
                id: MenuId.ViewTitle,
                when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', Markers.MARKERS_VIEW_ID)), ( MarkersContextKeys.MarkersViewModeContextKey.isEqualTo("tree" )))),
                group: 'navigation',
                order: 2,
            },
            icon: Codicon.collapseAll,
            viewId: Markers.MARKERS_VIEW_ID
        });
    }
    async runInView(serviceAccessor, view) {
        return view.collapseAll();
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: Markers.TOGGLE_MARKERS_VIEW_ACTION_ID,
            title: Messages.MARKERS_PANEL_TOGGLE_LABEL,
        });
    }
    async run(accessor) {
        const viewsService = accessor.get(IViewsService);
        if (viewsService.isViewVisible(Markers.MARKERS_VIEW_ID)) {
            viewsService.closeView(Markers.MARKERS_VIEW_ID);
        }
        else {
            viewsService.openView(Markers.MARKERS_VIEW_ID, true);
        }
    }
});
let MarkersStatusBarContributions = class MarkersStatusBarContributions extends Disposable {
    constructor(markerService, statusbarService, configurationService) {
        super();
        this.markerService = markerService;
        this.statusbarService = statusbarService;
        this.configurationService = configurationService;
        this.markersStatusItem = this._register(this.statusbarService.addEntry(this.getMarkersItem(), 'status.problems', 0 , 50 ));
        const addStatusBarEntry = () => {
            this.markersStatusItemOff = this.statusbarService.addEntry(this.getMarkersItemTurnedOff(), 'status.problemsVisibility', 0 , 49);
        };
        let config = this.configurationService.getValue('problems.visibility');
        if (!config) {
            addStatusBarEntry();
        }
        this._register(this.markerService.onMarkerChanged(() => {
            this.markersStatusItem.update(this.getMarkersItem());
        }));
        this._register(this.configurationService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration('problems.visibility')) {
                this.markersStatusItem.update(this.getMarkersItem());
                config = this.configurationService.getValue('problems.visibility');
                if (!config && !this.markersStatusItemOff) {
                    addStatusBarEntry();
                }
                else if (config && this.markersStatusItemOff) {
                    this.markersStatusItemOff.dispose();
                    this.markersStatusItemOff = undefined;
                }
            }
        }));
    }
    getMarkersItem() {
        const markersStatistics = this.markerService.getStatistics();
        const tooltip = this.getMarkersTooltip(markersStatistics);
        return {
            name: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'status.problems',
                "Problems"
            )),
            text: this.getMarkersText(markersStatistics),
            ariaLabel: tooltip,
            tooltip,
            command: 'workbench.actions.view.toggleProblems'
        };
    }
    getMarkersItemTurnedOff() {
        this.statusbarService.updateEntryVisibility('status.problemsVisibility', true);
        const openSettingsCommand = 'workbench.action.openSettings';
        const configureSettingsLabel = '@id:problems.visibility';
        const tooltip = ( localizeWithPath(
            'vs/workbench/contrib/markers/browser/markers.contribution',
            'status.problemsVisibilityOff',
            "Problems are turned off. Click to open settings."
        ));
        return {
            name: ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'status.problemsVisibility',
                "Problems Visibility"
            )),
            text: '$(whole-word)',
            ariaLabel: tooltip,
            tooltip,
            kind: 'warning',
            command: { title: openSettingsCommand, arguments: [configureSettingsLabel], id: openSettingsCommand }
        };
    }
    getMarkersTooltip(stats) {
        const errorTitle = (n) => ( localizeWithPath(
            'vs/workbench/contrib/markers/browser/markers.contribution',
            'totalErrors',
            "Errors: {0}",
            n
        ));
        const warningTitle = (n) => ( localizeWithPath(
            'vs/workbench/contrib/markers/browser/markers.contribution',
            'totalWarnings',
            "Warnings: {0}",
            n
        ));
        const infoTitle = (n) => ( localizeWithPath(
            'vs/workbench/contrib/markers/browser/markers.contribution',
            'totalInfos',
            "Infos: {0}",
            n
        ));
        const titles = [];
        if (stats.errors > 0) {
            titles.push(errorTitle(stats.errors));
        }
        if (stats.warnings > 0) {
            titles.push(warningTitle(stats.warnings));
        }
        if (stats.infos > 0) {
            titles.push(infoTitle(stats.infos));
        }
        if (titles.length === 0) {
            return ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'noProblems',
                "No Problems"
            ));
        }
        return titles.join(', ');
    }
    getMarkersText(stats) {
        const problemsText = [];
        problemsText.push('$(error) ' + this.packNumber(stats.errors));
        problemsText.push('$(warning) ' + this.packNumber(stats.warnings));
        if (stats.infos > 0) {
            problemsText.push('$(info) ' + this.packNumber(stats.infos));
        }
        return problemsText.join(' ');
    }
    packNumber(n) {
        const manyProblems = ( localizeWithPath(
            'vs/workbench/contrib/markers/browser/markers.contribution',
            'manyProblems',
            "10K+"
        ));
        return n > 9999 ? manyProblems : n > 999 ? ( n.toString()).charAt(0) + 'K' : ( n.toString());
    }
};
MarkersStatusBarContributions = ( __decorate([
    ( __param(0, IMarkerService)),
    ( __param(1, IStatusbarService)),
    ( __param(2, IConfigurationService))
], MarkersStatusBarContributions));
workbenchRegistry.registerWorkbenchContribution(MarkersStatusBarContributions, 3 );
let ActivityUpdater = class ActivityUpdater extends Disposable {
    constructor(activityService, markerService) {
        super();
        this.activityService = activityService;
        this.markerService = markerService;
        this.activity = this._register(( new MutableDisposable()));
        this._register(this.markerService.onMarkerChanged(() => this.updateBadge()));
        this.updateBadge();
    }
    updateBadge() {
        const { errors, warnings, infos } = this.markerService.getStatistics();
        const total = errors + warnings + infos;
        if (total > 0) {
            const message = ( localizeWithPath(
                'vs/workbench/contrib/markers/browser/markers.contribution',
                'totalProblems',
                'Total {0} Problems',
                total
            ));
            this.activity.value = this.activityService.showViewActivity(Markers.MARKERS_VIEW_ID, { badge: ( new NumberBadge(total, () => message)) });
        }
        else {
            this.activity.value = undefined;
        }
    }
};
ActivityUpdater = ( __decorate([
    ( __param(0, IActivityService)),
    ( __param(1, IMarkerService))
], ActivityUpdater));
workbenchRegistry.registerWorkbenchContribution(ActivityUpdater, 3 );
