import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { Event } from '../../../../base/common/event.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import '../../../../base/common/observableInternal/derived.js';
import '../../../../base/common/observableInternal/autorun.js';
import { observableFromEvent } from '../../../../base/common/observableInternal/utils.js';
import '../../../../base/common/cancellation.js';
import { URI } from '../../../../base/common/uri.js';
import { localize2WithPath, localizeWithPath } from '../../../../nls.js';
import { Action2, MenuId } from '../../../../platform/actions/common/actions.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { IMultiDiffSourceResolverService } from './multiDiffSourceResolverService.js';
import { ISCMService } from '../../scm/common/scm.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
var ScmMultiDiffSourceResolver_1;
let ScmMultiDiffSourceResolver = class ScmMultiDiffSourceResolver {
    static { ScmMultiDiffSourceResolver_1 = this; }
    static { this._scheme = 'scm-multi-diff-source'; }
    static getMultiDiffSourceUri(repositoryUri, groupId) {
        return ( URI.from({
            scheme: ScmMultiDiffSourceResolver_1._scheme,
            query: JSON.stringify({ repositoryUri, groupId }),
        }));
    }
    static parseUri(uri) {
        if (uri.scheme !== ScmMultiDiffSourceResolver_1._scheme) {
            return undefined;
        }
        let query;
        try {
            query = JSON.parse(uri.query);
        }
        catch (e) {
            return undefined;
        }
        if (typeof query !== 'object' || query === null) {
            return undefined;
        }
        const { repositoryUri, groupId } = query;
        if (typeof repositoryUri !== 'string' || typeof groupId !== 'string') {
            return undefined;
        }
        return { repositoryUri: ( URI.parse(repositoryUri)), groupId };
    }
    constructor(_scmService) {
        this._scmService = _scmService;
    }
    canHandleUri(uri) {
        return ScmMultiDiffSourceResolver_1.parseUri(uri) !== undefined;
    }
    async resolveDiffSource(uri) {
        const { repositoryUri, groupId } = ScmMultiDiffSourceResolver_1.parseUri(uri);
        const repository = await promiseFromEventState(this._scmService.onDidAddRepository, () => {
            const repository = [...this._scmService.repositories].find(r => r.provider.rootUri?.toString() === ( repositoryUri.toString()));
            return repository ?? false;
        });
        const group = await promiseFromEventState(repository.provider.onDidChangeResourceGroups, () => {
            const group = repository.provider.groups.find(g => g.id === groupId);
            return group ?? false;
        });
        const resources = observableFromEvent(group.onDidChangeResources, () => ( group.resources.map(e => {
            return {
                original: e.multiDiffEditorOriginalUri,
                modified: e.multiDiffEditorModifiedUri
            };
        })));
        return ( new ScmResolvedMultiDiffSource(resources, {
            scmResourceGroup: groupId,
            scmProvider: repository.provider.contextValue,
        }));
    }
};
ScmMultiDiffSourceResolver = ScmMultiDiffSourceResolver_1 = ( __decorate([
    ( __param(0, ISCMService))
], ScmMultiDiffSourceResolver));
class ScmResolvedMultiDiffSource {
    get resources() { return this._resources.get(); }
    constructor(_resources, contextKeys) {
        this._resources = _resources;
        this.contextKeys = contextKeys;
        this.onDidChange = Event.fromObservableLight(this._resources);
    }
}
function promiseFromEventState(event, checkState) {
    const state = checkState();
    if (state) {
        return Promise.resolve(state);
    }
    return ( new Promise(resolve => {
        const listener = event(() => {
            const state = checkState();
            if (state) {
                listener.dispose();
                resolve(state);
            }
        });
    }));
}
let ScmMultiDiffSourceResolverContribution = class ScmMultiDiffSourceResolverContribution extends Disposable {
    static { this.ID = 'workbench.contrib.scmMultiDiffSourceResolver'; }
    constructor(instantiationService, multiDiffSourceResolverService) {
        super();
        this._register(multiDiffSourceResolverService.registerResolver(instantiationService.createInstance(ScmMultiDiffSourceResolver)));
    }
};
ScmMultiDiffSourceResolverContribution = ( __decorate([
    ( __param(0, IInstantiationService)),
    ( __param(1, IMultiDiffSourceResolverService))
], ScmMultiDiffSourceResolverContribution));
class OpenScmGroupAction extends Action2 {
    constructor() {
        super({
            id: 'multiDiffEditor.openScmDiff',
            title: ( localize2WithPath(
                'vs/workbench/contrib/multiDiffEditor/browser/scmMultiDiffSourceResolver',
                'viewChanges',
                'View Changes'
            )),
            icon: Codicon.diffMultiple,
            menu: {
                when: ( ContextKeyExpr.and(( ContextKeyExpr.has('config.multiDiffEditor.experimental.enabled')), ( ContextKeyExpr.has('multiDiffEditorEnableViewChanges')))),
                id: MenuId.SCMResourceGroupContext,
                group: 'inline',
            },
            f1: false,
        });
    }
    async run(accessor, group) {
        const editorService = accessor.get(IEditorService);
        if (!group.provider.rootUri) {
            return;
        }
        const multiDiffSource = ScmMultiDiffSourceResolver.getMultiDiffSourceUri(( group.provider.rootUri.toString()), group.id);
        const label = ( localizeWithPath(
            'vs/workbench/contrib/multiDiffEditor/browser/scmMultiDiffSourceResolver',
            'scmDiffLabel',
            '{0}: {1}',
            group.provider.label,
            group.label
        ));
        await editorService.openEditor({ label, multiDiffSource });
    }
}
export { OpenScmGroupAction, ScmMultiDiffSourceResolver, ScmMultiDiffSourceResolverContribution };
