import { __decorate, __param } from '../../../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath } from '../../../../../../nls.js';
import { Disposable, DisposableStore, MutableDisposable } from '../../../../../../base/common/lifecycle.js';
import { Schemas } from '../../../../../../base/common/network.js';
import { ILanguageFeaturesService } from '../../../../../../editor/common/services/languageFeatures.js';
import { IConfigurationService } from '../../../../../../platform/configuration/common/configuration.js';
import { IInstantiationService } from '../../../../../../platform/instantiation/common/instantiation.js';
import { ILogService } from '../../../../../../platform/log/common/log.js';
import { Registry } from '../../../../../../platform/registry/common/platform.js';
import { Extensions, registerWorkbenchContribution2 } from '../../../../../common/contributions.js';
import { CENTER_ACTIVE_CELL } from '../navigation/arrow.js';
import { SELECT_KERNEL_ID } from '../../controller/coreActions.js';
import { SELECT_NOTEBOOK_INDENTATION_ID } from '../../controller/editActions.js';
import { getNotebookEditorFromEditorPane } from '../../notebookBrowser.js';
import { NotebookCellsChangeType } from '../../../common/notebookCommon.js';
import { INotebookKernelService } from '../../../common/notebookKernelService.js';
import { IEditorService } from '../../../../../services/editor/common/editorService.js';
import { IStatusbarService } from '../../../../../services/statusbar/browser/statusbar.js';
let ImplictKernelSelector = class ImplictKernelSelector {
    constructor(notebook, suggested, notebookKernelService, languageFeaturesService, logService) {
        const disposables = ( new DisposableStore());
        this.dispose = disposables.dispose.bind(disposables);
        const selectKernel = () => {
            disposables.clear();
            notebookKernelService.selectKernelForNotebook(suggested, notebook);
        };
        disposables.add(notebook.onDidChangeContent(e => {
            for (const event of e.rawEvents) {
                switch (event.kind) {
                    case NotebookCellsChangeType.ChangeCellContent:
                    case NotebookCellsChangeType.ModelChange:
                    case NotebookCellsChangeType.Move:
                    case NotebookCellsChangeType.ChangeCellLanguage:
                        logService.trace('IMPLICIT kernel selection because of change event', event.kind);
                        selectKernel();
                        break;
                }
            }
        }));
        disposables.add(languageFeaturesService.hoverProvider.register({ scheme: Schemas.vscodeNotebookCell, pattern: notebook.uri.path }, {
            provideHover() {
                logService.trace('IMPLICIT kernel selection because of hover');
                selectKernel();
                return undefined;
            }
        }));
    }
};
ImplictKernelSelector = ( __decorate([
    ( __param(2, INotebookKernelService)),
    ( __param(3, ILanguageFeaturesService)),
    ( __param(4, ILogService))
], ImplictKernelSelector));
let KernelStatus = class KernelStatus extends Disposable {
    constructor(_editorService, _statusbarService, _notebookKernelService, _instantiationService) {
        super();
        this._editorService = _editorService;
        this._statusbarService = _statusbarService;
        this._notebookKernelService = _notebookKernelService;
        this._instantiationService = _instantiationService;
        this._editorDisposables = this._register(( new DisposableStore()));
        this._kernelInfoElement = this._register(( new DisposableStore()));
        this._register(this._editorService.onDidActiveEditorChange(() => this._updateStatusbar()));
    }
    _updateStatusbar() {
        this._editorDisposables.clear();
        const activeEditor = getNotebookEditorFromEditorPane(this._editorService.activeEditorPane);
        if (!activeEditor) {
            this._kernelInfoElement.clear();
            return;
        }
        const updateStatus = () => {
            if (activeEditor.notebookOptions.getDisplayOptions().globalToolbar) {
                this._kernelInfoElement.clear();
                return;
            }
            const notebook = activeEditor.textModel;
            if (notebook) {
                this._showKernelStatus(notebook);
            }
            else {
                this._kernelInfoElement.clear();
            }
        };
        this._editorDisposables.add(this._notebookKernelService.onDidAddKernel(updateStatus));
        this._editorDisposables.add(this._notebookKernelService.onDidChangeSelectedNotebooks(updateStatus));
        this._editorDisposables.add(this._notebookKernelService.onDidChangeNotebookAffinity(updateStatus));
        this._editorDisposables.add(activeEditor.onDidChangeModel(updateStatus));
        this._editorDisposables.add(activeEditor.notebookOptions.onDidChangeOptions(updateStatus));
        updateStatus();
    }
    _showKernelStatus(notebook) {
        this._kernelInfoElement.clear();
        const { selected, suggestions, all } = this._notebookKernelService.getMatchingKernel(notebook);
        const suggested = (suggestions.length === 1 ? suggestions[0] : undefined)
            ?? (all.length === 1) ? all[0] : undefined;
        let isSuggested = false;
        if (all.length === 0) {
            return;
        }
        else if (selected || suggested) {
            let kernel = selected;
            if (!kernel) {
                kernel = suggested;
                isSuggested = true;
                this._kernelInfoElement.add(this._instantiationService.createInstance(ImplictKernelSelector, notebook, kernel));
            }
            const tooltip = kernel.description ?? kernel.detail ?? kernel.label;
            this._kernelInfoElement.add(this._statusbarService.addEntry({
                name: ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                    'notebook.info',
                    "Notebook Kernel Info"
                )),
                text: `$(notebook-kernel-select) ${kernel.label}`,
                ariaLabel: kernel.label,
                tooltip: isSuggested ? ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                    'tooltop',
                    "{0} (suggestion)",
                    tooltip
                )) : tooltip,
                command: SELECT_KERNEL_ID,
            }, SELECT_KERNEL_ID, 1 , 10));
            this._kernelInfoElement.add(kernel.onDidChange(() => this._showKernelStatus(notebook)));
        }
        else {
            this._kernelInfoElement.add(this._statusbarService.addEntry({
                name: ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                    'notebook.select',
                    "Notebook Kernel Selection"
                )),
                text: ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                    'kernel.select.label',
                    "Select Kernel"
                )),
                ariaLabel: ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                    'kernel.select.label',
                    "Select Kernel"
                )),
                command: SELECT_KERNEL_ID,
                kind: 'prominent'
            }, SELECT_KERNEL_ID, 1 , 10));
        }
    }
};
KernelStatus = ( __decorate([
    ( __param(0, IEditorService)),
    ( __param(1, IStatusbarService)),
    ( __param(2, INotebookKernelService)),
    ( __param(3, IInstantiationService))
], KernelStatus));
( Registry.as(Extensions.Workbench)).registerWorkbenchContribution(KernelStatus, 3 );
let ActiveCellStatus = class ActiveCellStatus extends Disposable {
    constructor(_editorService, _statusbarService) {
        super();
        this._editorService = _editorService;
        this._statusbarService = _statusbarService;
        this._itemDisposables = this._register(( new DisposableStore()));
        this._accessor = this._register(( new MutableDisposable()));
        this._register(this._editorService.onDidActiveEditorChange(() => this._update()));
    }
    _update() {
        this._itemDisposables.clear();
        const activeEditor = getNotebookEditorFromEditorPane(this._editorService.activeEditorPane);
        if (activeEditor) {
            this._itemDisposables.add(activeEditor.onDidChangeSelection(() => this._show(activeEditor)));
            this._itemDisposables.add(activeEditor.onDidChangeActiveCell(() => this._show(activeEditor)));
            this._show(activeEditor);
        }
        else {
            this._accessor.clear();
        }
    }
    _show(editor) {
        if (!editor.hasModel()) {
            this._accessor.clear();
            return;
        }
        const newText = this._getSelectionsText(editor);
        if (!newText) {
            this._accessor.clear();
            return;
        }
        const entry = {
            name: ( localizeWithPath(
                'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                'notebook.activeCellStatusName',
                "Notebook Editor Selections"
            )),
            text: newText,
            ariaLabel: newText,
            command: CENTER_ACTIVE_CELL
        };
        if (!this._accessor.value) {
            this._accessor.value = this._statusbarService.addEntry(entry, 'notebook.activeCellStatus', 1 , 100);
        }
        else {
            this._accessor.value.update(entry);
        }
    }
    _getSelectionsText(editor) {
        if (!editor.hasModel()) {
            return undefined;
        }
        const activeCell = editor.getActiveCell();
        if (!activeCell) {
            return undefined;
        }
        const idxFocused = editor.getCellIndex(activeCell) + 1;
        const numSelected = editor.getSelections().reduce((prev, range) => prev + (range.end - range.start), 0);
        const totalCells = editor.getLength();
        return numSelected > 1 ?
            ( localizeWithPath(
                'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                'notebook.multiActiveCellIndicator',
                "Cell {0} ({1} selected)",
                idxFocused,
                numSelected
            )) :
            ( localizeWithPath(
                'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                'notebook.singleActiveCellIndicator',
                "Cell {0} of {1}",
                idxFocused,
                totalCells
            ));
    }
};
ActiveCellStatus = ( __decorate([
    ( __param(0, IEditorService)),
    ( __param(1, IStatusbarService))
], ActiveCellStatus));
( Registry.as(Extensions.Workbench)).registerWorkbenchContribution(ActiveCellStatus, 3 );
let NotebookIndentationStatus = class NotebookIndentationStatus extends Disposable {
    static { this.ID = 'selectNotebookIndentation'; }
    constructor(_editorService, _statusbarService, _configurationService) {
        super();
        this._editorService = _editorService;
        this._statusbarService = _statusbarService;
        this._configurationService = _configurationService;
        this._itemDisposables = this._register(( new DisposableStore()));
        this._accessor = this._register(( new MutableDisposable()));
        this._register(this._editorService.onDidActiveEditorChange(() => this._update()));
        this._register(this._configurationService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration('editor') || e.affectsConfiguration('notebook')) {
                this._update();
            }
        }));
    }
    _update() {
        this._itemDisposables.clear();
        const activeEditor = getNotebookEditorFromEditorPane(this._editorService.activeEditorPane);
        if (activeEditor) {
            this._show(activeEditor);
            this._itemDisposables.add(activeEditor.onDidChangeSelection(() => {
                this._accessor.clear();
                this._show(activeEditor);
            }));
        }
        else {
            this._accessor.clear();
        }
    }
    _show(editor) {
        if (!editor.hasModel()) {
            this._accessor.clear();
            return;
        }
        const cellOptions = editor.getActiveCell()?.textModel?.getOptions();
        if (!cellOptions) {
            this._accessor.clear();
            return;
        }
        const cellEditorOverridesRaw = editor.notebookOptions.getDisplayOptions().editorOptionsCustomizations;
        const indentSize = cellEditorOverridesRaw['editor.indentSize'] ?? cellOptions?.indentSize;
        const insertSpaces = cellEditorOverridesRaw['editor.insertSpaces'] ?? cellOptions?.tabSize;
        const tabSize = cellEditorOverridesRaw['editor.tabSize'] ?? cellOptions?.insertSpaces;
        const width = typeof indentSize === 'number' ? indentSize : tabSize;
        const message = insertSpaces ? `Spaces: ${width}` : `Tab Size: ${width}`;
        const newText = message;
        if (!newText) {
            this._accessor.clear();
            return;
        }
        const entry = {
            name: ( localizeWithPath(
                'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                'notebook.indentation',
                "Notebook Indentation"
            )),
            text: newText,
            ariaLabel: newText,
            tooltip: ( localizeWithPath(
                'vs/workbench/contrib/notebook/browser/contrib/editorStatusBar/editorStatusBar',
                'selectNotebookIndentation',
                "Select Indentation"
            )),
            command: SELECT_NOTEBOOK_INDENTATION_ID
        };
        if (!this._accessor.value) {
            this._accessor.value = this._statusbarService.addEntry(entry, 'notebook.status.indentation', 1 , 100.4);
        }
        else {
            this._accessor.value.update(entry);
        }
    }
};
NotebookIndentationStatus = ( __decorate([
    ( __param(0, IEditorService)),
    ( __param(1, IStatusbarService)),
    ( __param(2, IConfigurationService))
], NotebookIndentationStatus));
registerWorkbenchContribution2(NotebookIndentationStatus.ID, NotebookIndentationStatus, 3 );
export { ActiveCellStatus, KernelStatus, NotebookIndentationStatus };
