import { __decorate, __param } from '../../../../../../../../../external/tslib/tslib.es6.js';
import { RunOnceScheduler } from '../../../../../../base/common/async.js';
import { localize2WithPath } from '../../../../../../nls.js';
import { createAndFillInContextMenuActions } from '../../../../../../platform/actions/browser/menuEntryActionViewItem.js';
import { MenuId, IMenuService } from '../../../../../../platform/actions/common/actions.js';
import { ICommandService } from '../../../../../../platform/commands/common/commands.js';
import { IConfigurationService } from '../../../../../../platform/configuration/common/configuration.js';
import { IContextKeyService } from '../../../../../../platform/contextkey/common/contextkey.js';
import { IContextMenuService } from '../../../../../../platform/contextview/browser/contextView.js';
import { IInstantiationService } from '../../../../../../platform/instantiation/common/instantiation.js';
import { IKeybindingService } from '../../../../../../platform/keybinding/common/keybinding.js';
import { WorkbenchAsyncDataTree } from '../../../../../../platform/list/browser/listService.js';
import { IOpenerService } from '../../../../../../platform/opener/common/opener.js';
import { IQuickInputService } from '../../../../../../platform/quickinput/common/quickInput.js';
import { ITelemetryService } from '../../../../../../platform/telemetry/common/telemetry.js';
import { IThemeService } from '../../../../../../platform/theme/common/themeService.js';
import { ViewPane } from '../../../../../browser/parts/views/viewPane.js';
import { IViewDescriptorService } from '../../../../../common/views.js';
import { CONTEXT_VARIABLE_NAME, CONTEXT_VARIABLE_VALUE, CONTEXT_VARIABLE_TYPE, CONTEXT_VARIABLE_INTERFACES, CONTEXT_VARIABLE_LANGUAGE, CONTEXT_VARIABLE_EXTENSIONID } from '../../../../debug/common/debug.js';
import { NotebookVariableDataSource } from './notebookVariablesDataSource.js';
import { NotebookVariablesDelegate, NotebookVariableRenderer, NotebookVariableAccessibilityProvider } from './notebookVariablesTree.js';
import { getNotebookEditorFromEditorPane } from '../../notebookBrowser.js';
import { INotebookExecutionStateService } from '../../../common/notebookExecutionStateService.js';
import { INotebookKernelService } from '../../../common/notebookKernelService.js';
import { IEditorService } from '../../../../../services/editor/common/editorService.js';
let NotebookVariablesView = class NotebookVariablesView extends ViewPane {
    static { this.ID = 'notebookVariablesView'; }
    static { this.TITLE = ( localize2WithPath(
        'vs/workbench/contrib/notebook/browser/contrib/notebookVariables/notebookVariablesView',
        'notebook.notebookVariables',
        "Notebook Variables"
    )); }
    constructor(options, editorService, notebookKernelService, notebookExecutionStateService, keybindingService, contextMenuService, contextKeyService, configurationService, instantiationService, viewDescriptorService, openerService, quickInputService, commandService, themeService, telemetryService, menuService) {
        super(options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService);
        this.editorService = editorService;
        this.notebookKernelService = notebookKernelService;
        this.notebookExecutionStateService = notebookExecutionStateService;
        this.quickInputService = quickInputService;
        this.commandService = commandService;
        this.menuService = menuService;
        this._register(this.editorService.onDidActiveEditorChange(this.handleActiveEditorChange.bind(this)));
        this._register(this.notebookKernelService.onDidNotebookVariablesUpdate(this.handleVariablesChanged.bind(this)));
        this._register(this.notebookExecutionStateService.onDidChangeExecution(this.handleExecutionStateChange.bind(this)));
        this.setActiveNotebook();
        this.dataSource = ( new NotebookVariableDataSource(this.notebookKernelService));
        this.updateScheduler = ( new RunOnceScheduler(() => this.tree?.updateChildren(), 100));
    }
    renderBody(container) {
        super.renderBody(container);
        this.element.classList.add('debug-pane');
        this.tree = this.instantiationService.createInstance(WorkbenchAsyncDataTree, 'notebookVariablesTree', container, ( new NotebookVariablesDelegate()), [( new NotebookVariableRenderer())], this.dataSource, {
            accessibilityProvider: ( new NotebookVariableAccessibilityProvider()),
            identityProvider: { getId: (e) => e.id },
        });
        this.tree.layout();
        if (this.activeNotebook) {
            this.tree.setInput({ kind: 'root', notebook: this.activeNotebook });
        }
        this._register(this.tree.onContextMenu(e => this.onContextMenu(e)));
    }
    onContextMenu(e) {
        if (!e.element) {
            return;
        }
        const element = e.element;
        const arg = {
            source: ( element.notebook.uri.toString()),
            name: element.name,
            value: element.value,
            type: element.type,
            expression: element.expression,
            language: element.language,
            extensionId: element.extensionId
        };
        const actions = [];
        const overlayedContext = this.contextKeyService.createOverlay([
            [CONTEXT_VARIABLE_NAME.key, element.name],
            [CONTEXT_VARIABLE_VALUE.key, element.value],
            [CONTEXT_VARIABLE_TYPE.key, element.type],
            [CONTEXT_VARIABLE_INTERFACES.key, element.interfaces],
            [CONTEXT_VARIABLE_LANGUAGE.key, element.language],
            [CONTEXT_VARIABLE_EXTENSIONID.key, element.extensionId]
        ]);
        const menu = this.menuService.createMenu(MenuId.NotebookVariablesContext, overlayedContext);
        createAndFillInContextMenuActions(menu, { arg, shouldForwardArgs: true }, actions);
        menu.dispose();
        this.contextMenuService.showContextMenu({
            getAnchor: () => e.anchor,
            getActions: () => actions
        });
    }
    layoutBody(height, width) {
        super.layoutBody(height, width);
        this.tree?.layout(height, width);
    }
    setActiveNotebook() {
        const current = this.activeNotebook;
        const activeEditorPane = this.editorService.activeEditorPane;
        if (activeEditorPane?.getId() === 'workbench.editor.notebook' || activeEditorPane?.getId() === 'workbench.editor.interactive') {
            const notebookDocument = getNotebookEditorFromEditorPane(activeEditorPane)?.getViewModel()?.notebookDocument;
            this.activeNotebook = notebookDocument;
        }
        return current !== this.activeNotebook;
    }
    handleActiveEditorChange() {
        if (this.setActiveNotebook() && this.activeNotebook) {
            this.tree?.setInput({ kind: 'root', notebook: this.activeNotebook });
            this.updateScheduler.schedule();
        }
    }
    handleExecutionStateChange(event) {
        if (this.activeNotebook) {
            if (event.affectsNotebook(this.activeNotebook.uri)) {
                this.dataSource.cancel();
                if (event.changed === undefined) {
                    this.updateScheduler.schedule();
                }
                else {
                    this.updateScheduler.cancel();
                }
            }
        }
    }
    handleVariablesChanged(notebookUri) {
        if (this.activeNotebook && ( notebookUri.toString()) === ( this.activeNotebook.uri.toString())) {
            this.tree?.setInput({ kind: 'root', notebook: this.activeNotebook });
            this.updateScheduler.schedule();
        }
    }
};
NotebookVariablesView = ( __decorate([
    ( __param(1, IEditorService)),
    ( __param(2, INotebookKernelService)),
    ( __param(3, INotebookExecutionStateService)),
    ( __param(4, IKeybindingService)),
    ( __param(5, IContextMenuService)),
    ( __param(6, IContextKeyService)),
    ( __param(7, IConfigurationService)),
    ( __param(8, IInstantiationService)),
    ( __param(9, IViewDescriptorService)),
    ( __param(10, IOpenerService)),
    ( __param(11, IQuickInputService)),
    ( __param(12, ICommandService)),
    ( __param(13, IThemeService)),
    ( __param(14, ITelemetryService)),
    ( __param(15, IMenuService))
], NotebookVariablesView));
export { NotebookVariablesView };
