import { __decorate, __param } from '../../../../../../../../external/tslib/tslib.es6.js';
import { onUnexpectedError } from '../../../../../base/common/errors.js';
import { Event } from '../../../../../base/common/event.js';
import { Disposable } from '../../../../../base/common/lifecycle.js';
import { localizeWithPath } from '../../../../../nls.js';
import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
import { Severity, INotificationService } from '../../../../../platform/notification/common/notification.js';
import { getInstalledExtensions } from '../../../extensions/common/extensionsUtils.js';
import { IWorkbenchExtensionEnablementService } from '../../../../services/extensionManagement/common/extensionManagement.js';
import { ILifecycleService } from '../../../../services/lifecycle/common/lifecycle.js';
import { IExtensionManagementService } from '../../../../../platform/extensionManagement/common/extensionManagement.js';
import { areSameExtensions } from '../../../../../platform/extensionManagement/common/extensionManagementUtil.js';
import { IStorageService } from '../../../../../platform/storage/common/storage.js';
import { Memento } from '../../../../common/memento.js';
import { distinct } from '../../../../../base/common/arrays.js';
function onExtensionChanged(accessor) {
    const extensionService = accessor.get(IExtensionManagementService);
    const extensionEnablementService = accessor.get(IWorkbenchExtensionEnablementService);
    const onDidInstallExtensions = Event.chain(extensionService.onDidInstallExtensions, $ => ( $.filter(e => ( e.some(({ operation }) => operation === 2 )))
        .map(e => ( e.map(({ identifier }) => identifier)))));
    return Event.debounce(Event.any(Event.any(onDidInstallExtensions, ( Event.map(extensionService.onDidUninstallExtension, e => [e.identifier]))), ( Event.map(
        extensionEnablementService.onEnablementChanged,
        extensions => ( extensions.map(e => e.identifier))
    ))), (result, identifiers) => {
        result = result || (identifiers.length ? [identifiers[0]] : []);
        for (const identifier of identifiers) {
            if (( result.some(l => !areSameExtensions(l, identifier)))) {
                result.push(identifier);
            }
        }
        return result;
    });
}
const hasRecommendedKeymapKey = 'hasRecommendedKeymap';
let NotebookKeymapService = class NotebookKeymapService extends Disposable {
    constructor(instantiationService, extensionEnablementService, notificationService, storageService, lifecycleService) {
        super();
        this.instantiationService = instantiationService;
        this.extensionEnablementService = extensionEnablementService;
        this.notificationService = notificationService;
        this.notebookKeymapMemento = ( new Memento('notebookKeymap', storageService));
        this.notebookKeymap = this.notebookKeymapMemento.getMemento(0 , 0 );
        this._register(lifecycleService.onDidShutdown(() => this.dispose()));
        this._register(this.instantiationService.invokeFunction(onExtensionChanged)((identifiers => {
            Promise.all(( identifiers.map(identifier => this.checkForOtherKeymaps(identifier))))
                .then(undefined, onUnexpectedError);
        })));
    }
    checkForOtherKeymaps(extensionIdentifier) {
        return this.instantiationService.invokeFunction(getInstalledExtensions).then(extensions => {
            const keymaps = extensions.filter(extension => isNotebookKeymapExtension(extension));
            const extension = keymaps.find(extension => areSameExtensions(extension.identifier, extensionIdentifier));
            if (extension && extension.globallyEnabled) {
                this.notebookKeymap[hasRecommendedKeymapKey] = true;
                this.notebookKeymapMemento.saveMemento();
                const otherKeymaps = keymaps.filter(extension => !areSameExtensions(extension.identifier, extensionIdentifier) && extension.globallyEnabled);
                if (otherKeymaps.length) {
                    return this.promptForDisablingOtherKeymaps(extension, otherKeymaps);
                }
            }
            return undefined;
        });
    }
    promptForDisablingOtherKeymaps(newKeymap, oldKeymaps) {
        const onPrompt = (confirmed) => {
            if (confirmed) {
                this.extensionEnablementService.setEnablement(( oldKeymaps.map(keymap => keymap.local)), 6 );
            }
        };
        this.notificationService.prompt(Severity.Info, ( localizeWithPath(
            'vs/workbench/contrib/notebook/browser/services/notebookKeymapServiceImpl',
            'disableOtherKeymapsConfirmation',
            "Disable other keymaps ({0}) to avoid conflicts between keybindings?",
            ( distinct(( oldKeymaps.map(k => k.local.manifest.displayName))).map(name => `'${name}'`)).join(', ')
        )), [{
                label: ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/services/notebookKeymapServiceImpl',
                    'yes',
                    "Yes"
                )),
                run: () => onPrompt(true)
            }, {
                label: ( localizeWithPath(
                    'vs/workbench/contrib/notebook/browser/services/notebookKeymapServiceImpl',
                    'no',
                    "No"
                )),
                run: () => onPrompt(false)
            }]);
    }
};
NotebookKeymapService = ( __decorate([
    ( __param(0, IInstantiationService)),
    ( __param(1, IWorkbenchExtensionEnablementService)),
    ( __param(2, INotificationService)),
    ( __param(3, IStorageService)),
    ( __param(4, ILifecycleService))
], NotebookKeymapService));
function isNotebookKeymapExtension(extension) {
    if (extension.local.manifest.extensionPack) {
        return false;
    }
    const keywords = extension.local.manifest.keywords;
    if (!keywords) {
        return false;
    }
    return keywords.indexOf('notebook-keymap') !== -1;
}
export { NotebookKeymapService, isNotebookKeymapExtension };
