import { localizeWithPath } from '../../../../nls.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { registerAction2, MenuId } from '../../../../platform/actions/common/actions.js';
import { ViewAction } from '../../../browser/parts/views/viewPane.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { IOutlinePane, ctxAllCollapsed, ctxFollowsCursor, ctxFilterOnType, ctxSortMode } from './outline.js';
registerAction2(class CollapseAll extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.collapse',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'collapse',
                "Collapse All"
            )),
            f1: false,
            icon: Codicon.collapseAll,
            menu: {
                id: MenuId.ViewTitle,
                group: 'navigation',
                when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', IOutlinePane.Id)), ( ctxAllCollapsed.isEqualTo(false))))
            }
        });
    }
    runInView(_accessor, view) {
        view.collapseAll();
    }
});
registerAction2(class ExpandAll extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.expand',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'expand',
                "Expand All"
            )),
            f1: false,
            icon: Codicon.expandAll,
            menu: {
                id: MenuId.ViewTitle,
                group: 'navigation',
                when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('view', IOutlinePane.Id)), ( ctxAllCollapsed.isEqualTo(true))))
            }
        });
    }
    runInView(_accessor, view) {
        view.expandAll();
    }
});
registerAction2(class FollowCursor extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.followCursor',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'followCur',
                "Follow Cursor"
            )),
            f1: false,
            toggled: ctxFollowsCursor,
            menu: {
                id: MenuId.ViewTitle,
                group: 'config',
                order: 1,
                when: ( ContextKeyExpr.equals('view', IOutlinePane.Id))
            }
        });
    }
    runInView(_accessor, view) {
        view.outlineViewState.followCursor = !view.outlineViewState.followCursor;
    }
});
registerAction2(class FilterOnType extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.filterOnType',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'filterOnType',
                "Filter on Type"
            )),
            f1: false,
            toggled: ctxFilterOnType,
            menu: {
                id: MenuId.ViewTitle,
                group: 'config',
                order: 2,
                when: ( ContextKeyExpr.equals('view', IOutlinePane.Id))
            }
        });
    }
    runInView(_accessor, view) {
        view.outlineViewState.filterOnType = !view.outlineViewState.filterOnType;
    }
});
registerAction2(class SortByPosition extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.sortByPosition',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'sortByPosition',
                "Sort By: Position"
            )),
            f1: false,
            toggled: ( ctxSortMode.isEqualTo(0 )),
            menu: {
                id: MenuId.ViewTitle,
                group: 'sort',
                order: 1,
                when: ( ContextKeyExpr.equals('view', IOutlinePane.Id))
            }
        });
    }
    runInView(_accessor, view) {
        view.outlineViewState.sortBy = 0 ;
    }
});
registerAction2(class SortByName extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.sortByName',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'sortByName',
                "Sort By: Name"
            )),
            f1: false,
            toggled: ( ctxSortMode.isEqualTo(1 )),
            menu: {
                id: MenuId.ViewTitle,
                group: 'sort',
                order: 2,
                when: ( ContextKeyExpr.equals('view', IOutlinePane.Id))
            }
        });
    }
    runInView(_accessor, view) {
        view.outlineViewState.sortBy = 1 ;
    }
});
registerAction2(class SortByKind extends ViewAction {
    constructor() {
        super({
            viewId: IOutlinePane.Id,
            id: 'outline.sortByKind',
            title: ( localizeWithPath(
                'vs/workbench/contrib/outline/browser/outlineActions',
                'sortByKind',
                "Sort By: Category"
            )),
            f1: false,
            toggled: ( ctxSortMode.isEqualTo(2 )),
            menu: {
                id: MenuId.ViewTitle,
                group: 'sort',
                order: 3,
                when: ( ContextKeyExpr.equals('view', IOutlinePane.Id))
            }
        });
    }
    runInView(_accessor, view) {
        view.outlineViewState.sortBy = 2 ;
    }
});
