import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { DropdownMenuActionViewItem } from '../../../../base/browser/ui/dropdown/dropdownActionViewItem.js';
import { SuggestController } from '../../../../editor/contrib/suggest/browser/suggestController.js';
import { localizeWithPath } from '../../../../nls.js';
import { IContextMenuService } from '../../../../platform/contextview/browser/contextView.js';
import { MODIFIED_SETTING_TAG, EXTENSION_SETTING_TAG, FEATURE_SETTING_TAG, GENERAL_TAG_SETTING_TAG, LANGUAGE_SETTING_TAG, POLICY_SETTING_TAG } from '../common/preferences.js';
let SettingsSearchFilterDropdownMenuActionViewItem = class SettingsSearchFilterDropdownMenuActionViewItem extends DropdownMenuActionViewItem {
    constructor(action, options, actionRunner, searchWidget, contextMenuService) {
        super(action, { getActions: () => this.getActions() }, contextMenuService, {
            ...options,
            actionRunner,
            classNames: action.class,
            anchorAlignmentProvider: () => 1 ,
            menuAsChild: true
        });
        this.searchWidget = searchWidget;
        this.suggestController = SuggestController.get(this.searchWidget.inputWidget);
    }
    render(container) {
        super.render(container);
    }
    doSearchWidgetAction(queryToAppend, triggerSuggest) {
        this.searchWidget.setValue(this.searchWidget.getValue().trimEnd() + ' ' + queryToAppend);
        this.searchWidget.focus();
        if (triggerSuggest && this.suggestController) {
            this.suggestController.triggerSuggest();
        }
    }
    createAction(id, label, tooltip, queryToAppend, triggerSuggest) {
        return {
            id,
            label,
            tooltip,
            class: undefined,
            enabled: true,
            run: () => { this.doSearchWidgetAction(queryToAppend, triggerSuggest); }
        };
    }
    createToggleAction(id, label, tooltip, queryToAppend) {
        const splitCurrentQuery = this.searchWidget.getValue().split(' ');
        const queryContainsQueryToAppend = splitCurrentQuery.includes(queryToAppend);
        return {
            id,
            label,
            tooltip,
            class: undefined,
            enabled: true,
            checked: queryContainsQueryToAppend,
            run: () => {
                if (!queryContainsQueryToAppend) {
                    const trimmedCurrentQuery = this.searchWidget.getValue().trimEnd();
                    const newQuery = trimmedCurrentQuery ? trimmedCurrentQuery + ' ' + queryToAppend : queryToAppend;
                    this.searchWidget.setValue(newQuery);
                }
                else {
                    const queryWithRemovedTags = this.searchWidget.getValue().split(' ')
                        .filter(word => word !== queryToAppend).join(' ');
                    this.searchWidget.setValue(queryWithRemovedTags);
                }
                this.searchWidget.focus();
            }
        };
    }
    getActions() {
        return [
            this.createToggleAction('modifiedSettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'modifiedSettingsSearch',
                "Modified"
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'modifiedSettingsSearchTooltip',
                "Add or remove modified settings filter"
            )), `@${MODIFIED_SETTING_TAG}`),
            this.createAction('extSettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'extSettingsSearch',
                "Extension ID..."
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'extSettingsSearchTooltip',
                "Add extension ID filter"
            )), `@${EXTENSION_SETTING_TAG}`, true),
            this.createAction('featuresSettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'featureSettingsSearch',
                "Feature..."
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'featureSettingsSearchTooltip',
                "Add feature filter"
            )), `@${FEATURE_SETTING_TAG}`, true),
            this.createAction('tagSettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'tagSettingsSearch',
                "Tag..."
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'tagSettingsSearchTooltip',
                "Add tag filter"
            )), `@${GENERAL_TAG_SETTING_TAG}`, true),
            this.createAction('langSettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'langSettingsSearch',
                "Language..."
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'langSettingsSearchTooltip',
                "Add language ID filter"
            )), `@${LANGUAGE_SETTING_TAG}`, true),
            this.createToggleAction('onlineSettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'onlineSettingsSearch',
                "Online services"
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'onlineSettingsSearchTooltip',
                "Show settings for online services"
            )), '@tag:usesOnlineServices'),
            this.createToggleAction('policySettingsSearch', ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'policySettingsSearch',
                "Policy services"
            )), ( localizeWithPath(
                'vs/workbench/contrib/preferences/browser/settingsSearchMenu',
                'policySettingsSearchTooltip',
                "Show settings for policy services"
            )), `@${POLICY_SETTING_TAG}`)
        ];
    }
};
SettingsSearchFilterDropdownMenuActionViewItem = ( __decorate([
    ( __param(4, IContextMenuService))
], SettingsSearchFilterDropdownMenuActionViewItem));
export { SettingsSearchFilterDropdownMenuActionViewItem };
