import { localizeWithPath } from '../../../../nls.js';
import { Extensions } from '../../../../platform/quickinput/common/quickAccess.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { HelpQuickAccessProvider } from '../../../../platform/quickinput/browser/helpQuickAccess.js';
import { ViewQuickAccessProvider, OpenViewPickerAction, QuickAccessViewPickerAction } from './viewQuickAccess.js';
import { CommandsQuickAccessProvider, ShowAllCommandsAction, ClearCommandHistoryAction } from './commandsQuickAccess.js';
import { MenuRegistry, MenuId, registerAction2 } from '../../../../platform/actions/common/actions.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { getQuickNavigateHandler, inQuickPickContext } from '../../../browser/quickaccess.js';
import { KeybindingsRegistry } from '../../../../platform/keybinding/common/keybindingsRegistry.js';
import { EditorContextKeys } from '../../../../editor/common/editorContextKeys.js';
const quickAccessRegistry = ( Registry.as(Extensions.Quickaccess));
quickAccessRegistry.registerQuickAccessProvider({
    ctor: HelpQuickAccessProvider,
    prefix: HelpQuickAccessProvider.PREFIX,
    placeholder: ( localizeWithPath(
        'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
        'helpQuickAccessPlaceholder',
        "Type '{0}' to get help on the actions you can take from here.",
        HelpQuickAccessProvider.PREFIX
    )),
    helpEntries: [{
            description: ( localizeWithPath(
                'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
                'helpQuickAccess',
                "Show all Quick Access Providers"
            )),
            commandCenterOrder: 70,
            commandCenterLabel: ( localizeWithPath(
                'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
                'more',
                'More'
            ))
        }]
});
quickAccessRegistry.registerQuickAccessProvider({
    ctor: ViewQuickAccessProvider,
    prefix: ViewQuickAccessProvider.PREFIX,
    contextKey: 'inViewsPicker',
    placeholder: ( localizeWithPath(
        'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
        'viewQuickAccessPlaceholder',
        "Type the name of a view, output channel or terminal to open."
    )),
    helpEntries: [{ description: ( localizeWithPath(
        'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
        'viewQuickAccess',
        "Open View"
    )), commandId: OpenViewPickerAction.ID }]
});
quickAccessRegistry.registerQuickAccessProvider({
    ctor: CommandsQuickAccessProvider,
    prefix: CommandsQuickAccessProvider.PREFIX,
    contextKey: 'inCommandsPicker',
    placeholder: ( localizeWithPath(
        'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
        'commandsQuickAccessPlaceholder',
        "Type the name of a command to run."
    )),
    helpEntries: [{ description: ( localizeWithPath(
        'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
        'commandsQuickAccess',
        "Show and Run Commands"
    )), commandId: ShowAllCommandsAction.ID, commandCenterOrder: 20 }]
});
MenuRegistry.appendMenuItem(MenuId.MenubarViewMenu, {
    group: '1_open',
    command: {
        id: ShowAllCommandsAction.ID,
        title: ( localizeWithPath(
            'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
            { key: 'miCommandPalette', comment: ['&& denotes a mnemonic'] },
            "&&Command Palette..."
        ))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.MenubarHelpMenu, {
    group: '1_welcome',
    command: {
        id: ShowAllCommandsAction.ID,
        title: ( localizeWithPath(
            'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
            { key: 'miShowAllCommands', comment: ['&& denotes a mnemonic'] },
            "Show All Commands"
        ))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarViewMenu, {
    group: '1_open',
    command: {
        id: OpenViewPickerAction.ID,
        title: ( localizeWithPath(
            'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
            { key: 'miOpenView', comment: ['&& denotes a mnemonic'] },
            "&&Open View..."
        ))
    },
    order: 2
});
MenuRegistry.appendMenuItem(MenuId.MenubarGoMenu, {
    group: '5_infile_nav',
    command: {
        id: 'workbench.action.gotoLine',
        title: ( localizeWithPath(
            'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
            { key: 'miGotoLine', comment: ['&& denotes a mnemonic'] },
            "Go to &&Line/Column..."
        ))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.GlobalActivity, {
    group: '1_command',
    command: {
        id: ShowAllCommandsAction.ID,
        title: ( localizeWithPath(
            'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
            'commandPalette',
            "Command Palette..."
        ))
    },
    order: 1
});
MenuRegistry.appendMenuItem(MenuId.EditorContext, {
    group: 'z_commands',
    when: ( EditorContextKeys.editorSimpleInput.toNegated()),
    command: {
        id: ShowAllCommandsAction.ID,
        title: ( localizeWithPath(
            'vs/workbench/contrib/quickaccess/browser/quickAccess.contribution',
            'commandPalette',
            "Command Palette..."
        )),
    },
    order: 1
});
registerAction2(ClearCommandHistoryAction);
registerAction2(ShowAllCommandsAction);
registerAction2(OpenViewPickerAction);
registerAction2(QuickAccessViewPickerAction);
const inViewsPickerContextKey = 'inViewsPicker';
const inViewsPickerContext = ( ContextKeyExpr.and(inQuickPickContext, ( ContextKeyExpr.has(inViewsPickerContextKey))));
const viewPickerKeybinding = QuickAccessViewPickerAction.KEYBINDING;
const quickAccessNavigateNextInViewPickerId = 'workbench.action.quickOpenNavigateNextInViewPicker';
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: quickAccessNavigateNextInViewPickerId,
    weight: 200  + 50,
    handler: getQuickNavigateHandler(quickAccessNavigateNextInViewPickerId, true),
    when: inViewsPickerContext,
    primary: viewPickerKeybinding.primary,
    linux: viewPickerKeybinding.linux,
    mac: viewPickerKeybinding.mac
});
const quickAccessNavigatePreviousInViewPickerId = 'workbench.action.quickOpenNavigatePreviousInViewPicker';
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: quickAccessNavigatePreviousInViewPickerId,
    weight: 200  + 50,
    handler: getQuickNavigateHandler(quickAccessNavigatePreviousInViewPickerId, false),
    when: inViewsPickerContext,
    primary: viewPickerKeybinding.primary | 1024 ,
    linux: viewPickerKeybinding.linux,
    mac: {
        primary: viewPickerKeybinding.mac.primary | 1024
    }
});
