import { localize2WithPath, localizeWithPath } from '../../../../nls.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { Extensions } from '../../../common/contributions.js';
import { DirtyDiffWorkbenchController } from './dirtydiffDecorator.js';
import { VIEWLET_ID, VIEW_PANE_ID, REPOSITORIES_VIEW_PANE_ID, ISCMService } from '../common/scm.js';
import { MenuRegistry, MenuId } from '../../../../platform/actions/common/actions.js';
import { SCMActiveResourceContextKeyController, SCMActiveRepositoryContextKeyController, SCMStatusController } from './activity.js';
import { Extensions as Extensions$2 } from '../../../../platform/configuration/common/configurationRegistry.js';
import { ContextKeyExpr, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { ICommandService, CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { KeybindingsRegistry } from '../../../../platform/keybinding/common/keybindingsRegistry.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import '../common/scmService.js';
import { Extensions as Extensions$1 } from '../../../common/views.js';
import { SCMViewPaneContainer } from './scmViewPaneContainer.js';
import { ModesRegistry } from '../../../../editor/common/languages/modesRegistry.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { registerIcon } from '../../../../platform/theme/common/iconRegistry.js';
import { SCMViewPane } from './scmViewPane.js';
import './scmViewService.js';
import { SCMRepositoriesViewPane } from './scmRepositoriesViewPane.js';
import { Context } from '../../../../editor/contrib/suggest/browser/suggest.js';
import { WorkspaceTrustContext, MANAGE_TRUST_COMMAND_ID } from '../../workspace/common/workspace.js';
import '../../../../platform/instantiation/common/instantiation.js';
import '../common/quickDiffService.js';
import { getActiveElement } from '../../../../base/browser/dom.js';
ModesRegistry.registerLanguage({
    id: 'scminput',
    extensions: [],
    aliases: [],
    mimetypes: ['text/x-scm-input']
});
( Registry.as(Extensions.Workbench))
    .registerWorkbenchContribution(DirtyDiffWorkbenchController, 3 );
const sourceControlViewIcon = registerIcon('source-control-view-icon', Codicon.sourceControl, ( localizeWithPath(
    'vs/workbench/contrib/scm/browser/scm.contribution',
    'sourceControlViewIcon',
    'View icon of the Source Control view.'
)));
const viewContainer = ( Registry.as(Extensions$1.ViewContainersRegistry)).registerViewContainer({
    id: VIEWLET_ID,
    title: ( localize2WithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'source control',
        'Source Control'
    )),
    ctorDescriptor: ( new SyncDescriptor(SCMViewPaneContainer)),
    storageId: 'workbench.scm.views.state',
    icon: sourceControlViewIcon,
    alwaysUseContainerInfo: true,
    order: 2,
    hideIfEmpty: true,
}, 0 , { doNotRegisterOpenCommand: true });
const viewsRegistry = ( Registry.as(Extensions$1.ViewsRegistry));
viewsRegistry.registerViewWelcomeContent(VIEW_PANE_ID, {
    content: ( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'no open repo',
        "No source control providers registered."
    )),
    when: 'default'
});
viewsRegistry.registerViewWelcomeContent(VIEW_PANE_ID, {
    content: ( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'no open repo in an untrusted workspace',
        "None of the registered source control providers work in Restricted Mode."
    )),
    when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('scm.providerCount', 0)), WorkspaceTrustContext.IsEnabled, ( WorkspaceTrustContext.IsTrusted.toNegated())))
});
viewsRegistry.registerViewWelcomeContent(VIEW_PANE_ID, {
    content: `[${( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'manageWorkspaceTrustAction',
        "Manage Workspace Trust"
    ))}](command:${MANAGE_TRUST_COMMAND_ID})`,
    when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('scm.providerCount', 0)), WorkspaceTrustContext.IsEnabled, ( WorkspaceTrustContext.IsTrusted.toNegated())))
});
viewsRegistry.registerViews([{
        id: VIEW_PANE_ID,
        name: ( localize2WithPath(
            'vs/workbench/contrib/scm/browser/scm.contribution',
            'source control',
            'Source Control'
        )),
        ctorDescriptor: ( new SyncDescriptor(SCMViewPane)),
        canToggleVisibility: true,
        canMoveView: true,
        weight: 80,
        order: -999,
        containerIcon: sourceControlViewIcon,
        openCommandActionDescriptor: {
            id: viewContainer.id,
            mnemonicTitle: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                { key: 'miViewSCM', comment: ['&& denotes a mnemonic'] },
                "Source &&Control"
            )),
            keybindings: {
                primary: 0,
                win: { primary: 2048  | 1024  | 37  },
                linux: { primary: 2048  | 1024  | 37  },
                mac: { primary: 256  | 1024  | 37  },
            },
            order: 2,
        }
    }], viewContainer);
viewsRegistry.registerViews([{
        id: REPOSITORIES_VIEW_PANE_ID,
        name: ( localize2WithPath(
            'vs/workbench/contrib/scm/browser/scm.contribution',
            'source control repositories',
            "Source Control Repositories"
        )),
        ctorDescriptor: ( new SyncDescriptor(SCMRepositoriesViewPane)),
        canToggleVisibility: true,
        hideByDefault: true,
        canMoveView: true,
        weight: 20,
        order: -1000,
        when: ( ContextKeyExpr.and(( ContextKeyExpr.has('scm.providerCount')), ( ContextKeyExpr.notEquals('scm.providerCount', 0)))),
        containerIcon: sourceControlViewIcon
    }], viewContainer);
( Registry.as(Extensions.Workbench))
    .registerWorkbenchContribution(SCMActiveResourceContextKeyController, 3 );
( Registry.as(Extensions.Workbench))
    .registerWorkbenchContribution(SCMActiveRepositoryContextKeyController, 3 );
( Registry.as(Extensions.Workbench))
    .registerWorkbenchContribution(SCMStatusController, 3 );
( Registry.as(Extensions$2.Configuration)).registerConfiguration({
    id: 'scm',
    order: 5,
    title: ( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'scmConfigurationTitle',
        "Source Control"
    )),
    type: 'object',
    scope: 4 ,
    properties: {
        'scm.diffDecorations': {
            type: 'string',
            enum: ['all', 'gutter', 'overview', 'minimap', 'none'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorations.all',
                    "Show the diff decorations in all available locations."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorations.gutter',
                    "Show the diff decorations only in the editor gutter."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorations.overviewRuler',
                    "Show the diff decorations only in the overview ruler."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorations.minimap',
                    "Show the diff decorations only in the minimap."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorations.none',
                    "Do not show the diff decorations."
                ))
            ],
            default: 'all',
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'diffDecorations',
                "Controls diff decorations in the editor."
            ))
        },
        'scm.diffDecorationsGutterWidth': {
            type: 'number',
            enum: [1, 2, 3, 4, 5],
            default: 3,
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'diffGutterWidth',
                "Controls the width(px) of diff decorations in gutter (added & modified)."
            ))
        },
        'scm.diffDecorationsGutterVisibility': {
            type: 'string',
            enum: ['always', 'hover'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsGutterVisibility.always',
                    "Show the diff decorator in the gutter at all times."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsGutterVisibility.hover',
                    "Show the diff decorator in the gutter only on hover."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.diffDecorationsGutterVisibility',
                "Controls the visibility of the Source Control diff decorator in the gutter."
            )),
            default: 'always'
        },
        'scm.diffDecorationsGutterAction': {
            type: 'string',
            enum: ['diff', 'none'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsGutterAction.diff',
                    "Show the inline diff Peek view on click."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsGutterAction.none',
                    "Do nothing."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.diffDecorationsGutterAction',
                "Controls the behavior of Source Control diff gutter decorations."
            )),
            default: 'diff'
        },
        'scm.diffDecorationsGutterPattern': {
            type: 'object',
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'diffGutterPattern',
                "Controls whether a pattern is used for the diff decorations in gutter."
            )),
            additionalProperties: false,
            properties: {
                'added': {
                    type: 'boolean',
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/scm/browser/scm.contribution',
                        'diffGutterPatternAdded',
                        "Use pattern for the diff decorations in gutter for added lines."
                    )),
                },
                'modified': {
                    type: 'boolean',
                    description: ( localizeWithPath(
                        'vs/workbench/contrib/scm/browser/scm.contribution',
                        'diffGutterPatternModifed',
                        "Use pattern for the diff decorations in gutter for modified lines."
                    )),
                },
            },
            default: {
                'added': false,
                'modified': true
            }
        },
        'scm.diffDecorationsIgnoreTrimWhitespace': {
            type: 'string',
            enum: ['true', 'false', 'inherit'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsIgnoreTrimWhitespace.true',
                    "Ignore leading and trailing whitespace."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsIgnoreTrimWhitespace.false',
                    "Do not ignore leading and trailing whitespace."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.diffDecorationsIgnoreTrimWhitespace.inherit',
                    "Inherit from `diffEditor.ignoreTrimWhitespace`."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'diffDecorationsIgnoreTrimWhitespace',
                "Controls whether leading and trailing whitespace is ignored in Source Control diff gutter decorations."
            )),
            default: 'false'
        },
        'scm.alwaysShowActions': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'alwaysShowActions',
                "Controls whether inline actions are always visible in the Source Control view."
            )),
            default: false
        },
        'scm.countBadge': {
            type: 'string',
            enum: ['all', 'focused', 'off'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.countBadge.all',
                    "Show the sum of all Source Control Provider count badges."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.countBadge.focused',
                    "Show the count badge of the focused Source Control Provider."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.countBadge.off',
                    "Disable the Source Control count badge."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.countBadge',
                "Controls the count badge on the Source Control icon on the Activity Bar."
            )),
            default: 'all'
        },
        'scm.providerCountBadge': {
            type: 'string',
            enum: ['hidden', 'auto', 'visible'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.providerCountBadge.hidden',
                    "Hide Source Control Provider count badges."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.providerCountBadge.auto',
                    "Only show count badge for Source Control Provider when non-zero."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.providerCountBadge.visible',
                    "Show Source Control Provider count badges."
                ))
            ],
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.providerCountBadge',
                "Controls the count badges on Source Control Provider headers. These headers appear in the Source Control view when there is more than one provider or when the {0} setting is enabled, and in the Source Control Repositories view.",
                '\`#scm.alwaysShowRepositories#\`'
            )),
            default: 'hidden'
        },
        'scm.defaultViewMode': {
            type: 'string',
            enum: ['tree', 'list'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.defaultViewMode.tree',
                    "Show the repository changes as a tree."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.defaultViewMode.list',
                    "Show the repository changes as a list."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.defaultViewMode',
                "Controls the default Source Control repository view mode."
            )),
            default: 'list'
        },
        'scm.defaultViewSortKey': {
            type: 'string',
            enum: ['name', 'path', 'status'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.defaultViewSortKey.name',
                    "Sort the repository changes by file name."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.defaultViewSortKey.path',
                    "Sort the repository changes by path."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.defaultViewSortKey.status',
                    "Sort the repository changes by Source Control status."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.defaultViewSortKey',
                "Controls the default Source Control repository changes sort order when viewed as a list."
            )),
            default: 'path'
        },
        'scm.autoReveal': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'autoReveal',
                "Controls whether the Source Control view should automatically reveal and select files when opening them."
            )),
            default: true
        },
        'scm.inputFontFamily': {
            type: 'string',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'inputFontFamily',
                "Controls the font for the input message. Use `default` for the workbench user interface font family, `editor` for the `#editor.fontFamily#`'s value, or a custom font family."
            )),
            default: 'default'
        },
        'scm.inputFontSize': {
            type: 'number',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'inputFontSize',
                "Controls the font size for the input message in pixels."
            )),
            default: 13
        },
        'scm.inputMaxLineCount': {
            type: 'number',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'inputMaxLines',
                "Controls the maximum number of lines that the input will auto-grow to."
            )),
            minimum: 1,
            maximum: 50,
            default: 10
        },
        'scm.inputMinLineCount': {
            type: 'number',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'inputMinLines',
                "Controls the minimum number of lines that the input will auto-grow from."
            )),
            minimum: 1,
            maximum: 50,
            default: 1
        },
        'scm.alwaysShowRepositories': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'alwaysShowRepository',
                "Controls whether repositories should always be visible in the Source Control view."
            )),
            default: false
        },
        'scm.repositories.sortOrder': {
            type: 'string',
            enum: ['discovery time', 'name', 'path'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.repositoriesSortOrder.discoveryTime',
                    "Repositories in the Source Control Repositories view are sorted by discovery time. Repositories in the Source Control view are sorted in the order that they were selected."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.repositoriesSortOrder.name',
                    "Repositories in the Source Control Repositories and Source Control views are sorted by repository name."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.repositoriesSortOrder.path',
                    "Repositories in the Source Control Repositories and Source Control views are sorted by repository path."
                ))
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'repositoriesSortOrder',
                "Controls the sort order of the repositories in the source control repositories view."
            )),
            default: 'discovery time'
        },
        'scm.repositories.visible': {
            type: 'number',
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'providersVisible',
                "Controls how many repositories are visible in the Source Control Repositories section. Set to 0, to be able to manually resize the view."
            )),
            default: 10
        },
        'scm.showActionButton': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'showActionButton',
                "Controls whether an action button can be shown in the Source Control view."
            )),
            default: true
        },
        'scm.showInputActionButton': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'showInputActionButton',
                "Controls whether an action button can be shown in the Source Control input."
            )),
            default: true
        },
        'scm.showIncomingChanges': {
            type: 'string',
            enum: ['always', 'never', 'auto'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.showIncomingChanges.always',
                    "Always show incoming changes in the Source Control view."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.showIncomingChanges.never',
                    "Never show incoming changes in the Source Control view."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.showIncomingChanges.auto',
                    "Only show incoming changes in the Source Control view when any exist."
                )),
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.showIncomingChanges',
                "Controls whether incoming changes are shown in the Source Control view."
            )),
            default: 'auto'
        },
        'scm.showOutgoingChanges': {
            type: 'string',
            enum: ['always', 'never', 'auto'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.showOutgoingChanges.always',
                    "Always show outgoing changes in the Source Control view."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.showOutgoingChanges.never',
                    "Never show outgoing changes in the Source Control view."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/scm/browser/scm.contribution',
                    'scm.showOutgoingChanges.auto',
                    "Only show outgoing changes in the Source Control view when any exist."
                )),
            ],
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.showOutgoingChanges',
                "Controls whether outgoing changes are shown in the Source Control view."
            )),
            default: 'auto'
        },
        'scm.showChangesSummary': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/scm/browser/scm.contribution',
                'scm.showChangesSummary',
                "Controls whether the All Changes entry is shown for incoming/outgoing changes in the Source Control view."
            )),
            default: true
        }
    }
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: 'scm.acceptInput',
    metadata: { description: ( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'scm accept',
        "Source Control: Accept Input"
    )), args: [] },
    weight: 200 ,
    when: ( ContextKeyExpr.has('scmRepository')),
    primary: 2048  | 3 ,
    handler: accessor => {
        const contextKeyService = accessor.get(IContextKeyService);
        const context = contextKeyService.getContext(getActiveElement());
        const repositoryId = context.getValue('scmRepository');
        if (!repositoryId) {
            return Promise.resolve(null);
        }
        const scmService = accessor.get(ISCMService);
        const repository = scmService.getRepository(repositoryId);
        if (!repository?.provider.acceptInputCommand) {
            return Promise.resolve(null);
        }
        const id = repository.provider.acceptInputCommand.id;
        const args = repository.provider.acceptInputCommand.arguments;
        const commandService = accessor.get(ICommandService);
        return commandService.executeCommand(id, ...(args || []));
    }
});
const viewNextCommitCommand = {
    description: { description: ( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'scm view next commit',
        "Source Control: View Next Commit"
    )), args: [] },
    weight: 200 ,
    handler: (accessor) => {
        const contextKeyService = accessor.get(IContextKeyService);
        const scmService = accessor.get(ISCMService);
        const context = contextKeyService.getContext(getActiveElement());
        const repositoryId = context.getValue('scmRepository');
        const repository = repositoryId ? scmService.getRepository(repositoryId) : undefined;
        repository?.input.showNextHistoryValue();
    }
};
const viewPreviousCommitCommand = {
    description: { description: ( localizeWithPath(
        'vs/workbench/contrib/scm/browser/scm.contribution',
        'scm view previous commit',
        "Source Control: View Previous Commit"
    )), args: [] },
    weight: 200 ,
    handler: (accessor) => {
        const contextKeyService = accessor.get(IContextKeyService);
        const scmService = accessor.get(ISCMService);
        const context = contextKeyService.getContext(getActiveElement());
        const repositoryId = context.getValue('scmRepository');
        const repository = repositoryId ? scmService.getRepository(repositoryId) : undefined;
        repository?.input.showPreviousHistoryValue();
    }
};
KeybindingsRegistry.registerCommandAndKeybindingRule({
    ...viewNextCommitCommand,
    id: 'scm.viewNextCommit',
    when: ( ContextKeyExpr.and(( ContextKeyExpr.has('scmRepository')), ( ContextKeyExpr.has('scmInputIsInLastPosition')), ( Context.Visible.toNegated()))),
    primary: 18
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    ...viewPreviousCommitCommand,
    id: 'scm.viewPreviousCommit',
    when: ( ContextKeyExpr.and(( ContextKeyExpr.has('scmRepository')), ( ContextKeyExpr.has('scmInputIsInFirstPosition')), ( Context.Visible.toNegated()))),
    primary: 16
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    ...viewNextCommitCommand,
    id: 'scm.forceViewNextCommit',
    when: ( ContextKeyExpr.has('scmRepository')),
    primary: 512  | 18
});
KeybindingsRegistry.registerCommandAndKeybindingRule({
    ...viewPreviousCommitCommand,
    id: 'scm.forceViewPreviousCommit',
    when: ( ContextKeyExpr.has('scmRepository')),
    primary: 512  | 16
});
CommandsRegistry.registerCommand('scm.openInIntegratedTerminal', async (accessor, provider) => {
    if (!provider || !provider.rootUri) {
        return;
    }
    const commandService = accessor.get(ICommandService);
    await commandService.executeCommand('openInIntegratedTerminal', provider.rootUri);
});
CommandsRegistry.registerCommand('scm.openInTerminal', async (accessor, provider) => {
    if (!provider || !provider.rootUri) {
        return;
    }
    const commandService = accessor.get(ICommandService);
    await commandService.executeCommand('openInTerminal', provider.rootUri);
});
MenuRegistry.appendMenuItem(MenuId.SCMSourceControl, {
    group: '100_end',
    command: {
        id: 'scm.openInTerminal',
        title: ( localizeWithPath(
            'vs/workbench/contrib/scm/browser/scm.contribution',
            'open in external terminal',
            "Open in External Terminal"
        ))
    },
    when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('scmProviderHasRootUri', true)), ( ContextKeyExpr.or(( ContextKeyExpr.equals('config.terminal.sourceControlRepositoriesKind', 'external')), ( ContextKeyExpr.equals('config.terminal.sourceControlRepositoriesKind', 'both'))))))
});
MenuRegistry.appendMenuItem(MenuId.SCMSourceControl, {
    group: '100_end',
    command: {
        id: 'scm.openInIntegratedTerminal',
        title: ( localizeWithPath(
            'vs/workbench/contrib/scm/browser/scm.contribution',
            'open in integrated terminal',
            "Open in Integrated Terminal"
        ))
    },
    when: ( ContextKeyExpr.and(( ContextKeyExpr.equals('scmProviderHasRootUri', true)), ( ContextKeyExpr.or(( ContextKeyExpr.equals('config.terminal.sourceControlRepositoriesKind', 'integrated')), ( ContextKeyExpr.equals('config.terminal.sourceControlRepositoriesKind', 'both'))))))
});
