import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import './media/scm.css.js';
import { DisposableStore, toDisposable, combinedDisposable } from '../../../../base/common/lifecycle.js';
import { append, $ } from '../../../../base/browser/dom.js';
import { ISCMViewService } from '../common/scm.js';
import { CountBadge } from '../../../../base/browser/ui/countBadge/countBadge.js';
import { IContextMenuService } from '../../../../platform/contextview/browser/contextView.js';
import { ICommandService } from '../../../../platform/commands/common/commands.js';
import { ActionRunner } from '../../../../base/common/actions.js';
import { connectPrimaryMenu, isSCMRepository, StatusBarAction } from './util.js';
import { defaultCountBadgeStyles } from '../../../../platform/theme/browser/defaultStyles.js';
import { WorkbenchToolBar } from '../../../../platform/actions/browser/toolbar.js';
import { MenuItemAction, MenuId, IMenuService } from '../../../../platform/actions/common/actions.js';
import { IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { IKeybindingService } from '../../../../platform/keybinding/common/keybinding.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.js';
var RepositoryRenderer_1;
class RepositoryActionRunner extends ActionRunner {
    constructor(getSelectedRepositories) {
        super();
        this.getSelectedRepositories = getSelectedRepositories;
    }
    async runAction(action, context) {
        if (!(action instanceof MenuItemAction)) {
            return super.runAction(action, context);
        }
        const selection = ( this.getSelectedRepositories().map(r => r.provider));
        const actionContext = ( selection.some(s => s === context)) ? selection : [context];
        await action.run(...actionContext);
    }
}
let RepositoryRenderer = class RepositoryRenderer {
    static { RepositoryRenderer_1 = this; }
    static { this.TEMPLATE_ID = 'repository'; }
    get templateId() { return RepositoryRenderer_1.TEMPLATE_ID; }
    constructor(toolbarMenuId, actionViewItemProvider, scmViewService, commandService, contextKeyService, contextMenuService, keybindingService, menuService, telemetryService) {
        this.toolbarMenuId = toolbarMenuId;
        this.actionViewItemProvider = actionViewItemProvider;
        this.scmViewService = scmViewService;
        this.commandService = commandService;
        this.contextKeyService = contextKeyService;
        this.contextMenuService = contextMenuService;
        this.keybindingService = keybindingService;
        this.menuService = menuService;
        this.telemetryService = telemetryService;
    }
    renderTemplate(container) {
        if (container.classList.contains('monaco-tl-contents')) {
            container.parentElement.parentElement.querySelector('.monaco-tl-twistie').classList.add('force-twistie');
        }
        const provider = append(container, $('.scm-provider'));
        const label = append(provider, $('.label'));
        const name = append(label, $('span.name'));
        const description = append(label, $('span.description'));
        const actions = append(provider, $('.actions'));
        const toolBar = ( new WorkbenchToolBar(
            actions,
            { actionViewItemProvider: this.actionViewItemProvider, resetMenu: this.toolbarMenuId },
            this.menuService,
            this.contextKeyService,
            this.contextMenuService,
            this.keybindingService,
            this.telemetryService
        ));
        const countContainer = append(provider, $('.count'));
        const count = ( new CountBadge(countContainer, {}, defaultCountBadgeStyles));
        const visibilityDisposable = toolBar.onDidChangeDropdownVisibility(e => provider.classList.toggle('active', e));
        const templateDisposable = combinedDisposable(visibilityDisposable, toolBar);
        return { label, name, description, countContainer, count, toolBar, elementDisposables: ( new DisposableStore()), templateDisposable };
    }
    renderElement(arg, index, templateData, height) {
        const repository = isSCMRepository(arg) ? arg : arg.element;
        templateData.name.textContent = repository.provider.name;
        if (repository.provider.rootUri) {
            templateData.label.title = `${repository.provider.label}: ${repository.provider.rootUri.fsPath}`;
            templateData.description.textContent = repository.provider.label;
        }
        else {
            templateData.label.title = repository.provider.label;
            templateData.description.textContent = '';
        }
        let statusPrimaryActions = [];
        let menuPrimaryActions = [];
        let menuSecondaryActions = [];
        const updateToolbar = () => {
            templateData.toolBar.setActions([...statusPrimaryActions, ...menuPrimaryActions], menuSecondaryActions);
        };
        const onDidChangeProvider = () => {
            const commands = repository.provider.statusBarCommands || [];
            statusPrimaryActions = ( commands.map(c => ( new StatusBarAction(c, this.commandService))));
            updateToolbar();
            const count = repository.provider.count || 0;
            templateData.countContainer.setAttribute('data-count', String(count));
            templateData.count.setCount(count);
        };
        let disposed = false;
        templateData.elementDisposables.add(toDisposable(() => disposed = true));
        templateData.elementDisposables.add(repository.provider.onDidChange(() => {
            if (disposed) {
                return;
            }
            onDidChangeProvider();
        }));
        onDidChangeProvider();
        const repositoryMenus = this.scmViewService.menus.getRepositoryMenus(repository.provider);
        const menu = this.toolbarMenuId === MenuId.SCMTitle ? repositoryMenus.titleMenu.menu : repositoryMenus.repositoryMenu;
        templateData.elementDisposables.add(connectPrimaryMenu(menu, (primary, secondary) => {
            menuPrimaryActions = primary;
            menuSecondaryActions = secondary;
            updateToolbar();
        }));
        templateData.toolBar.context = repository.provider;
    }
    renderCompressedElements() {
        throw new Error('Should never happen since node is incompressible');
    }
    disposeElement(group, index, template) {
        template.elementDisposables.clear();
    }
    disposeTemplate(templateData) {
        templateData.elementDisposables.dispose();
        templateData.templateDisposable.dispose();
    }
};
RepositoryRenderer = RepositoryRenderer_1 = ( __decorate([
    ( __param(2, ISCMViewService)),
    ( __param(3, ICommandService)),
    ( __param(4, IContextKeyService)),
    ( __param(5, IContextMenuService)),
    ( __param(6, IKeybindingService)),
    ( __param(7, IMenuService)),
    ( __param(8, ITelemetryService))
], RepositoryRenderer));
export { RepositoryActionRunner, RepositoryRenderer };
