import { isMacintosh } from '../../../../base/common/platform.js';
import { AbstractGotoLineQuickAccessProvider } from '../../../../editor/contrib/quickAccess/browser/gotoLineQuickAccess.js';
import { localize2WithPath, localizeWithPath } from '../../../../nls.js';
import { Extensions as Extensions$2 } from '../../../../platform/configuration/common/configurationRegistry.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { Extensions as Extensions$1 } from '../../../../platform/quickinput/common/quickAccess.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { ViewPaneContainer } from '../../../browser/parts/views/viewPaneContainer.js';
import { defaultQuickAccessContextKeyValue } from '../../../browser/quickaccess.js';
import { Extensions } from '../../../common/views.js';
import { GotoSymbolQuickAccessProvider } from '../../codeEditor/browser/quickaccess/gotoSymbolQuickAccess.js';
import { AnythingQuickAccessProvider } from './anythingQuickAccess.js';
import { registerContributions } from './replaceContributions.js';
import '../../../../platform/instantiation/common/instantiation.js';
import './notebookSearch/notebookSearchService.js';
import { searchViewIcon } from './searchIcons.js';
import { SearchView } from './searchView.js';
import { registerContributions as registerContributions$1 } from './searchWidget.js';
import { SymbolsQuickAccessProvider } from './symbolsQuickAccess.js';
import '../common/searchHistoryService.js';
import './searchModel.js';
import { VIEWLET_ID, SEARCH_EXCLUDE_CONFIG, VIEW_ID } from '../../../services/search/common/search.js';
import { CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { assertType } from '../../../../base/common/types.js';
import { getWorkspaceSymbols } from '../common/search.js';
import './searchActionsCopy.js';
import './searchActionsFind.js';
import './searchActionsNav.js';
import './searchActionsRemoveReplace.js';
import './searchActionsSymbol.js';
import './searchActionsTopBar.js';
import './searchActionsTextQuickAccess.js';
import { TextSearchQuickAccess, TEXT_SEARCH_QUICK_ACCESS_PREFIX } from './quickTextSearch/textSearchQuickAccess.js';
registerContributions();
registerContributions$1();
const SEARCH_MODE_CONFIG = 'search.mode';
const viewContainer = ( Registry.as(Extensions.ViewContainersRegistry)).registerViewContainer({
    id: VIEWLET_ID,
    title: ( localize2WithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'search',
        "Search"
    )),
    ctorDescriptor: ( new SyncDescriptor(
        ViewPaneContainer,
        [VIEWLET_ID, { mergeViewWithContainerWhenSingleView: true }]
    )),
    hideIfEmpty: true,
    icon: searchViewIcon,
    order: 1,
}, 0 , { doNotRegisterOpenCommand: true });
const viewDescriptor = {
    id: VIEW_ID,
    containerIcon: searchViewIcon,
    name: ( localize2WithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'search',
        "Search"
    )),
    ctorDescriptor: ( new SyncDescriptor(SearchView)),
    canToggleVisibility: false,
    canMoveView: true,
    openCommandActionDescriptor: {
        id: viewContainer.id,
        mnemonicTitle: ( localizeWithPath(
            'vs/workbench/contrib/search/browser/search.contribution',
            { key: 'miViewSearch', comment: ['&& denotes a mnemonic'] },
            "&&Search"
        )),
        keybindings: {
            primary: 2048  | 1024  | 36 ,
            when: ( ContextKeyExpr.regex('neverMatch', /doesNotMatch/))
        },
        order: 1
    }
};
( Registry.as(Extensions.ViewsRegistry)).registerViews([viewDescriptor], viewContainer);
const quickAccessRegistry = ( Registry.as(Extensions$1.Quickaccess));
quickAccessRegistry.registerQuickAccessProvider({
    ctor: AnythingQuickAccessProvider,
    prefix: AnythingQuickAccessProvider.PREFIX,
    placeholder: ( localizeWithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'anythingQuickAccessPlaceholder',
        "Search files by name (append {0} to go to line or {1} to go to symbol)",
        AbstractGotoLineQuickAccessProvider.PREFIX,
        GotoSymbolQuickAccessProvider.PREFIX
    )),
    contextKey: defaultQuickAccessContextKeyValue,
    helpEntries: [{
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'anythingQuickAccess',
                "Go to File"
            )),
            commandId: 'workbench.action.quickOpen',
            commandCenterOrder: 10
        }]
});
quickAccessRegistry.registerQuickAccessProvider({
    ctor: SymbolsQuickAccessProvider,
    prefix: SymbolsQuickAccessProvider.PREFIX,
    placeholder: ( localizeWithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'symbolsQuickAccessPlaceholder',
        "Type the name of a symbol to open."
    )),
    contextKey: 'inWorkspaceSymbolsPicker',
    helpEntries: [{ description: ( localizeWithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'symbolsQuickAccess',
        "Go to Symbol in Workspace"
    )), commandId: "workbench.action.showAllSymbols"  }]
});
quickAccessRegistry.registerQuickAccessProvider({
    ctor: TextSearchQuickAccess,
    prefix: TEXT_SEARCH_QUICK_ACCESS_PREFIX,
    contextKey: 'inTextSearchPicker',
    placeholder: ( localizeWithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'textSearchPickerPlaceholder',
        "Search for text in your workspace files (experimental)."
    )),
    helpEntries: [
        {
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'textSearchPickerHelp',
                "Search for Text (Experimental)"
            )),
            commandId: "workbench.action.experimental.quickTextSearch" ,
            commandCenterOrder: 65,
        }
    ]
});
const configurationRegistry = ( Registry.as(Extensions$2.Configuration));
configurationRegistry.registerConfiguration({
    id: 'search',
    order: 13,
    title: ( localizeWithPath(
        'vs/workbench/contrib/search/browser/search.contribution',
        'searchConfigurationTitle',
        "Search"
    )),
    type: 'object',
    properties: {
        [SEARCH_EXCLUDE_CONFIG]: {
            type: 'object',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'exclude',
                "Configure [glob patterns](https://code.visualstudio.com/docs/editor/codebasics#_advanced-search-options) for excluding files and folders in fulltext searches and file search in quick open. To exclude files from the recently opened list in quick open, patterns must be absolute (for example `**/node_modules/**`). Inherits all glob patterns from the `#files.exclude#` setting."
            )),
            default: { '**/node_modules': true, '**/bower_components': true, '**/*.code-search': true },
            additionalProperties: {
                anyOf: [
                    {
                        type: 'boolean',
                        description: ( localizeWithPath(
                            'vs/workbench/contrib/search/browser/search.contribution',
                            'exclude.boolean',
                            "The glob pattern to match file paths against. Set to true or false to enable or disable the pattern."
                        )),
                    },
                    {
                        type: 'object',
                        properties: {
                            when: {
                                type: 'string',
                                pattern: '\\w*\\$\\(basename\\)\\w*',
                                default: '$(basename).ext',
                                markdownDescription: ( localizeWithPath(
                                    'vs/workbench/contrib/search/browser/search.contribution',
                                    { key: 'exclude.when', comment: ['\\$(basename) should not be translated'] },
                                    'Additional check on the siblings of a matching file. Use \\$(basename) as variable for the matching file name.'
                                ))
                            }
                        }
                    }
                ]
            },
            scope: 4
        },
        [SEARCH_MODE_CONFIG]: {
            type: 'string',
            enum: ['view', 'reuseEditor', 'newEditor'],
            default: 'view',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.mode',
                "Controls where new `Search: Find in Files` and `Find in Folder` operations occur: either in the search view, or in a search editor."
            )),
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.mode.view',
                    "Search in the search view, either in the panel or side bars."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.mode.reuseEditor',
                    "Search in an existing search editor if present, otherwise in a new search editor."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.mode.newEditor',
                    "Search in a new search editor."
                )),
            ]
        },
        'search.useRipgrep': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'useRipgrep',
                "This setting is deprecated and now falls back on \"search.usePCRE2\"."
            )),
            deprecationMessage: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'useRipgrepDeprecated',
                "Deprecated. Consider \"search.usePCRE2\" for advanced regex feature support."
            )),
            default: true
        },
        'search.maintainFileSearchCache': {
            type: 'boolean',
            deprecationMessage: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'maintainFileSearchCacheDeprecated',
                "The search cache is kept in the extension host which never shuts down, so this setting is no longer needed."
            )),
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.maintainFileSearchCache',
                "When enabled, the searchService process will be kept alive instead of being shut down after an hour of inactivity. This will keep the file search cache in memory."
            )),
            default: false
        },
        'search.useIgnoreFiles': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'useIgnoreFiles',
                "Controls whether to use `.gitignore` and `.ignore` files when searching for files."
            )),
            default: true,
            scope: 4
        },
        'search.useGlobalIgnoreFiles': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'useGlobalIgnoreFiles',
                "Controls whether to use your global gitignore file (for example, from `$HOME/.config/git/ignore`) when searching for files. Requires `#search.useIgnoreFiles#` to be enabled."
            )),
            default: false,
            scope: 4
        },
        'search.useParentIgnoreFiles': {
            type: 'boolean',
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'useParentIgnoreFiles',
                "Controls whether to use `.gitignore` and `.ignore` files in parent directories when searching for files. Requires `#search.useIgnoreFiles#` to be enabled."
            )),
            default: false,
            scope: 4
        },
        'search.quickOpen.includeSymbols': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.quickOpen.includeSymbols',
                "Whether to include results from a global symbol search in the file results for Quick Open."
            )),
            default: false
        },
        'search.quickOpen.includeHistory': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.quickOpen.includeHistory',
                "Whether to include results from recently opened files in the file results for Quick Open."
            )),
            default: true
        },
        'search.quickOpen.history.filterSortOrder': {
            'type': 'string',
            'enum': ['default', 'recency'],
            'default': 'default',
            'enumDescriptions': [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'filterSortOrder.default',
                    'History entries are sorted by relevance based on the filter value used. More relevant entries appear first.'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'filterSortOrder.recency',
                    'History entries are sorted by recency. More recently opened entries appear first.'
                ))
            ],
            'description': ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'filterSortOrder',
                "Controls sorting order of editor history in quick open when filtering."
            ))
        },
        'search.followSymlinks': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.followSymlinks',
                "Controls whether to follow symlinks while searching."
            )),
            default: true
        },
        'search.smartCase': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.smartCase',
                "Search case-insensitively if the pattern is all lowercase, otherwise, search case-sensitively."
            )),
            default: false
        },
        'search.globalFindClipboard': {
            type: 'boolean',
            default: false,
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.globalFindClipboard',
                "Controls whether the search view should read or modify the shared find clipboard on macOS."
            )),
            included: isMacintosh
        },
        'search.location': {
            type: 'string',
            enum: ['sidebar', 'panel'],
            default: 'sidebar',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.location',
                "Controls whether the search will be shown as a view in the sidebar or as a panel in the panel area for more horizontal space."
            )),
            deprecationMessage: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.location.deprecationMessage',
                "This setting is deprecated. You can drag the search icon to a new location instead."
            ))
        },
        'search.maxResults': {
            type: ['number', 'null'],
            default: 20000,
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.maxResults',
                "Controls the maximum number of search results, this can be set to `null` (empty) to return unlimited results."
            ))
        },
        'search.collapseResults': {
            type: 'string',
            enum: ['auto', 'alwaysCollapse', 'alwaysExpand'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.collapseResults.auto',
                    "Files with less than 10 results are expanded. Others are collapsed."
                )),
                '',
                ''
            ],
            default: 'alwaysExpand',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.collapseAllResults',
                "Controls whether the search results will be collapsed or expanded."
            )),
        },
        'search.useReplacePreview': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.useReplacePreview',
                "Controls whether to open Replace Preview when selecting or replacing a match."
            )),
        },
        'search.showLineNumbers': {
            type: 'boolean',
            default: false,
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.showLineNumbers',
                "Controls whether to show line numbers for search results."
            )),
        },
        'search.usePCRE2': {
            type: 'boolean',
            default: false,
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.usePCRE2',
                "Whether to use the PCRE2 regex engine in text search. This enables using some advanced regex features like lookahead and backreferences. However, not all PCRE2 features are supported - only features that are also supported by JavaScript."
            )),
            deprecationMessage: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'usePCRE2Deprecated',
                "Deprecated. PCRE2 will be used automatically when using regex features that are only supported by PCRE2."
            )),
        },
        'search.actionsPosition': {
            type: 'string',
            enum: ['auto', 'right'],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.actionsPositionAuto',
                    "Position the actionbar to the right when the search view is narrow, and immediately after the content when the search view is wide."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.actionsPositionRight',
                    "Always position the actionbar to the right."
                )),
            ],
            default: 'right',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.actionsPosition',
                "Controls the positioning of the actionbar on rows in the search view."
            ))
        },
        'search.searchOnType': {
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.searchOnType',
                "Search all files as you type."
            ))
        },
        'search.seedWithNearestWord': {
            type: 'boolean',
            default: false,
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.seedWithNearestWord',
                "Enable seeding search from the word nearest the cursor when the active editor has no selection."
            ))
        },
        'search.seedOnFocus': {
            type: 'boolean',
            default: false,
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.seedOnFocus',
                "Update the search query to the editor's selected text when focusing the search view. This happens either on click or when triggering the `workbench.views.search.focus` command."
            ))
        },
        'search.searchOnTypeDebouncePeriod': {
            type: 'number',
            default: 300,
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.searchOnTypeDebouncePeriod',
                "When {0} is enabled, controls the timeout in milliseconds between a character being typed and the search starting. Has no effect when {0} is disabled.",
                '`#search.searchOnType#`'
            ))
        },
        'search.searchEditor.doubleClickBehaviour': {
            type: 'string',
            enum: ['selectWord', 'goToLocation', 'openLocationToSide'],
            default: 'goToLocation',
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.searchEditor.doubleClickBehaviour.selectWord',
                    "Double-clicking selects the word under the cursor."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.searchEditor.doubleClickBehaviour.goToLocation',
                    "Double-clicking opens the result in the active editor group."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.searchEditor.doubleClickBehaviour.openLocationToSide',
                    "Double-clicking opens the result in the editor group to the side, creating one if it does not yet exist."
                )),
            ],
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.searchEditor.doubleClickBehaviour',
                "Configure effect of double-clicking a result in a search editor."
            ))
        },
        'search.searchEditor.singleClickBehaviour': {
            type: 'string',
            enum: ['default', 'peekDefinition',],
            default: 'default',
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.searchEditor.singleClickBehaviour.default',
                    "Single-clicking does nothing."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'search.searchEditor.singleClickBehaviour.peekDefinition',
                    "Single-clicking opens a Peek Definition window."
                )),
            ],
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.searchEditor.singleClickBehaviour',
                "Configure effect of single-clicking a result in a search editor."
            ))
        },
        'search.searchEditor.reusePriorSearchConfiguration': {
            type: 'boolean',
            default: false,
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                { key: 'search.searchEditor.reusePriorSearchConfiguration', comment: ['"Search Editor" is a type of editor that can display search results. "includes, excludes, and flags" refers to the "files to include" and "files to exclude" input boxes, and the flags that control whether a query is case-sensitive or a regex.'] },
                "When enabled, new Search Editors will reuse the includes, excludes, and flags of the previously opened Search Editor."
            ))
        },
        'search.searchEditor.defaultNumberOfContextLines': {
            type: ['number', 'null'],
            default: 1,
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.searchEditor.defaultNumberOfContextLines',
                "The default number of surrounding context lines to use when creating new Search Editors. If using `#search.searchEditor.reusePriorSearchConfiguration#`, this can be set to `null` (empty) to use the prior Search Editor's configuration."
            ))
        },
        'search.sortOrder': {
            'type': 'string',
            'enum': ["default" , "fileNames" , "type" , "modified" , "countDescending" , "countAscending" ],
            'default': "default" ,
            'enumDescriptions': [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'searchSortOrder.default',
                    "Results are sorted by folder and file names, in alphabetical order."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'searchSortOrder.filesOnly',
                    "Results are sorted by file names ignoring folder order, in alphabetical order."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'searchSortOrder.type',
                    "Results are sorted by file extensions, in alphabetical order."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'searchSortOrder.modified',
                    "Results are sorted by file last modified date, in descending order."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'searchSortOrder.countDescending',
                    "Results are sorted by count per file, in descending order."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'searchSortOrder.countAscending',
                    "Results are sorted by count per file, in ascending order."
                ))
            ],
            'description': ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.sortOrder',
                "Controls sorting order of search results."
            ))
        },
        'search.decorations.colors': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.decorations.colors',
                "Controls whether search file decorations should use colors."
            )),
            default: true
        },
        'search.decorations.badges': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.decorations.badges',
                "Controls whether search file decorations should use badges."
            )),
            default: true
        },
        'search.defaultViewMode': {
            'type': 'string',
            'enum': ["tree" , "list" ],
            'default': "list" ,
            'enumDescriptions': [
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'scm.defaultViewMode.tree',
                    "Shows search results as a tree."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/search/browser/search.contribution',
                    'scm.defaultViewMode.list',
                    "Shows search results as a list."
                ))
            ],
            'description': ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.defaultViewMode',
                "Controls the default search result view mode."
            ))
        },
        'search.experimental.closedNotebookRichContentResults': {
            type: 'boolean',
            description: ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.experimental.closedNotebookResults',
                "Show notebook editor rich content results for closed notebooks. Please refresh your search results after changing this setting."
            )),
            default: false
        },
        'search.experimental.quickAccess.preserveInput': {
            'type': 'boolean',
            'description': ( localizeWithPath(
                'vs/workbench/contrib/search/browser/search.contribution',
                'search.experimental.quickAccess.preserveInput',
                "Controls whether the last typed input to Quick Search should be restored when opening it the next time."
            )),
            'default': false
        },
    }
});
CommandsRegistry.registerCommand('_executeWorkspaceSymbolProvider', async function (accessor, ...args) {
    const [query] = args;
    assertType(typeof query === 'string');
    const result = await getWorkspaceSymbols(query);
    return ( result.map(item => item.symbol));
});
