import { ContextScopedFindInput } from '../../../../platform/history/browser/contextScopedHistoryWidget.js';
import { NotebookFindInputFilterButton } from '../../notebook/browser/contrib/find/notebookFindReplaceWidget.js';
import { localizeWithPath } from '../../../../nls.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { getDefaultHoverDelegate } from '../../../../base/browser/ui/hover/hoverDelegateFactory.js';
import { Toggle } from '../../../../base/browser/ui/toggle/toggle.js';
import { Emitter } from '../../../../base/common/event.js';
const NLS_AI_TOGGLE_LABEL = ( localizeWithPath(
    'vs/workbench/contrib/search/browser/searchFindInput',
    'aiDescription',
    "Use AI"
));
class SearchFindInput extends ContextScopedFindInput {
    constructor(container, contextViewProvider, options, contextKeyService, contextMenuService, instantiationService, filters, _shouldShowAIButton,
    filterStartVisiblitity) {
        super(container, contextViewProvider, options, contextKeyService);
        this.contextMenuService = contextMenuService;
        this.instantiationService = instantiationService;
        this.filters = filters;
        this._shouldShowAIButton = _shouldShowAIButton;
        this._filterChecked = false;
        this._visible = false;
        this._onDidChangeAIToggle = this._register(( new Emitter()));
        this.onDidChangeAIToggle = this._onDidChangeAIToggle.event;
        this._findFilter = this._register(( new NotebookFindInputFilterButton(filters, contextMenuService, instantiationService, options, ( localizeWithPath(
            'vs/workbench/contrib/search/browser/searchFindInput',
            'searchFindInputNotebookFilter.label',
            "Notebook Find Filters"
        )))));
        this._aiButton = this._register(( new AIToggle({
            appendTitle: '',
            isChecked: false,
            ...options.toggleStyles
        })));
        this.setAdditionalToggles([this._aiButton]);
        this.inputBox.paddingRight = (this.caseSensitive?.width() ?? 0) + (this.wholeWords?.width() ?? 0) + (this.regex?.width() ?? 0) + this._findFilter.width;
        this.controls.appendChild(this._findFilter.container);
        this._findFilter.container.classList.add('monaco-custom-toggle');
        this.filterVisible = filterStartVisiblitity;
        this._register(this._aiButton.onChange(() => {
            if (this._aiButton.checked) {
                this.regex?.disable();
                this.wholeWords?.disable();
                this.caseSensitive?.disable();
                this._findFilter.disable();
            }
            else {
                this.regex?.enable();
                this.wholeWords?.enable();
                this.caseSensitive?.enable();
                this._findFilter.enable();
            }
        }));
        this._aiButton.domNode.style.display = _shouldShowAIButton ? '' : 'none';
    }
    set shouldShowAIButton(visible) {
        if (this._shouldShowAIButton !== visible) {
            this._shouldShowAIButton = visible;
            this._aiButton.domNode.style.display = visible ? '' : 'none';
        }
    }
    set filterVisible(visible) {
        this._findFilter.container.style.display = visible ? '' : 'none';
        this._visible = visible;
        this.updateStyles();
    }
    setEnabled(enabled) {
        super.setEnabled(enabled);
        if (enabled && (!this._filterChecked || !this._visible)) {
            this.regex?.enable();
        }
        else {
            this.regex?.disable();
        }
    }
    updateStyles() {
        this._filterChecked =
            !this.filters.markupInput ||
                !this.filters.markupPreview ||
                !this.filters.codeInput ||
                !this.filters.codeOutput;
        this._findFilter.applyStyles(this._filterChecked);
    }
    get isAIEnabled() {
        return this._aiButton.checked;
    }
}
class AIToggle extends Toggle {
    constructor(opts) {
        super({
            icon: Codicon.sparkle,
            title: NLS_AI_TOGGLE_LABEL + opts.appendTitle,
            isChecked: opts.isChecked,
            hoverDelegate: opts.hoverDelegate ?? getDefaultHoverDelegate('element'),
            inputActiveOptionBorder: opts.inputActiveOptionBorder,
            inputActiveOptionForeground: opts.inputActiveOptionForeground,
            inputActiveOptionBackground: opts.inputActiveOptionBackground
        });
    }
}
export { SearchFindInput };
