import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { isEqual, extname } from '../../../../base/common/resources.js';
import { URI } from '../../../../base/common/uri.js';
import { ToggleCaseSensitiveKeybinding, ToggleWholeWordKeybinding, ToggleRegexKeybinding } from '../../../../editor/contrib/find/browser/findModel.js';
import { localizeWithPath, localize2WithPath } from '../../../../nls.js';
import { registerAction2, Action2, MenuId } from '../../../../platform/actions/common/actions.js';
import { CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { IContextKeyService, ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { EditorPaneDescriptor } from '../../../browser/editor.js';
import { registerWorkbenchContribution2 } from '../../../common/contributions.js';
import { EditorExtensions, DEFAULT_EDITOR_ASSOCIATION } from '../../../common/editor.js';
import { ActiveEditorContext } from '../../../common/contextkeys.js';
import { IViewsService } from '../../../services/views/common/viewsService.js';
import { getSearchView } from '../../search/browser/searchActionsBase.js';
import { searchRefreshIcon, searchNewEditorIcon } from '../../search/browser/searchIcons.js';
import { SearchContext } from '../../search/common/constants.js';
import { SearchEditorScheme, InSearchEditor, OpenNewEditorCommandId, OpenEditorCommandId, SearchEditorID, ToggleSearchEditorContextLinesCommandId } from './constants.js';
import { SearchEditor } from './searchEditor.js';
import { openNewSearchEditor, createEditorFromSearchResult, toggleSearchEditorCaseSensitiveCommand, toggleSearchEditorWholeWordCommand, toggleSearchEditorRegexCommand, toggleSearchEditorContextLinesCommand, modifySearchEditorContextLinesCommand, selectAllSearchEditorMatchesCommand, openSearchEditor } from './searchEditorActions.js';
import { SearchEditorInput, getOrMakeSearchEditorInput, SEARCH_EDITOR_EXT } from './searchEditorInput.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { VIEW_ID } from '../../../services/search/common/search.js';
import { RegisteredEditorPriority, IEditorResolverService } from '../../../services/editor/common/editorResolverService.js';
import { IWorkingCopyEditorService } from '../../../services/workingCopy/common/workingCopyEditorService.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { getActiveElement } from '../../../../base/browser/dom.js';
const OpenInEditorCommandId = 'search.action.openInEditor';
const OpenNewEditorToSideCommandId = 'search.action.openNewEditorToSide';
const FocusQueryEditorWidgetCommandId = 'search.action.focusQueryEditorWidget';
const FocusQueryEditorFilesToIncludeCommandId = 'search.action.focusFilesToInclude';
const FocusQueryEditorFilesToExcludeCommandId = 'search.action.focusFilesToExclude';
const ToggleSearchEditorCaseSensitiveCommandId = 'toggleSearchEditorCaseSensitive';
const ToggleSearchEditorWholeWordCommandId = 'toggleSearchEditorWholeWord';
const ToggleSearchEditorRegexCommandId = 'toggleSearchEditorRegex';
const IncreaseSearchEditorContextLinesCommandId = 'increaseSearchEditorContextLines';
const DecreaseSearchEditorContextLinesCommandId = 'decreaseSearchEditorContextLines';
const RerunSearchEditorSearchCommandId = 'rerunSearchEditorSearch';
const CleanSearchEditorStateCommandId = 'cleanSearchEditorState';
const SelectAllSearchEditorMatchesCommandId = 'selectAllSearchEditorMatches';
( Registry.as(EditorExtensions.EditorPane)).registerEditorPane(EditorPaneDescriptor.create(SearchEditor, SearchEditor.ID, ( localizeWithPath(
    'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
    'searchEditor',
    "Search Editor"
))), [
    ( new SyncDescriptor(SearchEditorInput))
]);
let SearchEditorContribution = class SearchEditorContribution {
    static { this.ID = 'workbench.contrib.searchEditor'; }
    constructor(editorResolverService, instantiationService) {
        editorResolverService.registerEditor('*' + SEARCH_EDITOR_EXT, {
            id: SearchEditorInput.ID,
            label: ( localizeWithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'promptOpenWith.searchEditor.displayName',
                "Search Editor"
            )),
            detail: DEFAULT_EDITOR_ASSOCIATION.providerDisplayName,
            priority: RegisteredEditorPriority.default,
        }, {
            singlePerResource: true,
            canSupportResource: resource => (extname(resource) === SEARCH_EDITOR_EXT)
        }, {
            createEditorInput: ({ resource }) => {
                return { editor: instantiationService.invokeFunction(getOrMakeSearchEditorInput, { from: 'existingFile', fileUri: resource }) };
            }
        });
    }
};
SearchEditorContribution = ( __decorate([
    ( __param(0, IEditorResolverService)),
    ( __param(1, IInstantiationService))
], SearchEditorContribution));
registerWorkbenchContribution2(SearchEditorContribution.ID, SearchEditorContribution, 1 );
class SearchEditorInputSerializer {
    canSerialize(input) {
        return !!input.tryReadConfigSync();
    }
    serialize(input) {
        if (!this.canSerialize(input)) {
            return undefined;
        }
        if (input.isDisposed()) {
            return JSON.stringify({ modelUri: undefined, dirty: false, config: input.tryReadConfigSync(), name: input.getName(), matchRanges: [], backingUri: input.backingUri?.toString() });
        }
        let modelUri = undefined;
        if (input.modelUri.path || input.modelUri.fragment && input.isDirty()) {
            modelUri = ( input.modelUri.toString());
        }
        const config = input.tryReadConfigSync();
        const dirty = input.isDirty();
        const matchRanges = dirty ? input.getMatchRanges() : [];
        const backingUri = input.backingUri;
        return JSON.stringify({ modelUri, dirty, config, name: input.getName(), matchRanges, backingUri: backingUri?.toString() });
    }
    deserialize(instantiationService, serializedEditorInput) {
        const { modelUri, dirty, config, matchRanges, backingUri } = JSON.parse(serializedEditorInput);
        if (config && (config.query !== undefined)) {
            if (modelUri) {
                const input = instantiationService.invokeFunction(getOrMakeSearchEditorInput, { from: 'model', modelUri: ( URI.parse(modelUri)), config, backupOf: backingUri ? ( URI.parse(backingUri)) : undefined });
                input.setDirty(dirty);
                input.setMatchRanges(matchRanges);
                return input;
            }
            else {
                if (backingUri) {
                    return instantiationService.invokeFunction(getOrMakeSearchEditorInput, { from: 'existingFile', fileUri: ( URI.parse(backingUri)) });
                }
                else {
                    return instantiationService.invokeFunction(getOrMakeSearchEditorInput, { from: 'rawData', resultsContents: '', config });
                }
            }
        }
        return undefined;
    }
}
( Registry.as(EditorExtensions.EditorFactory)).registerEditorSerializer(SearchEditorInput.ID, SearchEditorInputSerializer);
CommandsRegistry.registerCommand(CleanSearchEditorStateCommandId, (accessor) => {
    const activeEditorPane = accessor.get(IEditorService).activeEditorPane;
    if (activeEditorPane instanceof SearchEditor) {
        activeEditorPane.cleanState();
    }
});
const category = ( localize2WithPath(
    'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
    'search',
    'Search Editor'
));
const translateLegacyConfig = (legacyConfig = {}) => {
    const config = {};
    const overrides = {
        includes: 'filesToInclude',
        excludes: 'filesToExclude',
        wholeWord: 'matchWholeWord',
        caseSensitive: 'isCaseSensitive',
        regexp: 'isRegexp',
        useIgnores: 'useExcludeSettingsAndIgnoreFiles',
    };
    Object.entries(legacyConfig).forEach(([key, value]) => {
        config[overrides[key] ?? key] = value;
    });
    return config;
};
const openArgMetadata = {
    description: 'Open a new search editor. Arguments passed can include variables like ${relativeFileDirname}.',
    args: [{
            name: 'Open new Search Editor args',
            schema: {
                properties: {
                    query: { type: 'string' },
                    filesToInclude: { type: 'string' },
                    filesToExclude: { type: 'string' },
                    contextLines: { type: 'number' },
                    matchWholeWord: { type: 'boolean' },
                    isCaseSensitive: { type: 'boolean' },
                    isRegexp: { type: 'boolean' },
                    useExcludeSettingsAndIgnoreFiles: { type: 'boolean' },
                    showIncludesExcludes: { type: 'boolean' },
                    triggerSearch: { type: 'boolean' },
                    focusResults: { type: 'boolean' },
                    onlyOpenEditors: { type: 'boolean' },
                }
            }
        }]
};
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'search.searchEditor.action.deleteFileResults',
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.deleteResultBlock',
                'Delete File Results'
            )),
            keybinding: {
                weight: 100 ,
                primary: 2048  | 1024  | 1 ,
            },
            precondition: InSearchEditor,
            category,
            f1: true,
        });
    }
    async run(accessor) {
        const contextService = accessor.get(IContextKeyService).getContext(getActiveElement());
        if (contextService.getValue(InSearchEditor.serialize())) {
            accessor.get(IEditorService).activeEditorPane.deleteResultBlock();
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: OpenNewEditorCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.openNewSearchEditor',
                'New Search Editor'
            )),
            category,
            f1: true,
            metadata: openArgMetadata
        });
    }
    async run(accessor, args) {
        await accessor.get(IInstantiationService).invokeFunction(openNewSearchEditor, translateLegacyConfig({ location: 'new', ...args }));
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: OpenEditorCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.openSearchEditor',
                'Open Search Editor'
            )),
            category,
            f1: true,
            metadata: openArgMetadata
        });
    }
    async run(accessor, args) {
        await accessor.get(IInstantiationService).invokeFunction(openNewSearchEditor, translateLegacyConfig({ location: 'reuse', ...args }));
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: OpenNewEditorToSideCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.openNewEditorToSide',
                'Open New Search Editor to the Side'
            )),
            category,
            f1: true,
            metadata: openArgMetadata
        });
    }
    async run(accessor, args) {
        await accessor.get(IInstantiationService).invokeFunction(openNewSearchEditor, translateLegacyConfig(args), true);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: OpenInEditorCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.openResultsInEditor',
                'Open Results in Editor'
            )),
            category,
            f1: true,
            keybinding: {
                primary: 512  | 3 ,
                when: ( ContextKeyExpr.and(
                    SearchContext.HasSearchResults,
                    SearchContext.SearchViewFocusedKey
                )),
                weight: 200 ,
                mac: {
                    primary: 2048  | 3
                }
            },
        });
    }
    async run(accessor) {
        const viewsService = accessor.get(IViewsService);
        const instantiationService = accessor.get(IInstantiationService);
        const searchView = getSearchView(viewsService);
        if (searchView) {
            await instantiationService.invokeFunction(createEditorFromSearchResult, searchView.searchResult, searchView.searchIncludePattern.getValue(), searchView.searchExcludePattern.getValue(), searchView.searchIncludePattern.onlySearchInOpenEditors());
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: RerunSearchEditorSearchCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.rerunSearchInEditor',
                'Search Again'
            )),
            category,
            keybinding: {
                primary: 2048  | 1024  | 48 ,
                when: InSearchEditor,
                weight: 100
            },
            icon: searchRefreshIcon,
            menu: [{
                    id: MenuId.EditorTitle,
                    group: 'navigation',
                    when: ( ActiveEditorContext.isEqualTo(SearchEditorID))
                },
                {
                    id: MenuId.CommandPalette,
                    when: ( ActiveEditorContext.isEqualTo(SearchEditorID))
                }]
        });
    }
    async run(accessor) {
        const editorService = accessor.get(IEditorService);
        const input = editorService.activeEditor;
        if (input instanceof SearchEditorInput) {
            editorService.activeEditorPane.triggerSearch({ resetCursor: false });
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: FocusQueryEditorWidgetCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.action.focusQueryEditorWidget',
                'Focus Search Editor Input'
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: {
                primary: 9 ,
                weight: 100
            }
        });
    }
    async run(accessor) {
        const editorService = accessor.get(IEditorService);
        const input = editorService.activeEditor;
        if (input instanceof SearchEditorInput) {
            editorService.activeEditorPane.focusSearchInput();
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: FocusQueryEditorFilesToIncludeCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.action.focusFilesToInclude',
                'Focus Search Editor Files to Include'
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
        });
    }
    async run(accessor) {
        const editorService = accessor.get(IEditorService);
        const input = editorService.activeEditor;
        if (input instanceof SearchEditorInput) {
            editorService.activeEditorPane.focusFilesToIncludeInput();
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: FocusQueryEditorFilesToExcludeCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.action.focusFilesToExclude',
                'Focus Search Editor Files to Exclude'
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
        });
    }
    async run(accessor) {
        const editorService = accessor.get(IEditorService);
        const input = editorService.activeEditor;
        if (input instanceof SearchEditorInput) {
            editorService.activeEditorPane.focusFilesToExcludeInput();
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: ToggleSearchEditorCaseSensitiveCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.toggleSearchEditorCaseSensitive',
                'Toggle Match Case'
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: Object.assign({
                weight: 200 ,
                when: SearchContext.SearchInputBoxFocusedKey,
            }, ToggleCaseSensitiveKeybinding)
        });
    }
    run(accessor) {
        toggleSearchEditorCaseSensitiveCommand(accessor);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: ToggleSearchEditorWholeWordCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.toggleSearchEditorWholeWord',
                'Toggle Match Whole Word'
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: Object.assign({
                weight: 200 ,
                when: SearchContext.SearchInputBoxFocusedKey,
            }, ToggleWholeWordKeybinding)
        });
    }
    run(accessor) {
        toggleSearchEditorWholeWordCommand(accessor);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: ToggleSearchEditorRegexCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.toggleSearchEditorRegex',
                "Toggle Use Regular Expression"
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: Object.assign({
                weight: 200 ,
                when: SearchContext.SearchInputBoxFocusedKey,
            }, ToggleRegexKeybinding)
        });
    }
    run(accessor) {
        toggleSearchEditorRegexCommand(accessor);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: ToggleSearchEditorContextLinesCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.toggleSearchEditorContextLines',
                "Toggle Context Lines"
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: {
                weight: 200 ,
                primary: 512  | 42 ,
                mac: { primary: 2048  | 512  | 42  }
            }
        });
    }
    run(accessor) {
        toggleSearchEditorContextLinesCommand(accessor);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: IncreaseSearchEditorContextLinesCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.increaseSearchEditorContextLines',
                "Increase Context Lines"
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: {
                weight: 200 ,
                primary: 512  | 86
            }
        });
    }
    run(accessor) { modifySearchEditorContextLinesCommand(accessor, true); }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: DecreaseSearchEditorContextLinesCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.decreaseSearchEditorContextLines',
                "Decrease Context Lines"
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: {
                weight: 200 ,
                primary: 512  | 88
            }
        });
    }
    run(accessor) { modifySearchEditorContextLinesCommand(accessor, false); }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: SelectAllSearchEditorMatchesCommandId,
            title: ( localize2WithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'searchEditor.action.selectAllSearchEditorMatches',
                "Select All Matches"
            )),
            category,
            f1: true,
            precondition: InSearchEditor,
            keybinding: {
                weight: 200 ,
                primary: 2048  | 1024  | 42 ,
            }
        });
    }
    run(accessor) {
        selectAllSearchEditorMatchesCommand(accessor);
    }
});
registerAction2(class OpenSearchEditorAction extends Action2 {
    constructor() {
        super({
            id: 'search.action.openNewEditorFromView',
            title: ( localizeWithPath(
                'vs/workbench/contrib/searchEditor/browser/searchEditor.contribution',
                'search.openNewEditor',
                "Open New Search Editor"
            )),
            category,
            icon: searchNewEditorIcon,
            menu: [{
                    id: MenuId.ViewTitle,
                    group: 'navigation',
                    order: 2,
                    when: ( ContextKeyExpr.equals('view', VIEW_ID)),
                }]
        });
    }
    run(accessor, ...args) {
        return openSearchEditor(accessor);
    }
});
let SearchEditorWorkingCopyEditorHandler = class SearchEditorWorkingCopyEditorHandler extends Disposable {
    static { this.ID = 'workbench.contrib.searchEditorWorkingCopyEditorHandler'; }
    constructor(instantiationService, workingCopyEditorService) {
        super();
        this.instantiationService = instantiationService;
        this._register(workingCopyEditorService.registerHandler(this));
    }
    handles(workingCopy) {
        return workingCopy.resource.scheme === SearchEditorScheme;
    }
    isOpen(workingCopy, editor) {
        if (!this.handles(workingCopy)) {
            return false;
        }
        return editor instanceof SearchEditorInput && isEqual(workingCopy.resource, editor.modelUri);
    }
    createEditor(workingCopy) {
        const input = this.instantiationService.invokeFunction(getOrMakeSearchEditorInput, { from: 'model', modelUri: workingCopy.resource });
        input.setDirty(true);
        return input;
    }
};
SearchEditorWorkingCopyEditorHandler = ( __decorate([
    ( __param(0, IInstantiationService)),
    ( __param(1, IWorkingCopyEditorService))
], SearchEditorWorkingCopyEditorHandler));
registerWorkbenchContribution2(SearchEditorWorkingCopyEditorHandler.ID, SearchEditorWorkingCopyEditorHandler, 2 );
