import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import './share.css.js';
import { CancellationToken } from '../../../../base/common/cancellation.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { MarkdownString } from '../../../../base/common/htmlContent.js';
import { localize2WithPath, localizeWithPath } from '../../../../nls.js';
import { registerAction2, Action2, MenuId, MenuRegistry } from '../../../../platform/actions/common/actions.js';
import { IClipboardService } from '../../../../platform/clipboard/common/clipboardService.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { EditorResourceAccessor, SideBySideEditor } from '../../../common/editor.js';
import '../../../../../../../override/vs/platform/dialogs/common/dialogs.js';
import { Severity } from '../../../../platform/notification/common/notification.js';
import { IOpenerService } from '../../../../platform/opener/common/opener.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { IWorkspaceContextService } from '../../../../platform/workspace/common/workspace.js';
import { WorkspaceFolderCountContext } from '../../../common/contextkeys.js';
import { Extensions as Extensions$1 } from '../../../common/contributions.js';
import { ShareProviderCountContext } from './shareService.js';
import { IShareService } from '../common/share.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { IProgressService } from '../../../../platform/progress/common/progress.js';
import { ICodeEditorService } from '../../../../editor/browser/services/codeEditorService.js';
import { Extensions } from '../../../../platform/configuration/common/configurationRegistry.js';
import { workbenchConfigurationNodeBase } from '../../../common/configuration.js';
import { DisposableStore } from '../../../../base/common/lifecycle.js';
import { IDialogService } from '../../../../platform/dialogs/common/dialogs.js';
var ShareWorkbenchContribution_1;
const targetMenus = [
    MenuId.EditorContextShare,
    MenuId.SCMResourceContextShare,
    MenuId.OpenEditorsContextShare,
    MenuId.EditorTitleContextShare,
    MenuId.MenubarShare,
    MenuId.ExplorerContextShare
];
let ShareWorkbenchContribution = class ShareWorkbenchContribution {
    static { ShareWorkbenchContribution_1 = this; }
    static { this.SHARE_ENABLED_SETTING = 'workbench.experimental.share.enabled'; }
    constructor(shareService, configurationService) {
        this.shareService = shareService;
        this.configurationService = configurationService;
        if (this.configurationService.getValue(ShareWorkbenchContribution_1.SHARE_ENABLED_SETTING)) {
            this.registerActions();
        }
        this.configurationService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration(ShareWorkbenchContribution_1.SHARE_ENABLED_SETTING)) {
                const settingValue = this.configurationService.getValue(ShareWorkbenchContribution_1.SHARE_ENABLED_SETTING);
                if (settingValue === true && this._disposables === undefined) {
                    this.registerActions();
                }
                else if (settingValue === false && this._disposables !== undefined) {
                    this._disposables?.clear();
                    this._disposables = undefined;
                }
            }
        });
    }
    registerActions() {
        if (!this._disposables) {
            this._disposables = ( new DisposableStore());
        }
        this._disposables.add(registerAction2(class ShareAction extends Action2 {
            static { this.ID = 'workbench.action.share'; }
            static { this.LABEL = ( localize2WithPath(
                'vs/workbench/contrib/share/browser/share.contribution',
                'share',
                'Share...'
            )); }
            constructor() {
                super({
                    id: ShareAction.ID,
                    title: ShareAction.LABEL,
                    f1: true,
                    icon: Codicon.linkExternal,
                    precondition: ( ContextKeyExpr.and(( ShareProviderCountContext.notEqualsTo(0)), ( WorkspaceFolderCountContext.notEqualsTo(0)))),
                    keybinding: {
                        weight: 200 ,
                        primary: 512  | 2048  | 49 ,
                    },
                    menu: [
                        { id: MenuId.CommandCenter, order: 1000 }
                    ]
                });
            }
            async run(accessor, ...args) {
                const shareService = accessor.get(IShareService);
                const activeEditor = accessor.get(IEditorService)?.activeEditor;
                const resourceUri = (activeEditor && EditorResourceAccessor.getOriginalUri(activeEditor, { supportSideBySide: SideBySideEditor.PRIMARY }))
                    ?? accessor.get(IWorkspaceContextService).getWorkspace().folders[0].uri;
                const clipboardService = accessor.get(IClipboardService);
                const dialogService = accessor.get(IDialogService);
                const urlService = accessor.get(IOpenerService);
                const progressService = accessor.get(IProgressService);
                const selection = accessor.get(ICodeEditorService).getActiveCodeEditor()?.getSelection() ?? undefined;
                const result = await progressService.withProgress({
                    location: 10 ,
                    detail: ( localizeWithPath(
                        'vs/workbench/contrib/share/browser/share.contribution',
                        'generating link',
                        'Generating link...'
                    ))
                }, async () => shareService.provideShare({ resourceUri, selection }, CancellationToken.None));
                if (result) {
                    const uriText = ( result.toString());
                    const isResultText = typeof result === 'string';
                    await clipboardService.writeText(uriText);
                    dialogService.prompt({
                        type: Severity.Info,
                        message: isResultText ? ( localizeWithPath(
                            'vs/workbench/contrib/share/browser/share.contribution',
                            'shareTextSuccess',
                            'Copied text to clipboard!'
                        )) : ( localizeWithPath(
                            'vs/workbench/contrib/share/browser/share.contribution',
                            'shareSuccess',
                            'Copied link to clipboard!'
                        )),
                        custom: {
                            icon: Codicon.check,
                            markdownDetails: [{
                                    markdown: ( new MarkdownString(`<div aria-label='${uriText}'>${uriText}</div>`, { supportHtml: true })),
                                    classes: [isResultText ? 'share-dialog-input-text' : 'share-dialog-input-link']
                                }]
                        },
                        cancelButton: ( localizeWithPath('vs/workbench/contrib/share/browser/share.contribution', 'close', 'Close')),
                        buttons: isResultText ? [] : [{ label: ( localizeWithPath(
                            'vs/workbench/contrib/share/browser/share.contribution',
                            'open link',
                            'Open Link'
                        )), run: () => { urlService.open(result, { openExternal: true }); } }]
                    });
                }
            }
        }));
        const actions = this.shareService.getShareActions();
        for (const menuId of targetMenus) {
            for (const action of actions) {
                this._disposables.add(MenuRegistry.appendMenuItem(menuId, action));
            }
        }
    }
};
ShareWorkbenchContribution = ShareWorkbenchContribution_1 = ( __decorate([
    ( __param(0, IShareService)),
    ( __param(1, IConfigurationService))
], ShareWorkbenchContribution));
const workbenchContributionsRegistry = ( Registry.as(Extensions$1.Workbench));
workbenchContributionsRegistry.registerWorkbenchContribution(ShareWorkbenchContribution, 4 );
( Registry.as(Extensions.Configuration)).registerConfiguration({
    ...workbenchConfigurationNodeBase,
    properties: {
        'workbench.experimental.share.enabled': {
            type: 'boolean',
            default: false,
            tags: ['experimental'],
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/share/browser/share.contribution',
                'experimental.share.enabled',
                "Controls whether to render the Share action next to the command center when {0} is {1}.",
                '`#window.commandCenter#`',
                '`true`'
            )),
            restricted: false,
        }
    }
});
