import { localizeWithPath } from '../../../../nls.js';
import { ExecutionEngine } from '../common/tasks.js';
import { AbstractTaskService } from './abstractTaskService.js';
import '../../../../platform/instantiation/common/instantiation.js';
import '../../../../platform/contextkey/common/contextkey.js';
class TaskService extends AbstractTaskService {
    static { this.ProcessTaskSystemSupportMessage = ( localizeWithPath(
        'vs/workbench/contrib/tasks/browser/taskService',
        'taskService.processTaskSystem',
        'Process task system is not support in the web.'
    )); }
    _getTaskSystem() {
        if (this._taskSystem) {
            return this._taskSystem;
        }
        if (this.executionEngine !== ExecutionEngine.Terminal) {
            throw new Error(TaskService.ProcessTaskSystemSupportMessage);
        }
        this._taskSystem = this._createTerminalTaskSystem();
        this._taskSystemListeners =
            [
                this._taskSystem.onDidStateChange((event) => {
                    this._taskRunningState.set(this._taskSystem.isActiveSync());
                    this._onDidStateChange.fire(event);
                }),
            ];
        return this._taskSystem;
    }
    _computeLegacyConfiguration(workspaceFolder) {
        throw new Error(TaskService.ProcessTaskSystemSupportMessage);
    }
    _versionAndEngineCompatible(filter) {
        return this.executionEngine === ExecutionEngine.Terminal;
    }
}
export { TaskService };
