import { localizeWithPath } from '../../../../nls.js';
import { createDecorator } from '../../../../platform/instantiation/common/instantiation.js';
import { RawContextKey, ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
const CustomExecutionSupportedContext = ( new RawContextKey('customExecutionSupported', false, ( localizeWithPath(
 'vs/workbench/contrib/tasks/common/taskService',
 'tasks.customExecutionSupported',
 "Whether CustomExecution tasks are supported. Consider using in the when clause of a \'taskDefinition\' contribution."
))));
const ShellExecutionSupportedContext = ( new RawContextKey('shellExecutionSupported', false, ( localizeWithPath(
 'vs/workbench/contrib/tasks/common/taskService',
 'tasks.shellExecutionSupported',
 "Whether ShellExecution tasks are supported. Consider using in the when clause of a \'taskDefinition\' contribution."
))));
const TaskCommandsRegistered = ( new RawContextKey('taskCommandsRegistered', false, ( localizeWithPath(
 'vs/workbench/contrib/tasks/common/taskService',
 'tasks.taskCommandsRegistered',
 "Whether the task commands have been registered yet"
))));
const ProcessExecutionSupportedContext = ( new RawContextKey('processExecutionSupported', false, ( localizeWithPath(
 'vs/workbench/contrib/tasks/common/taskService',
 'tasks.processExecutionSupported',
 "Whether ProcessExecution tasks are supported. Consider using in the when clause of a \'taskDefinition\' contribution."
))));
const ServerlessWebContext = ( new RawContextKey('serverlessWebContext', false, ( localizeWithPath(
 'vs/workbench/contrib/tasks/common/taskService',
 'tasks.serverlessWebContext',
 "True when in the web with no remote authority."
))));
const TaskExecutionSupportedContext = ( ContextKeyExpr.or(( ContextKeyExpr.and(ShellExecutionSupportedContext, ProcessExecutionSupportedContext)), CustomExecutionSupportedContext));
const ITaskService = ( createDecorator('taskService'));
export { CustomExecutionSupportedContext, ITaskService, ProcessExecutionSupportedContext, ServerlessWebContext, ShellExecutionSupportedContext, TaskCommandsRegistered, TaskExecutionSupportedContext };
