import { __decorate, __param } from '../../../../../../../../external/tslib/tslib.es6.js';
import { Disposable } from '../../../../../base/common/lifecycle.js';
import { format } from '../../../../../base/common/strings.js';
import { localizeWithPath } from '../../../../../nls.js';
import { IAccessibilityService } from '../../../../../platform/accessibility/common/accessibility.js';
import { ICommandService } from '../../../../../platform/commands/common/commands.js';
import { ContextKeyExpr, IContextKeyService } from '../../../../../platform/contextkey/common/contextkey.js';
import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
import { IKeybindingService } from '../../../../../platform/keybinding/common/keybinding.js';
import { accessibleViewIsShown, accessibleViewCurrentProviderId } from '../../../accessibility/browser/accessibilityConfiguration.js';
import { IConfigurationService } from '../../../../../platform/configuration/common/configuration.js';
let TerminalAccessibilityHelpProvider = class TerminalAccessibilityHelpProvider extends Disposable {
    onClose() {
        const expr = ( ContextKeyExpr.and(accessibleViewIsShown, ( ContextKeyExpr.equals(
            accessibleViewCurrentProviderId.key,
            "terminal-help"
        ))));
        if (expr?.evaluate(this._contextKeyService.getContext(null))) {
            this._commandService.executeCommand("workbench.action.terminal.focusAccessibleBuffer" );
        }
        else {
            this._instance.focus();
        }
        this.dispose();
    }
    constructor(_instance, _xterm, _instantiationService, _keybindingService, _contextKeyService, _commandService, _accessibilityService, _configurationService) {
        super();
        this._instance = _instance;
        this._keybindingService = _keybindingService;
        this._contextKeyService = _contextKeyService;
        this._commandService = _commandService;
        this._accessibilityService = _accessibilityService;
        this._configurationService = _configurationService;
        this.id = "terminal-help" ;
        this._hasShellIntegration = false;
        this.options = {
            type: "help" ,
            readMoreUrl: 'https://code.visualstudio.com/docs/editor/accessibility#_terminal-accessibility'
        };
        this.verbositySettingKey = "accessibility.verbosity.terminal" ;
        this._hasShellIntegration = _xterm.shellIntegration.status === 2 ;
    }
    _descriptionForCommand(commandId, msg, noKbMsg) {
        if (commandId === "workbench.action.terminal.runRecentCommand" ) {
            const kb = this._keybindingService.lookupKeybindings(commandId);
            const isScreenReaderOptimized = this._accessibilityService.isScreenReaderOptimized();
            if (isScreenReaderOptimized && kb[1]) {
                format(msg, kb[1].getAriaLabel());
            }
            else if (kb[0]) {
                format(msg, kb[0].getAriaLabel());
            }
            else {
                return format(noKbMsg, commandId);
            }
        }
        const kb = this._keybindingService.lookupKeybinding(commandId, this._contextKeyService)?.getAriaLabel();
        return !kb ? format(noKbMsg, commandId) : format(msg, kb);
    }
    provideContent() {
        const content = [];
        content.push(this._descriptionForCommand("workbench.action.terminal.focusAccessibleBuffer" , ( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'focusAccessibleTerminalView',
            'The Focus Accessible Terminal View ({0}) command enables screen readers to read terminal contents.'
        )), ( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'focusAccessibleTerminalViewNoKb',
            'The Focus Terminal Accessible View command enables screen readers to read terminal contents and is currently not triggerable by a keybinding.'
        ))));
        content.push(( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'preserveCursor',
            'Customize the behavior of the cursor when toggling between the terminal and accessible view with `terminal.integrated.accessibleViewPreserveCursorPosition.`'
        )));
        if (!this._configurationService.getValue("terminal.integrated.accessibleViewFocusOnCommandExecution" )) {
            content.push(( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'focusViewOnExecution',
                'Enable `terminal.integrated.accessibleViewFocusOnCommandExecution` to automatically focus the terminal accessible view when a command is executed in the terminal.'
            )));
        }
        if (this._instance.shellType === "cmd" ) {
            content.push(( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'commandPromptMigration',
                "Consider using powershell instead of command prompt for an improved experience"
            )));
        }
        if (this._hasShellIntegration) {
            const shellIntegrationCommandList = [];
            shellIntegrationCommandList.push(( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'shellIntegration',
                "The terminal has a feature called shell integration that offers an enhanced experience and provides useful commands for screen readers such as:"
            )));
            shellIntegrationCommandList.push('- ' + this._descriptionForCommand("workbench.action.terminal.accessibleBufferGoToNextCommand" , ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToNextCommand',
                'Go to Next Command ({0}) in the accessible view'
            )), ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToNextCommandNoKb',
                'Go to Next Command in the accessible view is currently not triggerable by a keybinding.'
            ))));
            shellIntegrationCommandList.push('- ' + this._descriptionForCommand("workbench.action.terminal.accessibleBufferGoToPreviousCommand" , ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToPreviousCommand',
                'Go to Previous Command ({0}) in the accessible view'
            )), ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToPreviousCommandNoKb',
                'Go to Previous Command in the accessible view is currently not triggerable by a keybinding.'
            ))));
            shellIntegrationCommandList.push('- ' + this._descriptionForCommand("editor.action.accessibleViewGoToSymbol" , ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToSymbol',
                'Go to Symbol ({0})'
            )), ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToSymbolNoKb',
                'Go to symbol is currently not triggerable by a keybinding.'
            ))));
            shellIntegrationCommandList.push('- ' + this._descriptionForCommand("workbench.action.terminal.runRecentCommand" , ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'runRecentCommand',
                'Run Recent Command ({0})'
            )), ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'runRecentCommandNoKb',
                'Run Recent Command is currently not triggerable by a keybinding.'
            ))));
            shellIntegrationCommandList.push('- ' + this._descriptionForCommand("workbench.action.terminal.goToRecentDirectory" , ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToRecentDirectory',
                'Go to Recent Directory ({0})'
            )), ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToRecentDirectoryNoKb',
                'Go to Recent Directory is currently not triggerable by a keybinding.'
            ))));
            content.push(shellIntegrationCommandList.join('\n'));
        }
        else {
            content.push(this._descriptionForCommand("workbench.action.terminal.runRecentCommand" , ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToRecentDirectoryNoShellIntegration',
                'The Go to Recent Directory command ({0}) enables screen readers to easily navigate to a directory that has been used in the terminal.'
            )), ( localizeWithPath(
                'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
                'goToRecentDirectoryNoKbNoShellIntegration',
                'The Go to Recent Directory command enables screen readers to easily navigate to a directory that has been used in the terminal and is currently not triggerable by a keybinding.'
            ))));
        }
        content.push(this._descriptionForCommand("workbench.action.terminal.openDetectedLink" , ( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'openDetectedLink',
            'The Open Detected Link ({0}) command enables screen readers to easily open links found in the terminal.'
        )), ( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'openDetectedLinkNoKb',
            'The Open Detected Link command enables screen readers to easily open links found in the terminal and is currently not triggerable by a keybinding.'
        ))));
        content.push(this._descriptionForCommand("workbench.action.terminal.newWithProfile" , ( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'newWithProfile',
            'The Create New Terminal (With Profile) ({0}) command allows for easy terminal creation using a specific profile.'
        )), ( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'newWithProfileNoKb',
            'The Create New Terminal (With Profile) command allows for easy terminal creation using a specific profile and is currently not triggerable by a keybinding.'
        ))));
        content.push(( localizeWithPath(
            'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp',
            'focusAfterRun',
            'Configure what gets focused after running selected text in the terminal with `{0}`.',
            "terminal.integrated.focusAfterRun"
        )));
        return content.join('\n\n');
    }
};
TerminalAccessibilityHelpProvider = ( __decorate([
    ( __param(2, IInstantiationService)),
    ( __param(3, IKeybindingService)),
    ( __param(4, IContextKeyService)),
    ( __param(5, ICommandService)),
    ( __param(6, IAccessibilityService)),
    ( __param(7, IConfigurationService))
], TerminalAccessibilityHelpProvider));
export { TerminalAccessibilityHelpProvider };
