import { Codicon } from '../../../../../base/common/codicons.js';
import { localize2WithPath } from '../../../../../nls.js';
import { ContextKeyExpr } from '../../../../../platform/contextkey/common/contextkey.js';
import { AbstractInlineChatAction } from '../../../inlineChat/browser/inlineChatActions.js';
import { CTX_INLINE_CHAT_HAS_PROVIDER, CTX_INLINE_CHAT_FOCUSED, CTX_INLINE_CHAT_EMPTY } from '../../../inlineChat/common/inlineChat.js';
import { isDetachedTerminalInstance } from '../../../terminal/browser/terminal.js';
import { registerActiveXtermAction } from '../../../terminal/browser/terminalActions.js';
import { TerminalContextKeys } from '../../../terminal/common/terminalContextKey.js';
import { TerminalChatContextKeys, MENU_TERMINAL_CHAT_WIDGET, MENU_TERMINAL_CHAT_WIDGET_STATUS, MENU_TERMINAL_CHAT_INPUT, MENU_TERMINAL_CHAT_WIDGET_FEEDBACK } from './terminalChat.js';
import { TerminalChatController } from './terminalChatController.js';
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.start" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'startChat',
        'Start in Terminal'
    )),
    keybinding: {
        primary: 2048  | 39 ,
        when: ( ContextKeyExpr.and(TerminalContextKeys.focusInAny)),
        weight: 400  + 1,
    },
    f1: true,
    category: AbstractInlineChatAction.category,
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.or(
        TerminalContextKeys.processSupported,
        TerminalContextKeys.terminalHasBeenCreated
    )),
    CTX_INLINE_CHAT_HAS_PROVIDER)),
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.reveal();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.close" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'closeChat',
        'Close Chat'
    )),
    keybinding: {
        primary: 9 ,
        secondary: [1024  | 9 ],
        when: ( ContextKeyExpr.and(TerminalChatContextKeys.focused, TerminalChatContextKeys.visible)),
        weight: 200 ,
    },
    icon: Codicon.close,
    menu: {
        id: MENU_TERMINAL_CHAT_WIDGET,
        group: 'navigation',
        order: 2
    },
    f1: true,
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.and(TerminalChatContextKeys.focused, TerminalChatContextKeys.visible)))),
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.clear();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.focusResponse" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'focusTerminalResponse',
        'Focus Terminal Response'
    )),
    keybinding: {
        primary: 2048  | 18 ,
        when: TerminalChatContextKeys.focused,
        weight: 200 ,
    },
    f1: true,
    category: AbstractInlineChatAction.category,
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), TerminalChatContextKeys.focused)),
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.chatWidget?.inlineChatWidget.chatWidget.focusLastMessage();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.focusInput" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'focusTerminalInput',
        'Focus Terminal Input'
    )),
    keybinding: {
        primary: 2048  | 16 ,
        secondary: [2048  | 39 ],
        when: ( ContextKeyExpr.and(TerminalChatContextKeys.focused, ( CTX_INLINE_CHAT_FOCUSED.toNegated()))),
        weight: 200 ,
    },
    f1: true,
    category: AbstractInlineChatAction.category,
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), TerminalChatContextKeys.focused)),
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.chatWidget?.focus();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.discard" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'discard',
        'Discard'
    )),
    icon: Codicon.discard,
    menu: {
        id: MENU_TERMINAL_CHAT_WIDGET_STATUS,
        group: '0_main',
        order: 2,
        when: ( ContextKeyExpr.and(
            TerminalChatContextKeys.focused,
            TerminalChatContextKeys.responseContainsCodeBlock
        ))
    },
    f1: true,
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.or(
        TerminalContextKeys.processSupported,
        TerminalContextKeys.terminalHasBeenCreated
    )), TerminalChatContextKeys.focused, TerminalChatContextKeys.responseContainsCodeBlock)),
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.clear();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.runCommand" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'runCommand',
        'Run Chat Command'
    )),
    shortTitle: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'run',
        'Run'
    )),
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.or(
        TerminalContextKeys.processSupported,
        TerminalContextKeys.terminalHasBeenCreated
    )), ( TerminalChatContextKeys.requestActive.negate()), TerminalChatContextKeys.agentRegistered, TerminalChatContextKeys.responseContainsCodeBlock)),
    icon: Codicon.play,
    keybinding: {
        when: ( TerminalChatContextKeys.requestActive.negate()),
        weight: 200 ,
        primary: 2048  | 3 ,
    },
    menu: {
        id: MENU_TERMINAL_CHAT_WIDGET_STATUS,
        group: '0_main',
        order: 0,
        when: ( ContextKeyExpr.and(TerminalChatContextKeys.responseContainsCodeBlock, ( TerminalChatContextKeys.requestActive.negate())))
    },
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.acceptCommand(true);
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.insertCommand" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'insertCommand',
        'Insert Chat Command'
    )),
    shortTitle: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'insert',
        'Insert'
    )),
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.or(
        TerminalContextKeys.processSupported,
        TerminalContextKeys.terminalHasBeenCreated
    )), ( TerminalChatContextKeys.requestActive.negate()), TerminalChatContextKeys.agentRegistered, TerminalChatContextKeys.responseContainsCodeBlock)),
    keybinding: {
        when: ( TerminalChatContextKeys.requestActive.negate()),
        weight: 200 ,
        primary: 512  | 3 ,
    },
    menu: {
        id: MENU_TERMINAL_CHAT_WIDGET_STATUS,
        group: '0_main',
        order: 1,
        when: ( ContextKeyExpr.and(TerminalChatContextKeys.responseContainsCodeBlock, ( TerminalChatContextKeys.requestActive.negate())))
    },
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.acceptCommand(false);
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.viewInChat" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'viewInChat',
        'View in Chat'
    )),
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.or(
        TerminalContextKeys.processSupported,
        TerminalContextKeys.terminalHasBeenCreated
    )), ( TerminalChatContextKeys.requestActive.negate()), TerminalChatContextKeys.agentRegistered)),
    icon: Codicon.commentDiscussion,
    menu: [{
            id: MENU_TERMINAL_CHAT_WIDGET_STATUS,
            group: '0_main',
            order: 1,
            when: ( ContextKeyExpr.and(( TerminalChatContextKeys.responseContainsCodeBlock.negate()), ( TerminalChatContextKeys.requestActive.negate()))),
        },
        {
            id: MENU_TERMINAL_CHAT_WIDGET,
            group: 'navigation',
            order: 1,
            when: ( ContextKeyExpr.and(( CTX_INLINE_CHAT_EMPTY.negate()), TerminalChatContextKeys.responseContainsCodeBlock, ( TerminalChatContextKeys.requestActive.negate()))),
        }],
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.viewInChat();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.makeRequest" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'makeChatRequest',
        'Make Chat Request'
    )),
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( ContextKeyExpr.or(
        TerminalContextKeys.processSupported,
        TerminalContextKeys.terminalHasBeenCreated
    )), ( TerminalChatContextKeys.requestActive.negate()), TerminalChatContextKeys.agentRegistered, ( CTX_INLINE_CHAT_EMPTY.negate()))),
    icon: Codicon.send,
    keybinding: {
        when: ( ContextKeyExpr.and(CTX_INLINE_CHAT_FOCUSED, ( TerminalChatContextKeys.requestActive.negate()))),
        weight: 200 ,
        primary: 3
    },
    menu: {
        id: MENU_TERMINAL_CHAT_INPUT,
        group: 'navigation',
        order: 1,
        when: ( TerminalChatContextKeys.requestActive.negate()),
    },
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.acceptInput();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.cancel" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'cancelChat',
        'Cancel Chat'
    )),
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), TerminalChatContextKeys.requestActive, TerminalChatContextKeys.agentRegistered)),
    icon: Codicon.debugStop,
    menu: {
        id: MENU_TERMINAL_CHAT_INPUT,
        group: 'navigation',
        when: TerminalChatContextKeys.requestActive,
    },
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.cancel();
    }
});
registerActiveXtermAction({
    id: "workbench.action.terminal.chat.feedbackReportIssue" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/terminalContrib/chat/browser/terminalChatActions',
        'reportIssue',
        'Report Issue'
    )),
    precondition: ( ContextKeyExpr.and(( ContextKeyExpr.has(
        `config.${"terminal.integrated.experimentalInlineChat" }`
    )), ( TerminalChatContextKeys.requestActive.negate()), ( TerminalChatContextKeys.responseContainsCodeBlock.notEqualsTo(undefined)), TerminalChatContextKeys.responseSupportsIssueReporting)),
    icon: Codicon.report,
    menu: [{
            id: MENU_TERMINAL_CHAT_WIDGET_FEEDBACK,
            when: ( ContextKeyExpr.and(( TerminalChatContextKeys.responseContainsCodeBlock.notEqualsTo(undefined)), TerminalChatContextKeys.responseSupportsIssueReporting)),
            group: 'inline',
            order: 3
        }],
    run: (_xterm, _accessor, activeInstance) => {
        if (isDetachedTerminalInstance(activeInstance)) {
            return;
        }
        const contr = TerminalChatController.activeChatWidget || TerminalChatController.get(activeInstance);
        contr?.acceptFeedback();
    }
});
