import { addDisposableListener } from '../../../../../base/browser/dom.js';
import { Disposable } from '../../../../../base/common/lifecycle.js';
import { registerTerminalContribution } from '../../../terminal/browser/terminalExtensions.js';
class TerminalHighlightContribution extends Disposable {
    static { this.ID = 'terminal.highlight'; }
    static get(instance) {
        return instance.getContribution(TerminalHighlightContribution.ID);
    }
    constructor(_instance, processManager, widgetManager) {
        super();
        this._instance = _instance;
    }
    xtermOpen(xterm) {
        const screenElement = xterm.raw.element.querySelector('.xterm-screen');
        this._register(addDisposableListener(screenElement, 'mousemove', (e) => {
            if (e.target.tagName !== 'CANVAS') {
                return;
            }
            const rect = xterm.raw.element?.getBoundingClientRect();
            if (!rect) {
                return;
            }
            const mouseCursorY = Math.floor(e.offsetY / (rect.height / xterm.raw.rows));
            const command = this._instance.capabilities.get(2 )?.getCommandForLine(xterm.raw.buffer.active.viewportY + mouseCursorY);
            if (command && 'getOutput' in command) {
                xterm.markTracker.showCommandGuide(command);
            }
            else {
                xterm.markTracker.showCommandGuide(undefined);
            }
        }));
        this._register(addDisposableListener(screenElement, 'mouseout', () => xterm.markTracker.showCommandGuide(undefined)));
        this._register(xterm.raw.onData(() => xterm.markTracker.showCommandGuide(undefined)));
    }
}
registerTerminalContribution(TerminalHighlightContribution.ID, TerminalHighlightContribution, false);
