import { renderStringAsPlaintext } from '../../../../base/browser/markdownRenderer.js';
import { toDisposable } from '../../../../base/common/lifecycle.js';
import { removeAnsiEscapeCodes, forAnsiStringParts, GraphemeIterator } from '../../../../base/common/strings.js';
import './media/testMessageColorizer.css.js';
import { Position } from '../../../../editor/common/core/position.js';
import { Range } from '../../../../editor/common/core/range.js';
const colorAttrRe = /^\x1b\[([0-9]+)m$/;
const renderTestMessageAsText = (tm) => typeof tm === 'string' ? removeAnsiEscapeCodes(tm) : renderStringAsPlaintext(tm);
const colorizeTestMessageInEditor = (message, editor) => {
    const decos = [];
    editor.changeDecorations(changeAccessor => {
        let start = ( new Position(1, 1));
        let cls = [];
        for (const part of forAnsiStringParts(message)) {
            if (part.isCode) {
                const colorAttr = colorAttrRe.exec(part.str)?.[1];
                if (!colorAttr) {
                    continue;
                }
                const n = Number(colorAttr);
                if (n === 0) {
                    cls.length = 0;
                }
                else if (n === 22) {
                    cls = cls.filter(c => c !== "tstm-ansidec-1"  && c !== "tstm-ansidec-3" );
                }
                else if (n === 23) {
                    cls = cls.filter(c => c !== "tstm-ansidec-3" );
                }
                else if (n === 24) {
                    cls = cls.filter(c => c !== "tstm-ansidec-4" );
                }
                else if ((n >= 30 && n <= 39) || (n >= 90 && n <= 99)) {
                    cls = cls.filter(c => !c.startsWith("tstm-ansidec-fg" ));
                    cls.push("tstm-ansidec-fg"  + colorAttr);
                }
                else if ((n >= 40 && n <= 49) || (n >= 100 && n <= 109)) {
                    cls = cls.filter(c => !c.startsWith("tstm-ansidec-bg" ));
                    cls.push("tstm-ansidec-bg"  + colorAttr);
                }
                else {
                    cls.push("tstm-ansidec-"  + colorAttr);
                }
            }
            else {
                let line = start.lineNumber;
                let col = start.column;
                const graphemes = ( new GraphemeIterator(part.str));
                for (let i = 0; !graphemes.eol(); i += graphemes.nextGraphemeLength()) {
                    if (part.str[i] === '\n') {
                        line++;
                        col = 1;
                    }
                    else {
                        col++;
                    }
                }
                const end = ( new Position(line, col));
                if (cls.length) {
                    decos.push(changeAccessor.addDecoration(Range.fromPositions(start, end), {
                        inlineClassName: cls.join(' '),
                        description: 'test-message-colorized',
                    }));
                }
                start = end;
            }
        }
    });
    return toDisposable(() => editor.removeDecorations(decos));
};
export { colorizeTestMessageInEditor, renderTestMessageAsText };
