import { registerEditorContribution } from '../../../../editor/browser/editorExtensions.js';
import { localize2WithPath, localizeWithPath } from '../../../../nls.js';
import { registerAction2 } from '../../../../platform/actions/common/actions.js';
import { CommandsRegistry, ICommandService } from '../../../../platform/commands/common/commands.js';
import { Extensions as Extensions$2 } from '../../../../platform/configuration/common/configurationRegistry.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { IFileService } from '../../../../platform/files/common/files.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { IOpenerService } from '../../../../platform/opener/common/opener.js';
import { IProgressService } from '../../../../platform/progress/common/progress.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { ViewPaneContainer } from '../../../browser/parts/views/viewPaneContainer.js';
import { Extensions as Extensions$1 } from '../../../common/contributions.js';
import { Extensions } from '../../../common/views.js';
import { REVEAL_IN_EXPLORER_COMMAND_ID } from '../../files/browser/fileConstants.js';
import { CodeCoverageDecorations } from './codeCoverageDecorations.js';
import { testingViewIcon, testingResultsIcon } from './icons.js';
import { ExplorerTestCoverageBars } from './testCoverageBars.js';
import { TestCoverageView } from './testCoverageView.js';
import { TestingDecorations } from './testingDecorations.js';
import { TestingExplorerView } from './testingExplorerView.js';
import { TestResultsView, TestingPeekOpener, TestingOutputPeekController, OpenMessageInEditorAction, GoToPreviousMessageAction, GoToNextMessageAction, CloseTestPeek, ToggleTestingPeekHistory } from './testingOutputPeek.js';
import { TestingProgressTrigger } from './testingProgressUiService.js';
import { TestingViewPaneContainer } from './testingViewPaneContainer.js';
import { testingConfiguration } from '../common/configuration.js';
import '../common/testCoverageService.js';
import { ITestExplorerFilterState } from '../common/testExplorerFilterState.js';
import { TestId } from '../common/testId.js';
import '../common/testProfileService.js';
import { ITestResultService } from '../common/testResultService.js';
import '../common/testResultStorage.js';
import { ITestService } from '../common/testService.js';
import '../common/testServiceImpl.js';
import { TestingContentProvider } from '../common/testingContentProvider.js';
import { TestingContextKeys } from '../common/testingContextKeys.js';
import '../common/testingContinuousRunService.js';
import { ITestingDecorationsService } from '../common/testingDecorations.js';
import { ITestingPeekOpener } from '../common/testingPeekOpener.js';
import { IViewsService } from '../../../services/views/common/viewsService.js';
import { allTestActions, discoverAndRunTests } from './testExplorerActions.js';
import './testingConfigurationUi.js';
const viewContainer = ( Registry.as(Extensions.ViewContainersRegistry)).registerViewContainer({
    id: "workbench.view.extension.test" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/testing/browser/testing.contribution',
        'test',
        'Testing'
    )),
    ctorDescriptor: ( new SyncDescriptor(TestingViewPaneContainer)),
    icon: testingViewIcon,
    alwaysUseContainerInfo: true,
    order: 6,
    openCommandActionDescriptor: {
        id: "workbench.view.extension.test" ,
        mnemonicTitle: ( localizeWithPath(
            'vs/workbench/contrib/testing/browser/testing.contribution',
            { key: 'miViewTesting', comment: ['&& denotes a mnemonic'] },
            "T&&esting"
        )),
        order: 4,
    },
    hideIfEmpty: true,
}, 0 );
const testResultsViewContainer = ( Registry.as(Extensions.ViewContainersRegistry)).registerViewContainer({
    id: "workbench.panel.testResults" ,
    title: ( localize2WithPath(
        'vs/workbench/contrib/testing/browser/testing.contribution',
        'testResultsPanelName',
        "Test Results"
    )),
    icon: testingResultsIcon,
    ctorDescriptor: ( new SyncDescriptor(
        ViewPaneContainer,
        ["workbench.panel.testResults" , { mergeViewWithContainerWhenSingleView: true }]
    )),
    hideIfEmpty: true,
    order: 3,
}, 1 , { doNotRegisterOpenCommand: true });
const viewsRegistry = ( Registry.as(Extensions.ViewsRegistry));
viewsRegistry.registerViews([{
        id: "workbench.panel.testResults.view" ,
        name: ( localize2WithPath(
            'vs/workbench/contrib/testing/browser/testing.contribution',
            'testResultsPanelName',
            "Test Results"
        )),
        containerIcon: testingResultsIcon,
        canToggleVisibility: false,
        canMoveView: true,
        when: ( TestingContextKeys.hasAnyResults.isEqualTo(true)),
        ctorDescriptor: ( new SyncDescriptor(TestResultsView)),
    }], testResultsViewContainer);
viewsRegistry.registerViewWelcomeContent("workbench.view.testing" , {
    content: ( localizeWithPath(
        'vs/workbench/contrib/testing/browser/testing.contribution',
        'noTestProvidersRegistered',
        "No tests have been found in this workspace yet."
    )),
});
viewsRegistry.registerViewWelcomeContent("workbench.view.testing" , {
    content: '[' + ( localizeWithPath(
        'vs/workbench/contrib/testing/browser/testing.contribution',
        'searchForAdditionalTestExtensions',
        "Install Additional Test Extensions..."
    )) + `](command:${"testing.searchForTestExtension" })`,
    order: 10
});
viewsRegistry.registerViews([{
        id: "workbench.view.testing" ,
        name: ( localize2WithPath(
            'vs/workbench/contrib/testing/browser/testing.contribution',
            'testExplorer',
            "Test Explorer"
        )),
        ctorDescriptor: ( new SyncDescriptor(TestingExplorerView)),
        canToggleVisibility: true,
        canMoveView: true,
        weight: 80,
        order: -999,
        containerIcon: testingViewIcon,
        when: ContextKeyExpr.greater(TestingContextKeys.providerCount.key, 0),
    }, {
        id: "workbench.view.testCoverage" ,
        name: ( localize2WithPath(
            'vs/workbench/contrib/testing/browser/testing.contribution',
            'testCoverage',
            "Test Coverage"
        )),
        ctorDescriptor: ( new SyncDescriptor(TestCoverageView)),
        canToggleVisibility: true,
        canMoveView: true,
        weight: 80,
        order: -998,
        containerIcon: testingViewIcon,
        when: TestingContextKeys.isTestCoverageOpen,
    }], viewContainer);
allTestActions.forEach(registerAction2);
registerAction2(OpenMessageInEditorAction);
registerAction2(GoToPreviousMessageAction);
registerAction2(GoToNextMessageAction);
registerAction2(CloseTestPeek);
registerAction2(ToggleTestingPeekHistory);
( Registry.as(Extensions$1.Workbench)).registerWorkbenchContribution(TestingContentProvider, 3 );
( Registry.as(Extensions$1.Workbench)).registerWorkbenchContribution(TestingPeekOpener, 4 );
( Registry.as(Extensions$1.Workbench)).registerWorkbenchContribution(TestingProgressTrigger, 4 );
registerEditorContribution("editor.contrib.testingOutputPeek" , TestingOutputPeekController, 1 );
registerEditorContribution("editor.contrib.testingDecorations" , TestingDecorations, 1 );
registerEditorContribution("editor.contrib.coverageDecorations" , CodeCoverageDecorations, 3 );
CommandsRegistry.registerCommand({
    id: '_revealTestInExplorer',
    handler: async (accessor, testId, focus) => {
        accessor.get(ITestExplorerFilterState).reveal.value = typeof testId === 'string' ? testId : testId.extId;
        accessor.get(IViewsService).openView("workbench.view.testing" , focus);
    }
});
CommandsRegistry.registerCommand({
    id: 'vscode.peekTestError',
    handler: async (accessor, extId) => {
        const lookup = accessor.get(ITestResultService).getStateById(extId);
        if (!lookup) {
            return false;
        }
        const [result, ownState] = lookup;
        const opener = accessor.get(ITestingPeekOpener);
        if (opener.tryPeekFirstError(result, ownState)) {
            return true;
        }
        for (const test of result.tests) {
            if (TestId.compare(ownState.item.extId, test.item.extId) === 2  && opener.tryPeekFirstError(result, test)) {
                return true;
            }
        }
        return false;
    }
});
CommandsRegistry.registerCommand({
    id: 'vscode.revealTest',
    handler: async (accessor, extId) => {
        const test = accessor.get(ITestService).collection.getNodeById(extId);
        if (!test) {
            return;
        }
        const commandService = accessor.get(ICommandService);
        const fileService = accessor.get(IFileService);
        const openerService = accessor.get(IOpenerService);
        const { range, uri } = test.item;
        if (!uri) {
            return;
        }
        const position = accessor.get(ITestingDecorationsService).getDecoratedTestPosition(uri, extId) || range?.getStartPosition();
        accessor.get(ITestExplorerFilterState).reveal.value = extId;
        accessor.get(ITestingPeekOpener).closeAllPeeks();
        let isFile = true;
        try {
            if (!(await fileService.stat(uri)).isFile) {
                isFile = false;
            }
        }
        catch {
        }
        if (!isFile) {
            await commandService.executeCommand(REVEAL_IN_EXPLORER_COMMAND_ID, uri);
            return;
        }
        await openerService.open(position
            ? uri.with({ fragment: `L${position.lineNumber}:${position.column}` })
            : uri);
    }
});
CommandsRegistry.registerCommand({
    id: 'vscode.runTestsById',
    handler: async (accessor, group, ...testIds) => {
        const testService = accessor.get(ITestService);
        await discoverAndRunTests(accessor.get(ITestService).collection, accessor.get(IProgressService), testIds, tests => testService.runTests({ group, tests }));
    }
});
( Registry.as(Extensions$2.Configuration)).registerConfiguration(testingConfiguration);
( Registry.as(
    "workbench.registry.explorer.fileContributions"
)).register({
    create(insta, container) {
        return insta.createInstance(ExplorerTestCoverageBars, { compact: true, container });
    },
});
