import { Color, RGBA } from '../../../../base/common/color.js';
import { localizeWithPath } from '../../../../nls.js';
import { registerColor, transparent, opaque } from '../../../../platform/theme/common/colorUtils.js';
import { contrastBorder } from '../../../../platform/theme/common/colors/baseColors.js';
import { chartsGreen, chartsRed } from '../../../../platform/theme/common/colors/chartsColors.js';
import { editorErrorForeground, editorInfoForeground, diffInserted, diffRemoved, editorForeground, editorBackground } from '../../../../platform/theme/common/colors/editorColors.js';
import '../../../../platform/theme/common/colors/inputColors.js';
import '../../../../platform/theme/common/colors/listColors.js';
import '../../../../platform/theme/common/colors/menuColors.js';
import '../../../../platform/theme/common/colors/minimapColors.js';
import { badgeBackground, badgeForeground } from '../../../../platform/theme/common/colors/miscColors.js';
import '../../../../platform/theme/common/colors/quickpickColors.js';
import '../../../../platform/theme/common/colors/searchColors.js';
import { registerThemingParticipant } from '../../../../platform/theme/common/themeService.js';
const testingColorIconFailed = registerColor('testing.iconFailed', {
    dark: '#f14c4c',
    light: '#f14c4c',
    hcDark: '#f14c4c',
    hcLight: '#B5200D'
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconFailed',
    "Color for the 'failed' icon in the test explorer."
)));
const testingColorIconErrored = registerColor('testing.iconErrored', {
    dark: '#f14c4c',
    light: '#f14c4c',
    hcDark: '#f14c4c',
    hcLight: '#B5200D'
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconErrored',
    "Color for the 'Errored' icon in the test explorer."
)));
const testingColorIconPassed = registerColor('testing.iconPassed', {
    dark: '#73c991',
    light: '#73c991',
    hcDark: '#73c991',
    hcLight: '#007100'
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconPassed',
    "Color for the 'passed' icon in the test explorer."
)));
const testingColorRunAction = registerColor('testing.runAction', {
    dark: testingColorIconPassed,
    light: testingColorIconPassed,
    hcDark: testingColorIconPassed,
    hcLight: testingColorIconPassed
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.runAction',
    "Color for 'run' icons in the editor."
)));
const testingColorIconQueued = registerColor('testing.iconQueued', {
    dark: '#cca700',
    light: '#cca700',
    hcDark: '#cca700',
    hcLight: '#cca700'
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconQueued',
    "Color for the 'Queued' icon in the test explorer."
)));
const testingColorIconUnset = registerColor('testing.iconUnset', {
    dark: '#848484',
    light: '#848484',
    hcDark: '#848484',
    hcLight: '#848484'
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconUnset',
    "Color for the 'Unset' icon in the test explorer."
)));
const testingColorIconSkipped = registerColor('testing.iconSkipped', {
    dark: '#848484',
    light: '#848484',
    hcDark: '#848484',
    hcLight: '#848484'
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconSkipped',
    "Color for the 'Skipped' icon in the test explorer."
)));
const testingPeekBorder = registerColor('testing.peekBorder', {
    dark: editorErrorForeground,
    light: editorErrorForeground,
    hcDark: contrastBorder,
    hcLight: contrastBorder
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.peekBorder',
    'Color of the peek view borders and arrow.'
)));
const testingMessagePeekBorder = registerColor('testing.messagePeekBorder', {
    dark: editorInfoForeground,
    light: editorInfoForeground,
    hcDark: contrastBorder,
    hcLight: contrastBorder
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.messagePeekBorder',
    'Color of the peek view borders and arrow when peeking a logged message.'
)));
const testingPeekHeaderBackground = registerColor('testing.peekHeaderBackground', {
    dark: ( transparent(editorErrorForeground, 0.1)),
    light: ( transparent(editorErrorForeground, 0.1)),
    hcDark: null,
    hcLight: null
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.peekBorder',
    'Color of the peek view borders and arrow.'
)));
const testingPeekMessageHeaderBackground = registerColor('testing.messagePeekHeaderBackground', {
    dark: ( transparent(editorInfoForeground, 0.1)),
    light: ( transparent(editorInfoForeground, 0.1)),
    hcDark: null,
    hcLight: null
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.messagePeekHeaderBackground',
    'Color of the peek view borders and arrow when peeking a logged message.'
)));
const testingCoveredBackground = registerColor('testing.coveredBackground', {
    dark: diffInserted,
    light: diffInserted,
    hcDark: null,
    hcLight: null
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.coveredBackground',
    'Background color of text that was covered.'
)));
const testingCoveredBorder = registerColor('testing.coveredBorder', {
    dark: ( transparent(testingCoveredBackground, 0.75)),
    light: ( transparent(testingCoveredBackground, 0.75)),
    hcDark: contrastBorder,
    hcLight: contrastBorder
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.coveredBorder',
    'Border color of text that was covered.'
)));
registerColor('testing.coveredGutterBackground', {
    dark: ( transparent(diffInserted, 0.6)),
    light: ( transparent(diffInserted, 0.6)),
    hcDark: chartsGreen,
    hcLight: chartsGreen
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.coveredGutterBackground',
    'Gutter color of regions where code was covered.'
)));
registerColor('testing.uncoveredBranchBackground', {
    dark: opaque(( transparent(diffRemoved, 2)), editorBackground),
    light: opaque(( transparent(diffRemoved, 2)), editorBackground),
    hcDark: null,
    hcLight: null
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.uncoveredBranchBackground',
    'Background of the widget shown for an uncovered branch.'
)));
const testingUncoveredBackground = registerColor('testing.uncoveredBackground', {
    dark: diffRemoved,
    light: diffRemoved,
    hcDark: null,
    hcLight: null
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.uncoveredBackground',
    'Background color of text that was not covered.'
)));
const testingUncoveredBorder = registerColor('testing.uncoveredBorder', {
    dark: ( transparent(testingUncoveredBackground, 0.75)),
    light: ( transparent(testingUncoveredBackground, 0.75)),
    hcDark: contrastBorder,
    hcLight: contrastBorder
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.uncoveredBorder',
    'Border color of text that was not covered.'
)));
registerColor('testing.uncoveredGutterBackground', {
    dark: ( transparent(diffRemoved, 1.5)),
    light: ( transparent(diffRemoved, 1.5)),
    hcDark: chartsRed,
    hcLight: chartsRed
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.uncoveredGutterBackground',
    'Gutter color of regions where code not covered.'
)));
registerColor('testing.coverCountBadgeBackground', {
    dark: badgeBackground,
    light: badgeBackground,
    hcDark: badgeBackground,
    hcLight: badgeBackground
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.coverCountBadgeBackground',
    'Background for the badge indicating execution count'
)));
registerColor('testing.coverCountBadgeForeground', {
    dark: badgeForeground,
    light: badgeForeground,
    hcDark: badgeForeground,
    hcLight: badgeForeground
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.coverCountBadgeForeground',
    'Foreground for the badge indicating execution count'
)));
({
    [0 ]: {
        decorationForeground: registerColor('testing.message.error.decorationForeground', { dark: editorErrorForeground, light: editorErrorForeground, hcDark: editorForeground, hcLight: editorForeground }, ( localizeWithPath(
            'vs/workbench/contrib/testing/browser/theme',
            'testing.message.error.decorationForeground',
            'Text color of test error messages shown inline in the editor.'
        ))),
        marginBackground: registerColor('testing.message.error.lineBackground', { dark: ( new Color(( new RGBA(255, 0, 0, 0.2)))), light: ( new Color(( new RGBA(255, 0, 0, 0.2)))), hcDark: null, hcLight: null }, ( localizeWithPath(
            'vs/workbench/contrib/testing/browser/theme',
            'testing.message.error.marginBackground',
            'Margin color beside error messages shown inline in the editor.'
        ))),
    },
    [1 ]: {
        decorationForeground: registerColor('testing.message.info.decorationForeground', { dark: ( transparent(editorForeground, 0.5)), light: ( transparent(editorForeground, 0.5)), hcDark: ( transparent(editorForeground, 0.5)), hcLight: ( transparent(editorForeground, 0.5)) }, ( localizeWithPath(
            'vs/workbench/contrib/testing/browser/theme',
            'testing.message.info.decorationForeground',
            'Text color of test info messages shown inline in the editor.'
        ))),
        marginBackground: registerColor('testing.message.info.lineBackground', { dark: null, light: null, hcDark: null, hcLight: null }, ( localizeWithPath(
            'vs/workbench/contrib/testing/browser/theme',
            'testing.message.info.marginBackground',
            'Margin color beside info messages shown inline in the editor.'
        ))),
    },
});
const testStatesToIconColors = {
    [6 ]: testingColorIconErrored,
    [4 ]: testingColorIconFailed,
    [3 ]: testingColorIconPassed,
    [1 ]: testingColorIconQueued,
    [0 ]: testingColorIconUnset,
    [5 ]: testingColorIconSkipped,
};
const testingRetiredColorIconErrored = registerColor('testing.iconErrored.retired', {
    dark: ( transparent(testingColorIconErrored, 0.7)),
    light: ( transparent(testingColorIconErrored, 0.7)),
    hcDark: ( transparent(testingColorIconErrored, 0.7)),
    hcLight: ( transparent(testingColorIconErrored, 0.7))
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconErrored.retired',
    "Retired color for the 'Errored' icon in the test explorer."
)));
const testingRetiredColorIconFailed = registerColor('testing.iconFailed.retired', {
    dark: ( transparent(testingColorIconFailed, 0.7)),
    light: ( transparent(testingColorIconFailed, 0.7)),
    hcDark: ( transparent(testingColorIconFailed, 0.7)),
    hcLight: ( transparent(testingColorIconFailed, 0.7))
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconFailed.retired',
    "Retired color for the 'failed' icon in the test explorer."
)));
const testingRetiredColorIconPassed = registerColor('testing.iconPassed.retired', {
    dark: ( transparent(testingColorIconPassed, 0.7)),
    light: ( transparent(testingColorIconPassed, 0.7)),
    hcDark: ( transparent(testingColorIconPassed, 0.7)),
    hcLight: ( transparent(testingColorIconPassed, 0.7))
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconPassed.retired',
    "Retired color for the 'passed' icon in the test explorer."
)));
const testingRetiredColorIconQueued = registerColor('testing.iconQueued.retired', {
    dark: ( transparent(testingColorIconQueued, 0.7)),
    light: ( transparent(testingColorIconQueued, 0.7)),
    hcDark: ( transparent(testingColorIconQueued, 0.7)),
    hcLight: ( transparent(testingColorIconQueued, 0.7))
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconQueued.retired',
    "Retired color for the 'Queued' icon in the test explorer."
)));
const testingRetiredColorIconUnset = registerColor('testing.iconUnset.retired', {
    dark: ( transparent(testingColorIconUnset, 0.7)),
    light: ( transparent(testingColorIconUnset, 0.7)),
    hcDark: ( transparent(testingColorIconUnset, 0.7)),
    hcLight: ( transparent(testingColorIconUnset, 0.7))
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconUnset.retired',
    "Retired color for the 'Unset' icon in the test explorer."
)));
const testingRetiredColorIconSkipped = registerColor('testing.iconSkipped.retired', {
    dark: ( transparent(testingColorIconSkipped, 0.7)),
    light: ( transparent(testingColorIconSkipped, 0.7)),
    hcDark: ( transparent(testingColorIconSkipped, 0.7)),
    hcLight: ( transparent(testingColorIconSkipped, 0.7))
}, ( localizeWithPath(
    'vs/workbench/contrib/testing/browser/theme',
    'testing.iconSkipped.retired',
    "Retired color for the 'Skipped' icon in the test explorer."
)));
const testStatesToRetiredIconColors = {
    [6 ]: testingRetiredColorIconErrored,
    [4 ]: testingRetiredColorIconFailed,
    [3 ]: testingRetiredColorIconPassed,
    [1 ]: testingRetiredColorIconQueued,
    [0 ]: testingRetiredColorIconUnset,
    [5 ]: testingRetiredColorIconSkipped,
};
registerThemingParticipant((theme, collector) => {
    const editorBg = theme.getColor(editorBackground);
    const missBadgeBackground = editorBg && theme.getColor(testingUncoveredBackground)?.transparent(2).makeOpaque(editorBg);
    collector.addRule(`
	.coverage-deco-inline.coverage-deco-hit.coverage-deco-hovered {
		background: ${theme.getColor(testingCoveredBackground)?.transparent(1.3)};
		outline-color: ${theme.getColor(testingCoveredBorder)?.transparent(2)};
	}
	.coverage-deco-inline.coverage-deco-miss.coverage-deco-hovered {
		background: ${theme.getColor(testingUncoveredBackground)?.transparent(1.3)};
		outline-color: ${theme.getColor(testingUncoveredBorder)?.transparent(2)};
	}
	.coverage-deco-branch-miss-indicator::before {
		border-color: ${missBadgeBackground?.transparent(1.3)};
		background-color: ${missBadgeBackground};
	}
	`);
});
export { testStatesToIconColors, testStatesToRetiredIconColors, testingColorIconErrored, testingColorIconFailed, testingColorIconPassed, testingColorIconQueued, testingColorIconSkipped, testingColorIconUnset, testingColorRunAction, testingCoveredBackground, testingCoveredBorder, testingMessagePeekBorder, testingPeekBorder, testingPeekHeaderBackground, testingPeekMessageHeaderBackground, testingRetiredColorIconErrored, testingRetiredColorIconFailed, testingRetiredColorIconPassed, testingRetiredColorIconQueued, testingRetiredColorIconSkipped, testingRetiredColorIconUnset, testingUncoveredBackground, testingUncoveredBorder };
