import '../../../../base/common/observableInternal/derived.js';
import '../../../../base/common/observableInternal/autorun.js';
import { observableFromEvent } from '../../../../base/common/observableInternal/utils.js';
import '../../../../base/common/cancellation.js';
import { localizeWithPath } from '../../../../nls.js';
const testingConfiguration = {
    id: 'testing',
    order: 21,
    title: ( localizeWithPath(
        'vs/workbench/contrib/testing/common/configuration',
        'testConfigurationTitle',
        "Testing"
    )),
    type: 'object',
    properties: {
        ["testing.autoRun.delay" ]: {
            type: 'integer',
            minimum: 0,
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.autoRun.delay',
                "How long to wait, in milliseconds, after a test is marked as outdated and starting a new run."
            )),
            default: 1000,
        },
        ["testing.automaticallyOpenPeekView" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.automaticallyOpenPeekView',
                "Configures when the error Peek view is automatically opened."
            )),
            enum: [
                "failureAnywhere" ,
                "failureInVisibleDocument" ,
                "never" ,
            ],
            default: "failureInVisibleDocument" ,
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.automaticallyOpenPeekView.failureAnywhere',
                    "Open automatically no matter where the failure is."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.automaticallyOpenPeekView.failureInVisibleDocument',
                    "Open automatically when a test fails in a visible document."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.automaticallyOpenPeekView.never',
                    "Never automatically open."
                )),
            ],
        },
        ["testing.showAllMessages" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.showAllMessages',
                "Controls whether to show messages from all test runs."
            )),
            type: 'boolean',
            default: false,
        },
        ["testing.automaticallyOpenPeekViewDuringAutoRun" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.automaticallyOpenPeekViewDuringContinuousRun',
                "Controls whether to automatically open the Peek view during continuous run mode."
            )),
            type: 'boolean',
            default: false,
        },
        ["testing.countBadge" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.countBadge',
                'Controls the count badge on the Testing icon on the Activity Bar.'
            )),
            enum: [
                "failed" ,
                "off" ,
                "passed" ,
                "skipped" ,
            ],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.countBadge.failed',
                    'Show the number of failed tests'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.countBadge.off',
                    'Disable the testing count badge'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.countBadge.passed',
                    'Show the number of passed tests'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.countBadge.skipped',
                    'Show the number of skipped tests'
                )),
            ],
            default: "failed" ,
        },
        ["testing.followRunningTest" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.followRunningTest',
                'Controls whether the running test should be followed in the Test Explorer view.'
            )),
            type: 'boolean',
            default: true,
        },
        ["testing.defaultGutterClickAction" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.defaultGutterClickAction',
                'Controls the action to take when left-clicking on a test decoration in the gutter.'
            )),
            enum: [
                "run" ,
                "debug" ,
                "runWithCoverage" ,
                "contextMenu" ,
            ],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.defaultGutterClickAction.run',
                    'Run the test.'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.defaultGutterClickAction.debug',
                    'Debug the test.'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.defaultGutterClickAction.coverage',
                    'Run the test with coverage.'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.defaultGutterClickAction.contextMenu',
                    'Open the context menu for more options.'
                )),
            ],
            default: "run" ,
        },
        ["testing.gutterEnabled" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.gutterEnabled',
                'Controls whether test decorations are shown in the editor gutter.'
            )),
            type: 'boolean',
            default: true,
        },
        ["testing.saveBeforeTest" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.saveBeforeTest',
                'Control whether save all dirty editors before running a test.'
            )),
            type: 'boolean',
            default: true,
        },
        ["testing.openTesting" ]: {
            enum: [
                "neverOpen" ,
                "openOnTestStart" ,
                "openOnTestFailure" ,
                "openExplorerOnTestStart" ,
            ],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.openTesting.neverOpen',
                    'Never automatically open the testing views'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.openTesting.openOnTestStart',
                    'Open the test results view when tests start'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.openTesting.openOnTestFailure',
                    'Open the test result view on any test failure'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.openTesting.openExplorerOnTestStart',
                    'Open the test explorer when tests start'
                )),
            ],
            default: 'openOnTestStart',
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.openTesting',
                "Controls when the testing view should open."
            ))
        },
        ["testing.alwaysRevealTestOnStateChange" ]: {
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.alwaysRevealTestOnStateChange',
                "Always reveal the executed test when `#testing.followRunningTest#` is on. If this setting is turned off, only failed tests will be revealed."
            )),
            type: 'boolean',
            default: false,
        },
        ["testing.showCoverageInExplorer" ]: {
            description: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.ShowCoverageInExplorer',
                "Whether test coverage should be down in the File Explorer view."
            )),
            type: 'boolean',
            default: true,
        },
        ["testing.displayedCoveragePercent" ]: {
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.displayedCoveragePercent',
                "Configures what percentage is displayed by default for test coverage."
            )),
            default: "totalCoverage" ,
            enum: [
                "totalCoverage" ,
                "statement" ,
                "minimum" ,
            ],
            enumDescriptions: [
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.displayedCoveragePercent.totalCoverage',
                    'A calculation of the combined statement, function, and branch coverage.'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.displayedCoveragePercent.statement',
                    'The statement coverage.'
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/testing/common/configuration',
                    'testing.displayedCoveragePercent.minimum',
                    'The minimum of statement, function, and branch coverage.'
                )),
            ],
        },
        ["testing.coverageBarThresholds" ]: {
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/testing/common/configuration',
                'testing.coverageBarThresholds',
                "Configures the colors used for percentages in test coverage bars."
            )),
            default: { red: 0, yellow: 60, green: 90 },
            properties: {
                red: { type: 'number', minimum: 0, maximum: 100, default: 0 },
                yellow: { type: 'number', minimum: 0, maximum: 100, default: 60 },
                green: { type: 'number', minimum: 0, maximum: 100, default: 90 },
            },
        },
    }
};
const getTestingConfiguration = (config, key) => config.getValue(key);
const observeTestingConfiguration = (config, key) => observableFromEvent(config.onDidChangeConfiguration, () => getTestingConfiguration(config, key));
export { getTestingConfiguration, observeTestingConfiguration, testingConfiguration };
