import { localizeWithPath } from '../../../../nls.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { Extensions } from '../../../common/views.js';
import { VIEW_CONTAINER } from '../../files/browser/explorerViewlet.js';
import { TimelinePaneId, ITimelineService } from '../common/timeline.js';
import { TimelineHasProviderContext } from '../common/timelineService.js';
import { TimelinePane } from './timelinePane.js';
import { Extensions as Extensions$1 } from '../../../../platform/configuration/common/configurationRegistry.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { MenuRegistry, MenuId } from '../../../../platform/actions/common/actions.js';
import { CommandsRegistry } from '../../../../platform/commands/common/commands.js';
import { ExplorerFolderContext } from '../../files/common/files.js';
import { ResourceContextKey } from '../../../common/contextkeys.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { registerIcon } from '../../../../platform/theme/common/iconRegistry.js';
const timelineViewIcon = registerIcon('timeline-view-icon', Codicon.history, ( localizeWithPath(
    'vs/workbench/contrib/timeline/browser/timeline.contribution',
    'timelineViewIcon',
    'View icon of the timeline view.'
)));
const timelineOpenIcon = registerIcon('timeline-open', Codicon.history, ( localizeWithPath(
    'vs/workbench/contrib/timeline/browser/timeline.contribution',
    'timelineOpenIcon',
    'Icon for the open timeline action.'
)));
class TimelinePaneDescriptor {
    constructor() {
        this.id = TimelinePaneId;
        this.name = TimelinePane.TITLE;
        this.containerIcon = timelineViewIcon;
        this.ctorDescriptor = ( new SyncDescriptor(TimelinePane));
        this.order = 2;
        this.weight = 30;
        this.collapsed = true;
        this.canToggleVisibility = true;
        this.hideByDefault = false;
        this.canMoveView = true;
        this.when = TimelineHasProviderContext;
        this.focusCommand = { id: 'timeline.focus' };
    }
}
const configurationRegistry = ( Registry.as(Extensions$1.Configuration));
configurationRegistry.registerConfiguration({
    id: 'timeline',
    order: 1001,
    title: ( localizeWithPath(
        'vs/workbench/contrib/timeline/browser/timeline.contribution',
        'timelineConfigurationTitle',
        "Timeline"
    )),
    type: 'object',
    properties: {
        'timeline.pageSize': {
            type: ['number', 'null'],
            default: null,
            markdownDescription: ( localizeWithPath(
                'vs/workbench/contrib/timeline/browser/timeline.contribution',
                'timeline.pageSize',
                "The number of items to show in the Timeline view by default and when loading more items. Setting to `null` (the default) will automatically choose a page size based on the visible area of the Timeline view."
            )),
        },
        'timeline.pageOnScroll': {
            type: 'boolean',
            default: false,
            description: ( localizeWithPath(
                'vs/workbench/contrib/timeline/browser/timeline.contribution',
                'timeline.pageOnScroll',
                "Experimental. Controls whether the Timeline view will load the next page of items when you scroll to the end of the list."
            )),
        },
    }
});
( Registry.as(Extensions.ViewsRegistry)).registerViews([( new TimelinePaneDescriptor())], VIEW_CONTAINER);
var OpenTimelineAction;
( (function(OpenTimelineAction) {
    OpenTimelineAction.ID = 'files.openTimeline';
    OpenTimelineAction.LABEL = ( localizeWithPath(
        'vs/workbench/contrib/timeline/browser/timeline.contribution',
        'files.openTimeline',
        "Open Timeline"
    ));
    function handler() {
        return (accessor, arg) => {
            const service = accessor.get(ITimelineService);
            return service.setUri(arg);
        };
    }
    OpenTimelineAction.handler = handler;
})(OpenTimelineAction || (OpenTimelineAction = {})));
CommandsRegistry.registerCommand(OpenTimelineAction.ID, OpenTimelineAction.handler());
MenuRegistry.appendMenuItem(MenuId.ExplorerContext, ({
    group: '4_timeline',
    order: 1,
    command: {
        id: OpenTimelineAction.ID,
        title: OpenTimelineAction.LABEL,
        icon: timelineOpenIcon
    },
    when: ( ContextKeyExpr.and(( ExplorerFolderContext.toNegated()), ResourceContextKey.HasResource, TimelineHasProviderContext))
}));
const timelineFilter = registerIcon('timeline-filter', Codicon.filter, ( localizeWithPath(
    'vs/workbench/contrib/timeline/browser/timeline.contribution',
    'timelineFilter',
    'Icon for the filter timeline action.'
)));
MenuRegistry.appendMenuItem(MenuId.TimelineTitle, {
    submenu: MenuId.TimelineFilterSubMenu,
    title: ( localizeWithPath(
        'vs/workbench/contrib/timeline/browser/timeline.contribution',
        'filterTimeline',
        "Filter Timeline"
    )),
    group: 'navigation',
    order: 100,
    icon: timelineFilter
});
export { TimelinePaneDescriptor };
