import '../../../../platform/update/common/update.config.contribution.js';
import { localize2WithPath, localizeWithPath } from '../../../../nls.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { Extensions } from '../../../common/contributions.js';
import { Categories } from '../../../../platform/action/common/actionCommonCategories.js';
import { registerAction2, Action2, MenuId } from '../../../../platform/actions/common/actions.js';
import { ProductContribution, UpdateContribution, SwitchProductQualityContribution, DOWNLOAD_URL, RELEASE_NOTES_URL, showReleaseNotesInEditor, CONTEXT_UPDATE_STATE } from './update.js';
import product$1 from '../../../../platform/product/common/product.js';
import { IUpdateService } from '../../../../platform/update/common/update.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { isWindows } from '../../../../base/common/platform.js';
import '../../../../../../../override/vs/platform/dialogs/common/dialogs.js';
import { mnemonicButtonLabel } from '../../../../base/common/labels.js';
import { ShowCurrentReleaseNotesActionId, ShowCurrentReleaseNotesFromCurrentFileActionId } from '../common/update.js';
import { IsWebContext } from '../../../../platform/contextkey/common/contextkeys.js';
import { IOpenerService } from '../../../../platform/opener/common/opener.js';
import { IProductService } from '../../../../platform/product/common/productService.js';
import { URI } from '../../../../base/common/uri.js';
import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { IFileDialogService } from '../../../../platform/dialogs/common/dialogs.js';
const workbench = ( Registry.as(Extensions.Workbench));
workbench.registerWorkbenchContribution(ProductContribution, 3 );
workbench.registerWorkbenchContribution(UpdateContribution, 3 );
workbench.registerWorkbenchContribution(SwitchProductQualityContribution, 3 );
class ShowCurrentReleaseNotesAction extends Action2 {
    constructor() {
        super({
            id: ShowCurrentReleaseNotesActionId,
            title: {
                ...( localize2WithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    'showReleaseNotes',
                    "Show Release Notes"
                )),
                mnemonicTitle: ( localizeWithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    { key: 'mshowReleaseNotes', comment: ['&& denotes a mnemonic'] },
                    "Show &&Release Notes"
                )),
            },
            category: { value: product$1.nameShort, original: product$1.nameShort },
            f1: true,
            precondition: RELEASE_NOTES_URL,
            menu: [{
                    id: MenuId.MenubarHelpMenu,
                    group: '1_welcome',
                    order: 5,
                    when: RELEASE_NOTES_URL,
                }]
        });
    }
    async run(accessor) {
        const instantiationService = accessor.get(IInstantiationService);
        const productService = accessor.get(IProductService);
        const openerService = accessor.get(IOpenerService);
        try {
            await showReleaseNotesInEditor(instantiationService, productService.version, false);
        }
        catch (err) {
            if (productService.releaseNotesUrl) {
                await openerService.open(( URI.parse(productService.releaseNotesUrl)));
            }
            else {
                throw new Error(localizeWithPath('vs/workbench/contrib/update/browser/update.contribution', 'update.noReleaseNotesOnline', "This version of {0} does not have release notes online", productService.nameLong));
            }
        }
    }
}
class ShowCurrentReleaseNotesFromCurrentFileAction extends Action2 {
    constructor() {
        super({
            id: ShowCurrentReleaseNotesFromCurrentFileActionId,
            title: {
                ...( localize2WithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    'showReleaseNotesCurrentFile',
                    "Open Current File as Release Notes"
                )),
                mnemonicTitle: ( localizeWithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    { key: 'mshowReleaseNotes', comment: ['&& denotes a mnemonic'] },
                    "Show &&Release Notes"
                )),
            },
            category: ( localize2WithPath(
                'vs/workbench/contrib/update/browser/update.contribution',
                'developerCategory',
                "Developer"
            )),
            f1: true,
            precondition: RELEASE_NOTES_URL
        });
    }
    async run(accessor) {
        const instantiationService = accessor.get(IInstantiationService);
        const productService = accessor.get(IProductService);
        try {
            await showReleaseNotesInEditor(instantiationService, productService.version, true);
        }
        catch (err) {
            throw new Error(localizeWithPath('vs/workbench/contrib/update/browser/update.contribution', 'releaseNotesFromFileNone', "Cannot open the current file as Release Notes"));
        }
    }
}
registerAction2(ShowCurrentReleaseNotesAction);
registerAction2(ShowCurrentReleaseNotesFromCurrentFileAction);
class CheckForUpdateAction extends Action2 {
    constructor() {
        super({
            id: 'update.checkForUpdate',
            title: ( localize2WithPath(
                'vs/workbench/contrib/update/browser/update.contribution',
                'checkForUpdates',
                'Check for Updates...'
            )),
            category: { value: product$1.nameShort, original: product$1.nameShort },
            f1: true,
            precondition: ( CONTEXT_UPDATE_STATE.isEqualTo("idle" )),
        });
    }
    async run(accessor) {
        const updateService = accessor.get(IUpdateService);
        return updateService.checkForUpdates(true);
    }
}
class DownloadUpdateAction extends Action2 {
    constructor() {
        super({
            id: 'update.downloadUpdate',
            title: ( localize2WithPath(
                'vs/workbench/contrib/update/browser/update.contribution',
                'downloadUpdate',
                'Download Update'
            )),
            category: { value: product$1.nameShort, original: product$1.nameShort },
            f1: true,
            precondition: ( CONTEXT_UPDATE_STATE.isEqualTo("available for download" ))
        });
    }
    async run(accessor) {
        await accessor.get(IUpdateService).downloadUpdate();
    }
}
class InstallUpdateAction extends Action2 {
    constructor() {
        super({
            id: 'update.installUpdate',
            title: ( localize2WithPath(
                'vs/workbench/contrib/update/browser/update.contribution',
                'installUpdate',
                'Install Update'
            )),
            category: { value: product$1.nameShort, original: product$1.nameShort },
            f1: true,
            precondition: ( CONTEXT_UPDATE_STATE.isEqualTo("downloaded" ))
        });
    }
    async run(accessor) {
        await accessor.get(IUpdateService).applyUpdate();
    }
}
class RestartToUpdateAction extends Action2 {
    constructor() {
        super({
            id: 'update.restartToUpdate',
            title: ( localize2WithPath(
                'vs/workbench/contrib/update/browser/update.contribution',
                'restartToUpdate',
                'Restart to Update'
            )),
            category: { value: product$1.nameShort, original: product$1.nameShort },
            f1: true,
            precondition: ( CONTEXT_UPDATE_STATE.isEqualTo("ready" ))
        });
    }
    async run(accessor) {
        await accessor.get(IUpdateService).quitAndInstall();
    }
}
class DownloadAction extends Action2 {
    static { this.ID = 'workbench.action.download'; }
    constructor() {
        super({
            id: DownloadAction.ID,
            title: ( localize2WithPath(
                'vs/workbench/contrib/update/browser/update.contribution',
                'openDownloadPage',
                "Download {0}",
                product$1.nameLong
            )),
            precondition: ( ContextKeyExpr.and(IsWebContext, DOWNLOAD_URL)),
            f1: true,
            menu: [{
                    id: MenuId.StatusBarWindowIndicatorMenu,
                    when: ( ContextKeyExpr.and(IsWebContext, DOWNLOAD_URL))
                }]
        });
    }
    run(accessor) {
        const productService = accessor.get(IProductService);
        const openerService = accessor.get(IOpenerService);
        if (productService.downloadUrl) {
            openerService.open(( URI.parse(productService.downloadUrl)));
        }
    }
}
registerAction2(DownloadAction);
registerAction2(CheckForUpdateAction);
registerAction2(DownloadUpdateAction);
registerAction2(InstallUpdateAction);
registerAction2(RestartToUpdateAction);
if (isWindows) {
    class DeveloperApplyUpdateAction extends Action2 {
        constructor() {
            super({
                id: '_update.applyupdate',
                title: ( localize2WithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    'applyUpdate',
                    'Apply Update...'
                )),
                category: Categories.Developer,
                f1: true,
                precondition: ( CONTEXT_UPDATE_STATE.isEqualTo("idle" ))
            });
        }
        async run(accessor) {
            const updateService = accessor.get(IUpdateService);
            const fileDialogService = accessor.get(IFileDialogService);
            const updatePath = await fileDialogService.showOpenDialog({
                title: ( localizeWithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    'pickUpdate',
                    "Apply Update"
                )),
                filters: [{ name: 'Setup', extensions: ['exe'] }],
                canSelectFiles: true,
                openLabel: mnemonicButtonLabel(( localizeWithPath(
                    'vs/workbench/contrib/update/browser/update.contribution',
                    { key: 'updateButton', comment: ['&& denotes a mnemonic'] },
                    "&&Update"
                )))
            });
            if (!updatePath || !updatePath[0]) {
                return;
            }
            await updateService._applySpecificUpdate(updatePath[0].fsPath);
        }
    }
    registerAction2(DeveloperApplyUpdateAction);
}
export { CheckForUpdateAction, ShowCurrentReleaseNotesAction, ShowCurrentReleaseNotesFromCurrentFileAction };
