import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { WindowIdleValue } from '../../../../base/browser/dom.js';
import { mainWindow } from '../../../../base/browser/window.js';
import { matchesScheme, Schemas } from '../../../../base/common/network.js';
import Severity$1 from '../../../../base/common/severity.js';
import { URI } from '../../../../base/common/uri.js';
import { localizeWithPath } from '../../../../nls.js';
import { IClipboardService } from '../../../../platform/clipboard/common/clipboardService.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import '../../../../../../../override/vs/platform/dialogs/common/dialogs.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { IOpenerService } from '../../../../platform/opener/common/opener.js';
import { IProductService } from '../../../../platform/product/common/productService.js';
import { IQuickInputService } from '../../../../platform/quickinput/common/quickInput.js';
import { IStorageService } from '../../../../platform/storage/common/storage.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.js';
import { IWorkspaceContextService } from '../../../../platform/workspace/common/workspace.js';
import { IWorkspaceTrustManagementService } from '../../../../platform/workspace/common/workspaceTrust.js';
import { readAuthenticationTrustedDomains, readWorkspaceTrustedDomains, readStaticTrustedDomains, configureOpenerTrustedDomainsHandler } from './trustedDomains.js';
import { testUrlMatchesGlob } from '../common/urlGlob.js';
import { IAuthenticationService } from '../../../services/authentication/common/authentication.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { IDialogService } from '../../../../platform/dialogs/common/dialogs.js';
let OpenerValidatorContributions = class OpenerValidatorContributions {
    constructor(_openerService, _storageService, _dialogService, _productService, _quickInputService, _editorService, _clipboardService, _telemetryService, _instantiationService, _authenticationService, _workspaceContextService, _configurationService, _workspaceTrustService) {
        this._openerService = _openerService;
        this._storageService = _storageService;
        this._dialogService = _dialogService;
        this._productService = _productService;
        this._quickInputService = _quickInputService;
        this._editorService = _editorService;
        this._clipboardService = _clipboardService;
        this._telemetryService = _telemetryService;
        this._instantiationService = _instantiationService;
        this._authenticationService = _authenticationService;
        this._workspaceContextService = _workspaceContextService;
        this._configurationService = _configurationService;
        this._workspaceTrustService = _workspaceTrustService;
        this._openerService.registerValidator({ shouldOpen: (uri, options) => this.validateLink(uri, options) });
        this._readAuthenticationTrustedDomainsResult = ( new WindowIdleValue(
            mainWindow,
            () => this._instantiationService.invokeFunction(readAuthenticationTrustedDomains)
        ));
        this._authenticationService.onDidRegisterAuthenticationProvider(() => {
            this._readAuthenticationTrustedDomainsResult?.dispose();
            this._readAuthenticationTrustedDomainsResult = ( new WindowIdleValue(
                mainWindow,
                () => this._instantiationService.invokeFunction(readAuthenticationTrustedDomains)
            ));
        });
        this._readWorkspaceTrustedDomainsResult = ( new WindowIdleValue(
            mainWindow,
            () => this._instantiationService.invokeFunction(readWorkspaceTrustedDomains)
        ));
        this._workspaceContextService.onDidChangeWorkspaceFolders(() => {
            this._readWorkspaceTrustedDomainsResult?.dispose();
            this._readWorkspaceTrustedDomainsResult = ( new WindowIdleValue(
                mainWindow,
                () => this._instantiationService.invokeFunction(readWorkspaceTrustedDomains)
            ));
        });
    }
    async validateLink(resource, openOptions) {
        if (!matchesScheme(resource, Schemas.http) && !matchesScheme(resource, Schemas.https)) {
            return true;
        }
        if (openOptions?.fromWorkspace && this._workspaceTrustService.isWorkspaceTrusted() && !this._configurationService.getValue('workbench.trustedDomains.promptInTrustedWorkspace')) {
            return true;
        }
        const originalResource = resource;
        let resourceUri;
        if (typeof resource === 'string') {
            resourceUri = ( URI.parse(resource));
        }
        else {
            resourceUri = resource;
        }
        const { scheme, authority, path, query, fragment } = resourceUri;
        const domainToOpen = `${scheme}://${authority}`;
        const [workspaceDomains, userDomains] = await Promise.all([this._readWorkspaceTrustedDomainsResult.value, this._readAuthenticationTrustedDomainsResult.value]);
        const { defaultTrustedDomains, trustedDomains, } = this._instantiationService.invokeFunction(readStaticTrustedDomains);
        const allTrustedDomains = [...defaultTrustedDomains, ...trustedDomains, ...userDomains, ...workspaceDomains];
        if (isURLDomainTrusted(resourceUri, allTrustedDomains)) {
            return true;
        }
        else {
            let formattedLink = `${scheme}://${authority}${path}`;
            const linkTail = `${query ? '?' + query : ''}${fragment ? '#' + fragment : ''}`;
            const remainingLength = Math.max(0, 60 - formattedLink.length);
            const linkTailLengthToKeep = Math.min(Math.max(5, remainingLength), linkTail.length);
            if (linkTailLengthToKeep === linkTail.length) {
                formattedLink += linkTail;
            }
            else {
                formattedLink += linkTail.charAt(0) + '...' + linkTail.substring(linkTail.length - linkTailLengthToKeep + 1);
            }
            const { result } = await this._dialogService.prompt({
                type: Severity$1.Info,
                message: ( localizeWithPath(
                    'vs/workbench/contrib/url/browser/trustedDomainsValidator',
                    'openExternalLinkAt',
                    'Do you want {0} to open the external website?',
                    this._productService.nameShort
                )),
                detail: typeof originalResource === 'string' ? originalResource : formattedLink,
                buttons: [
                    {
                        label: ( localizeWithPath(
                            'vs/workbench/contrib/url/browser/trustedDomainsValidator',
                            { key: 'open', comment: ['&& denotes a mnemonic'] },
                            '&&Open'
                        )),
                        run: () => true
                    },
                    {
                        label: ( localizeWithPath(
                            'vs/workbench/contrib/url/browser/trustedDomainsValidator',
                            { key: 'copy', comment: ['&& denotes a mnemonic'] },
                            '&&Copy'
                        )),
                        run: () => {
                            this._clipboardService.writeText(typeof originalResource === 'string' ? originalResource : ( resourceUri.toString(true)));
                            return false;
                        }
                    },
                    {
                        label: ( localizeWithPath(
                            'vs/workbench/contrib/url/browser/trustedDomainsValidator',
                            { key: 'configureTrustedDomains', comment: ['&& denotes a mnemonic'] },
                            'Configure &&Trusted Domains'
                        )),
                        run: async () => {
                            const pickedDomains = await configureOpenerTrustedDomainsHandler(trustedDomains, domainToOpen, resourceUri, this._quickInputService, this._storageService, this._editorService);
                            if (pickedDomains.indexOf('*') !== -1) {
                                return true;
                            }
                            if (isURLDomainTrusted(resourceUri, pickedDomains)) {
                                return true;
                            }
                            return false;
                        }
                    }
                ],
                cancelButton: {
                    run: () => false
                }
            });
            return result;
        }
    }
};
OpenerValidatorContributions = ( __decorate([
    ( __param(0, IOpenerService)),
    ( __param(1, IStorageService)),
    ( __param(2, IDialogService)),
    ( __param(3, IProductService)),
    ( __param(4, IQuickInputService)),
    ( __param(5, IEditorService)),
    ( __param(6, IClipboardService)),
    ( __param(7, ITelemetryService)),
    ( __param(8, IInstantiationService)),
    ( __param(9, IAuthenticationService)),
    ( __param(10, IWorkspaceContextService)),
    ( __param(11, IConfigurationService)),
    ( __param(12, IWorkspaceTrustManagementService))
], OpenerValidatorContributions));
const rLocalhost = /^localhost(:\d+)?$/i;
const r127 = /^127.0.0.1(:\d+)?$/;
function isLocalhostAuthority(authority) {
    return rLocalhost.test(authority) || r127.test(authority);
}
function normalizeURL(url) {
    const caseInsensitiveAuthorities = ['github.com'];
    try {
        const parsed = typeof url === 'string' ? ( URI.parse(url, true)) : url;
        if (caseInsensitiveAuthorities.includes(parsed.authority)) {
            return ( parsed.with({ path: parsed.path.toLowerCase() }).toString(true));
        }
        else {
            return ( parsed.toString(true));
        }
    }
    catch {
        return ( url.toString());
    }
}
function isURLDomainTrusted(url, trustedDomains) {
    url = ( URI.parse(normalizeURL(url)));
    trustedDomains = ( trustedDomains.map(normalizeURL));
    if (isLocalhostAuthority(url.authority)) {
        return true;
    }
    for (let i = 0; i < trustedDomains.length; i++) {
        if (trustedDomains[i] === '*') {
            return true;
        }
        if (testUrlMatchesGlob(url, trustedDomains[i])) {
            return true;
        }
    }
    return false;
}
export { OpenerValidatorContributions, isURLDomainTrusted };
