import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localizeWithPath, localize2WithPath } from '../../../../nls.js';
import { GettingStartedInputSerializer, GettingStartedPage, inWelcomeContext } from './gettingStarted.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { EditorExtensions } from '../../../common/editor.js';
import { registerAction2, Action2, MenuId } from '../../../../platform/actions/common/actions.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { RawContextKey, ContextKeyExpr, IContextKeyService } from '../../../../platform/contextkey/common/contextkey.js';
import { IEditorService, SIDE_GROUP } from '../../../services/editor/common/editorService.js';
import { EditorPaneDescriptor } from '../../../browser/editor.js';
import { SyncDescriptor } from '../../../../platform/instantiation/common/descriptors.js';
import { IWalkthroughsService } from './gettingStartedService.js';
import { GettingStartedInput } from './gettingStartedInput.js';
import { registerWorkbenchContribution2 } from '../../../common/contributions.js';
import { Extensions } from '../../../../platform/configuration/common/configurationRegistry.js';
import { workbenchConfigurationNodeBase } from '../../../common/configuration.js';
import { IEditorGroupsService } from '../../../services/editor/common/editorGroupsService.js';
import { CommandsRegistry, ICommandService } from '../../../../platform/commands/common/commands.js';
import { IQuickInputService } from '../../../../platform/quickinput/common/quickInput.js';
import { IRemoteAgentService } from '../../../services/remote/common/remoteAgentService.js';
import { isMacintosh, isLinux, isWindows } from '../../../../base/common/platform.js';
import { IExtensionManagementServerService } from '../../../services/extensionManagement/common/extensionManagement.js';
import { IExtensionService } from '../../../services/extensions/common/extensions.js';
import { StartupPageEditorResolverContribution, StartupPageRunnerContribution } from './startupPage.js';
import { ExtensionsInput } from '../../extensions/common/extensionsInput.js';
import { Categories } from '../../../../platform/action/common/actionCommonCategories.js';
import './gettingStartedIcons.js';
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.openWalkthrough',
            title: ( localize2WithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'miWelcome',
                'Welcome'
            )),
            category: Categories.Help,
            f1: true,
            menu: {
                id: MenuId.MenubarHelpMenu,
                group: '1_welcome',
                order: 1,
            }
        });
    }
    run(accessor, walkthroughID, toSide) {
        const editorGroupsService = accessor.get(IEditorGroupsService);
        const instantiationService = accessor.get(IInstantiationService);
        const editorService = accessor.get(IEditorService);
        const commandService = accessor.get(ICommandService);
        if (walkthroughID) {
            const selectedCategory = typeof walkthroughID === 'string' ? walkthroughID : walkthroughID.category;
            const selectedStep = typeof walkthroughID === 'string' ? undefined : walkthroughID.category + '#' + walkthroughID.step;
            if (!selectedCategory && !selectedStep) {
                editorService.openEditor({
                    resource: GettingStartedInput.RESOURCE,
                    options: { preserveFocus: toSide ?? false }
                }, toSide ? SIDE_GROUP : undefined);
                return;
            }
            for (const group of editorGroupsService.groups) {
                if (group.activeEditor instanceof GettingStartedInput) {
                    group.activeEditorPane.makeCategoryVisibleWhenAvailable(selectedCategory, selectedStep);
                    return;
                }
            }
            const result = editorService.findEditors({ typeId: GettingStartedInput.ID, editorId: undefined, resource: GettingStartedInput.RESOURCE });
            for (const { editor, groupId } of result) {
                if (editor instanceof GettingStartedInput) {
                    const group = editorGroupsService.getGroup(groupId);
                    if (!editor.selectedCategory && group) {
                        editor.selectedCategory = selectedCategory;
                        editor.selectedStep = selectedStep;
                        group.openEditor(editor, { revealIfOpened: true });
                        return;
                    }
                }
            }
            const activeEditor = editorService.activeEditor;
            if (selectedStep && activeEditor instanceof GettingStartedInput && activeEditor.selectedCategory === selectedCategory) {
                commandService.executeCommand('walkthroughs.selectStep', selectedStep);
                return;
            }
            if (activeEditor instanceof ExtensionsInput) {
                const activeGroup = editorGroupsService.activeGroup;
                activeGroup.replaceEditors([{
                        editor: activeEditor,
                        replacement: instantiationService.createInstance(GettingStartedInput, { selectedCategory: selectedCategory, selectedStep: selectedStep })
                    }]);
            }
            else {
                editorService.openEditor({
                    resource: GettingStartedInput.RESOURCE,
                    options: { selectedCategory: selectedCategory, selectedStep: selectedStep, preserveFocus: toSide ?? false }
                }, toSide ? SIDE_GROUP : undefined).then((editor) => {
                    editor?.makeCategoryVisibleWhenAvailable(selectedCategory, selectedStep);
                });
            }
        }
        else {
            editorService.openEditor({
                resource: GettingStartedInput.RESOURCE,
                options: { preserveFocus: toSide ?? false }
            }, toSide ? SIDE_GROUP : undefined);
        }
    }
});
( Registry.as(EditorExtensions.EditorFactory)).registerEditorSerializer(GettingStartedInput.ID, GettingStartedInputSerializer);
( Registry.as(EditorExtensions.EditorPane)).registerEditorPane(EditorPaneDescriptor.create(GettingStartedPage, GettingStartedPage.ID, ( localizeWithPath(
    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
    'welcome',
    "Welcome"
))), [
    ( new SyncDescriptor(GettingStartedInput))
]);
const category = ( localize2WithPath(
    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
    'welcome',
    "Welcome"
));
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'welcome.goBack',
            title: ( localize2WithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'welcome.goBack',
                'Go Back'
            )),
            category,
            keybinding: {
                weight: 100 ,
                primary: 9 ,
                when: inWelcomeContext
            },
            precondition: ( ContextKeyExpr.equals('activeEditor', 'gettingStartedPage')),
            f1: true
        });
    }
    run(accessor) {
        const editorService = accessor.get(IEditorService);
        const editorPane = editorService.activeEditorPane;
        if (editorPane instanceof GettingStartedPage) {
            editorPane.escape();
        }
    }
});
CommandsRegistry.registerCommand({
    id: 'walkthroughs.selectStep',
    handler: (accessor, stepID) => {
        const editorService = accessor.get(IEditorService);
        const editorPane = editorService.activeEditorPane;
        if (editorPane instanceof GettingStartedPage) {
            editorPane.selectStepLoose(stepID);
        }
        else {
            console.error('Cannot run walkthroughs.selectStep outside of walkthrough context');
        }
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'welcome.markStepComplete',
            title: ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'welcome.markStepComplete',
                "Mark Step Complete"
            )),
            category,
        });
    }
    run(accessor, arg) {
        if (!arg) {
            return;
        }
        const gettingStartedService = accessor.get(IWalkthroughsService);
        gettingStartedService.progressStep(arg);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'welcome.markStepIncomplete',
            title: ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'welcome.markStepInomplete',
                "Mark Step Incomplete"
            )),
            category,
        });
    }
    run(accessor, arg) {
        if (!arg) {
            return;
        }
        const gettingStartedService = accessor.get(IWalkthroughsService);
        gettingStartedService.deprogressStep(arg);
    }
});
registerAction2(class extends Action2 {
    constructor() {
        super({
            id: 'welcome.showAllWalkthroughs',
            title: ( localize2WithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'welcome.showAllWalkthroughs',
                'Open Walkthrough...'
            )),
            category,
            f1: true,
        });
    }
    async getQuickPickItems(contextService, gettingStartedService) {
        const categories = await gettingStartedService.getWalkthroughs();
        return ( categories
            .filter(c => contextService.contextMatchesRules(c.when))
            .map(x => ({
            id: x.id,
            label: x.title,
            detail: x.description,
            description: x.source,
        })));
    }
    async run(accessor) {
        const commandService = accessor.get(ICommandService);
        const contextService = accessor.get(IContextKeyService);
        const quickInputService = accessor.get(IQuickInputService);
        const gettingStartedService = accessor.get(IWalkthroughsService);
        const extensionService = accessor.get(IExtensionService);
        const quickPick = quickInputService.createQuickPick();
        quickPick.canSelectMany = false;
        quickPick.matchOnDescription = true;
        quickPick.matchOnDetail = true;
        quickPick.placeholder = ( localizeWithPath(
            'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
            'pickWalkthroughs',
            'Select a walkthrough to open'
        ));
        quickPick.items = await this.getQuickPickItems(contextService, gettingStartedService);
        quickPick.busy = true;
        quickPick.onDidAccept(() => {
            const selection = quickPick.selectedItems[0];
            if (selection) {
                commandService.executeCommand('workbench.action.openWalkthrough', selection.id);
            }
            quickPick.hide();
        });
        quickPick.onDidHide(() => quickPick.dispose());
        await extensionService.whenInstalledExtensionsRegistered();
        gettingStartedService.onDidAddWalkthrough(async () => {
            quickPick.items = await this.getQuickPickItems(contextService, gettingStartedService);
        });
        quickPick.show();
        quickPick.busy = false;
    }
});
const WorkspacePlatform = ( new RawContextKey('workspacePlatform', undefined, ( localizeWithPath(
    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
    'workspacePlatform',
    "The platform of the current workspace, which in remote or serverless contexts may be different from the platform of the UI"
))));
let WorkspacePlatformContribution = class WorkspacePlatformContribution {
    static { this.ID = 'workbench.contrib.workspacePlatform'; }
    constructor(extensionManagementServerService, remoteAgentService, contextService) {
        this.extensionManagementServerService = extensionManagementServerService;
        this.remoteAgentService = remoteAgentService;
        this.contextService = contextService;
        this.remoteAgentService.getEnvironment().then(env => {
            const remoteOS = env?.os;
            const remotePlatform = remoteOS === 2  ? 'mac'
                : remoteOS === 1  ? 'windows'
                    : remoteOS === 3  ? 'linux'
                        : undefined;
            if (remotePlatform) {
                WorkspacePlatform.bindTo(this.contextService).set(remotePlatform);
            }
            else if (this.extensionManagementServerService.localExtensionManagementServer) {
                if (isMacintosh) {
                    WorkspacePlatform.bindTo(this.contextService).set('mac');
                }
                else if (isLinux) {
                    WorkspacePlatform.bindTo(this.contextService).set('linux');
                }
                else if (isWindows) {
                    WorkspacePlatform.bindTo(this.contextService).set('windows');
                }
            }
            else if (this.extensionManagementServerService.webExtensionManagementServer) {
                WorkspacePlatform.bindTo(this.contextService).set('webworker');
            }
            else {
                console.error('Error: Unable to detect workspace platform');
            }
        });
    }
};
WorkspacePlatformContribution = ( __decorate([
    ( __param(0, IExtensionManagementServerService)),
    ( __param(1, IRemoteAgentService)),
    ( __param(2, IContextKeyService))
], WorkspacePlatformContribution));
const configurationRegistry = ( Registry.as(Extensions.Configuration));
configurationRegistry.registerConfiguration({
    ...workbenchConfigurationNodeBase,
    properties: {
        'workbench.welcomePage.walkthroughs.openOnInstall': {
            scope: 2 ,
            type: 'boolean',
            default: true,
            description: ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'workbench.welcomePage.walkthroughs.openOnInstall',
                "When enabled, an extension's walkthrough will open upon install of the extension."
            ))
        },
        'workbench.startupEditor': {
            'scope': 4 ,
            'type': 'string',
            'enum': ['none', 'welcomePage', 'readme', 'newUntitledFile', 'welcomePageInEmptyWorkbench', 'terminal'],
            'enumDescriptions': [
                ( localizeWithPath(
                    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                    { comment: ['This is the description for a setting. Values surrounded by single quotes are not to be translated.'], key: 'workbench.startupEditor.none' },
                    "Start without an editor."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                    { comment: ['This is the description for a setting. Values surrounded by single quotes are not to be translated.'], key: 'workbench.startupEditor.welcomePage' },
                    "Open the Welcome page, with content to aid in getting started with VS Code and extensions."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                    { comment: ['This is the description for a setting. Values surrounded by single quotes are not to be translated.'], key: 'workbench.startupEditor.readme' },
                    "Open the README when opening a folder that contains one, fallback to 'welcomePage' otherwise. Note: This is only observed as a global configuration, it will be ignored if set in a workspace or folder configuration."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                    { comment: ['This is the description for a setting. Values surrounded by single quotes are not to be translated.'], key: 'workbench.startupEditor.newUntitledFile' },
                    "Open a new untitled text file (only applies when opening an empty window)."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                    { comment: ['This is the description for a setting. Values surrounded by single quotes are not to be translated.'], key: 'workbench.startupEditor.welcomePageInEmptyWorkbench' },
                    "Open the Welcome page when opening an empty workbench."
                )),
                ( localizeWithPath(
                    'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                    { comment: ['This is the description for a setting. Values surrounded by single quotes are not to be translated.'], key: 'workbench.startupEditor.terminal' },
                    "Open a new terminal in the editor area."
                )),
            ],
            'default': 'welcomePage',
            'description': ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'workbench.startupEditor',
                "Controls which editor is shown at startup, if none are restored from the previous session."
            ))
        },
        'workbench.welcomePage.preferReducedMotion': {
            scope: 1 ,
            type: 'boolean',
            default: false,
            deprecationMessage: ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'deprecationMessage',
                "Deprecated, use the global `workbench.reduceMotion`."
            )),
            description: ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/gettingStarted.contribution',
                'workbench.welcomePage.preferReducedMotion',
                "When enabled, reduce motion in welcome page."
            ))
        }
    }
});
registerWorkbenchContribution2(WorkspacePlatformContribution.ID, WorkspacePlatformContribution, 3 );
registerWorkbenchContribution2(StartupPageEditorResolverContribution.ID, StartupPageEditorResolverContribution, 2 );
registerWorkbenchContribution2(StartupPageRunnerContribution.ID, StartupPageRunnerContribution, 3 );
export { WorkspacePlatform };
