import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { ICommandService } from '../../../../platform/commands/common/commands.js';
import { coalesce } from '../../../../base/common/arrays.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { IEditorService } from '../../../services/editor/common/editorService.js';
import { onUnexpectedError } from '../../../../base/common/errors.js';
import { UNKNOWN_EMPTY_WINDOW_WORKSPACE, IWorkspaceContextService } from '../../../../platform/workspace/common/workspace.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.js';
import { IWorkingCopyBackupService } from '../../../services/workingCopy/common/workingCopyBackup.js';
import { ILifecycleService } from '../../../services/lifecycle/common/lifecycle.js';
import { IFileService } from '../../../../platform/files/common/files.js';
import { joinPath } from '../../../../base/common/resources.js';
import { IWorkbenchLayoutService } from '../../../services/layout/browser/layoutService.js';
import { GettingStartedInput, gettingStartedInputTypeId } from './gettingStartedInput.js';
import { IWorkbenchEnvironmentService } from '../../../services/environment/common/environmentService.js';
import { IStorageService } from '../../../../platform/storage/common/storage.js';
import { getTelemetryLevel } from '../../../../platform/telemetry/common/telemetryUtils.js';
import { IProductService } from '../../../../platform/product/common/productService.js';
import { ILogService } from '../../../../platform/log/common/log.js';
import { INotificationService } from '../../../../platform/notification/common/notification.js';
import { localizeWithPath } from '../../../../nls.js';
import { RegisteredEditorPriority, IEditorResolverService } from '../../../services/editor/common/editorResolverService.js';
const restoreWalkthroughsConfigurationKey = 'workbench.welcomePage.restorableWalkthroughs';
const configurationKey = 'workbench.startupEditor';
const oldConfigurationKey = 'workbench.welcome.enabled';
const telemetryOptOutStorageKey = 'workbench.telemetryOptOutShown';
let StartupPageEditorResolverContribution = class StartupPageEditorResolverContribution {
    static { this.ID = 'workbench.contrib.startupPageEditorResolver'; }
    constructor(instantiationService, editorResolverService) {
        this.instantiationService = instantiationService;
        editorResolverService.registerEditor(`${GettingStartedInput.RESOURCE.scheme}:/**`, {
            id: GettingStartedInput.ID,
            label: ( localizeWithPath(
                'vs/workbench/contrib/welcomeGettingStarted/browser/startupPage',
                'welcome.displayName',
                "Welcome Page"
            )),
            priority: RegisteredEditorPriority.builtin,
        }, {
            singlePerResource: false,
            canSupportResource: uri => uri.scheme === GettingStartedInput.RESOURCE.scheme,
        }, {
            createEditorInput: ({ resource, options }) => {
                return {
                    editor: this.instantiationService.createInstance(GettingStartedInput, options),
                    options: {
                        ...options,
                        pinned: false
                    }
                };
            }
        });
    }
};
StartupPageEditorResolverContribution = ( __decorate([
    ( __param(0, IInstantiationService)),
    ( __param(1, IEditorResolverService))
], StartupPageEditorResolverContribution));
let StartupPageRunnerContribution = class StartupPageRunnerContribution {
    static { this.ID = 'workbench.contrib.startupPageRunner'; }
    constructor(configurationService, editorService, workingCopyBackupService, fileService, contextService, lifecycleService, layoutService, productService, commandService, environmentService, storageService, logService, notificationService) {
        this.configurationService = configurationService;
        this.editorService = editorService;
        this.workingCopyBackupService = workingCopyBackupService;
        this.fileService = fileService;
        this.contextService = contextService;
        this.lifecycleService = lifecycleService;
        this.layoutService = layoutService;
        this.productService = productService;
        this.commandService = commandService;
        this.environmentService = environmentService;
        this.storageService = storageService;
        this.logService = logService;
        this.notificationService = notificationService;
        this.run().then(undefined, onUnexpectedError);
    }
    async run() {
        await this.lifecycleService.when(3 );
        if (this.productService.enableTelemetry
            && this.productService.showTelemetryOptOut
            && getTelemetryLevel(this.configurationService) !== 0
            && !this.environmentService.skipWelcome
            && !this.storageService.get(telemetryOptOutStorageKey, 0 )) {
            this.storageService.store(telemetryOptOutStorageKey, true, 0 , 0 );
            await this.openGettingStarted(true);
            return;
        }
        if (this.tryOpenWalkthroughForFolder()) {
            return;
        }
        const enabled = isStartupPageEnabled(this.configurationService, this.contextService, this.environmentService);
        if (enabled && this.lifecycleService.startupKind !== 3 ) {
            const hasBackups = await this.workingCopyBackupService.hasBackups();
            if (hasBackups) {
                return;
            }
            if (!this.editorService.activeEditor || this.layoutService.openedDefaultEditors) {
                const startupEditorSetting = this.configurationService.inspect(configurationKey);
                const isStartupEditorReadme = startupEditorSetting.value === 'readme';
                const isStartupEditorUserReadme = startupEditorSetting.userValue === 'readme';
                const isStartupEditorDefaultReadme = startupEditorSetting.defaultValue === 'readme';
                if (isStartupEditorReadme && (!isStartupEditorUserReadme || !isStartupEditorDefaultReadme)) {
                    this.logService.warn(`Warning: 'workbench.startupEditor: readme' setting ignored due to being set somewhere other than user or default settings (user=${startupEditorSetting.userValue}, default=${startupEditorSetting.defaultValue})`);
                }
                const openWithReadme = isStartupEditorReadme && (isStartupEditorUserReadme || isStartupEditorDefaultReadme);
                if (openWithReadme) {
                    await this.openReadme();
                }
                else if (startupEditorSetting.value === 'welcomePage' || startupEditorSetting.value === 'welcomePageInEmptyWorkbench') {
                    await this.openGettingStarted();
                }
                else if (startupEditorSetting.value === 'terminal') {
                    this.commandService.executeCommand("workbench.action.createTerminalEditor" );
                }
            }
        }
    }
    tryOpenWalkthroughForFolder() {
        const toRestore = this.storageService.get(restoreWalkthroughsConfigurationKey, 0 );
        if (!toRestore) {
            return false;
        }
        else {
            const restoreData = JSON.parse(toRestore);
            const currentWorkspace = this.contextService.getWorkspace();
            if (restoreData.folder === UNKNOWN_EMPTY_WINDOW_WORKSPACE.id || restoreData.folder === ( currentWorkspace.folders[0].uri.toString())) {
                this.editorService.openEditor({
                    resource: GettingStartedInput.RESOURCE,
                    options: { selectedCategory: restoreData.category, selectedStep: restoreData.step, pinned: false },
                });
                this.storageService.remove(restoreWalkthroughsConfigurationKey, 0 );
                return true;
            }
        }
        return false;
    }
    async openReadme() {
        const readmes = coalesce(await Promise.all(( this.contextService.getWorkspace().folders.map(async (folder) => {
            const folderUri = folder.uri;
            const folderStat = await this.fileService.resolve(folderUri).catch(onUnexpectedError);
            const files = folderStat?.children ? ( folderStat.children.map(child => child.name)).sort() : [];
            const file = files.find(file => file.toLowerCase() === 'readme.md') || files.find(file => file.toLowerCase().startsWith('readme'));
            if (file) {
                return joinPath(folderUri, file);
            }
            else {
                return undefined;
            }
        }))));
        if (!this.editorService.activeEditor) {
            if (readmes.length) {
                const isMarkDown = (readme) => readme.path.toLowerCase().endsWith('.md');
                await Promise.all([
                    this.commandService.executeCommand('markdown.showPreview', null, readmes.filter(isMarkDown), { locked: true }).catch(error => {
                        this.notificationService.error(( localizeWithPath(
                            'vs/workbench/contrib/welcomeGettingStarted/browser/startupPage',
                            'startupPage.markdownPreviewError',
                            'Could not open markdown preview: {0}.\n\nPlease make sure the markdown extension is enabled.',
                            error.message
                        )));
                    }),
                    this.editorService.openEditors(( readmes.filter(readme => !isMarkDown(readme)).map(readme => ({ resource: readme })))),
                ]);
            }
            else {
                await this.openGettingStarted();
            }
        }
    }
    async openGettingStarted(showTelemetryNotice) {
        const startupEditorTypeID = gettingStartedInputTypeId;
        const editor = this.editorService.activeEditor;
        if (editor?.typeId === startupEditorTypeID || ( this.editorService.editors.some(e => e.typeId === startupEditorTypeID))) {
            return;
        }
        const options = editor ? { pinned: false, index: 0 } : { pinned: false };
        {
            this.editorService.openEditor({
                resource: GettingStartedInput.RESOURCE,
                options: { showTelemetryNotice, ...options },
            });
        }
    }
};
StartupPageRunnerContribution = ( __decorate([
    ( __param(0, IConfigurationService)),
    ( __param(1, IEditorService)),
    ( __param(2, IWorkingCopyBackupService)),
    ( __param(3, IFileService)),
    ( __param(4, IWorkspaceContextService)),
    ( __param(5, ILifecycleService)),
    ( __param(6, IWorkbenchLayoutService)),
    ( __param(7, IProductService)),
    ( __param(8, ICommandService)),
    ( __param(9, IWorkbenchEnvironmentService)),
    ( __param(10, IStorageService)),
    ( __param(11, ILogService)),
    ( __param(12, INotificationService))
], StartupPageRunnerContribution));
function isStartupPageEnabled(configurationService, contextService, environmentService) {
    if (environmentService.skipWelcome) {
        return false;
    }
    const startupEditor = configurationService.inspect(configurationKey);
    if (!startupEditor.userValue && !startupEditor.workspaceValue) {
        const welcomeEnabled = configurationService.inspect(oldConfigurationKey);
        if (welcomeEnabled.value !== undefined && welcomeEnabled.value !== null) {
            return welcomeEnabled.value;
        }
    }
    return startupEditor.value === 'welcomePage'
        || startupEditor.value === 'readme' && (startupEditor.userValue === 'readme' || startupEditor.defaultValue === 'readme')
        || (contextService.getWorkbenchState() === 1  && startupEditor.value === 'welcomePageInEmptyWorkbench')
        || startupEditor.value === 'terminal';
}
export { StartupPageEditorResolverContribution, StartupPageRunnerContribution, restoreWalkthroughsConfigurationKey };
