import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { ITextModelService } from '../../../../editor/common/services/resolverService.js';
import { IModelService } from '../../../../editor/common/services/model.js';
import { ILanguageService } from '../../../../editor/common/languages/language.js';
import marked_1 from '../../../../../../../external/vscode-marked/lib/marked.esm.js';
import { Schemas, FileAccess } from '../../../../base/common/network.js';
import { Range } from '../../../../editor/common/core/range.js';
import { createTextBufferFactory } from '../../../../editor/common/model/textModel.js';
import { assertIsDefined } from '../../../../base/common/types.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
async function requireToContent(instantiationService, resource) {
    if (!resource.query) {
        throw new Error('Welcome: invalid resource');
    }
    const query = JSON.parse(resource.query);
    if (!query.moduleId) {
        throw new Error('Welcome: invalid resource');
    }
    const content = await FileAccess.toModuleContent(query.moduleId);
    return instantiationService.invokeFunction(content);
}
let WalkThroughSnippetContentProvider = class WalkThroughSnippetContentProvider {
    static { this.ID = 'workbench.contrib.walkThroughSnippetContentProvider'; }
    constructor(textModelResolverService, languageService, modelService, instantiationService) {
        this.textModelResolverService = textModelResolverService;
        this.languageService = languageService;
        this.modelService = modelService;
        this.instantiationService = instantiationService;
        this.loads = ( new Map());
        this.textModelResolverService.registerTextModelContentProvider(Schemas.walkThroughSnippet, this);
    }
    async textBufferFactoryFromResource(resource) {
        let ongoing = this.loads.get(( resource.toString()));
        if (!ongoing) {
            ongoing = requireToContent(this.instantiationService, resource)
                .then(content => createTextBufferFactory(content))
                .finally(() => this.loads.delete(( resource.toString())));
            this.loads.set(( resource.toString()), ongoing);
        }
        return ongoing;
    }
    async provideTextContent(resource) {
        const factory = await this.textBufferFactoryFromResource(resource.with({ fragment: '' }));
        let codeEditorModel = this.modelService.getModel(resource);
        if (!codeEditorModel) {
            const j = parseInt(resource.fragment);
            let i = 0;
            const renderer = new marked_1.Renderer();
            renderer.code = (code, lang) => {
                i++;
                const languageId = typeof lang === 'string' ? this.languageService.getLanguageIdByLanguageName(lang) || '' : '';
                const languageSelection = this.languageService.createById(languageId);
                const model = this.modelService.createModel(code, languageSelection, resource.with({ fragment: `${i}.${lang}` }));
                if (i === j) {
                    codeEditorModel = model;
                }
                return '';
            };
            const textBuffer = factory.create(1 ).textBuffer;
            const lineCount = textBuffer.getLineCount();
            const range = ( new Range(1, 1, lineCount, textBuffer.getLineLength(lineCount) + 1));
            const markdown = textBuffer.getValueInRange(range, 0 );
            marked_1(markdown, { renderer });
        }
        return assertIsDefined(codeEditorModel);
    }
};
WalkThroughSnippetContentProvider = ( __decorate([
    ( __param(0, ITextModelService)),
    ( __param(1, ILanguageService)),
    ( __param(2, IModelService)),
    ( __param(3, IInstantiationService))
], WalkThroughSnippetContentProvider));
export { WalkThroughSnippetContentProvider, requireToContent };
