import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { IRemoteAgentService } from './remoteAgentService.js';
import { RemoteExtensionsScannerChannelName } from '../../../../platform/remote/common/remoteExtensionsScanner.js';
import { language } from '../../../../base/common/platform.js';
import { URI } from '../../../../base/common/uri.js';
import { IUserDataProfileService } from '../../userDataProfile/common/userDataProfile.js';
import { IRemoteUserDataProfilesService } from '../../userDataProfile/common/remoteUserDataProfiles.js';
import { IWorkbenchEnvironmentService } from '../../environment/common/environmentService.js';
import { ILogService } from '../../../../platform/log/common/log.js';
import { IActiveLanguagePackService } from '../../localization/common/locale.js';
let RemoteExtensionsScannerService = class RemoteExtensionsScannerService {
    constructor(remoteAgentService, environmentService, userDataProfileService, remoteUserDataProfilesService, logService, activeLanguagePackService) {
        this.remoteAgentService = remoteAgentService;
        this.environmentService = environmentService;
        this.userDataProfileService = userDataProfileService;
        this.remoteUserDataProfilesService = remoteUserDataProfilesService;
        this.logService = logService;
        this.activeLanguagePackService = activeLanguagePackService;
    }
    whenExtensionsReady() {
        return this.withChannel(channel => channel.call('whenExtensionsReady'), undefined);
    }
    async scanExtensions() {
        try {
            const languagePack = await this.activeLanguagePackService.getExtensionIdProvidingCurrentLocale();
            return await this.withChannel(async (channel) => {
                const profileLocation = this.userDataProfileService.currentProfile.isDefault ? undefined : (await this.remoteUserDataProfilesService.getRemoteProfile(this.userDataProfileService.currentProfile)).extensionsResource;
                const scannedExtensions = await channel.call('scanExtensions', [language, profileLocation, this.environmentService.extensionDevelopmentLocationURI, languagePack]);
                scannedExtensions.forEach((extension) => {
                    extension.extensionLocation = URI.revive(extension.extensionLocation);
                });
                return scannedExtensions;
            }, []);
        }
        catch (error) {
            this.logService.error(error);
            return [];
        }
    }
    async scanSingleExtension(extensionLocation, isBuiltin) {
        try {
            return await this.withChannel(async (channel) => {
                const extension = await channel.call('scanSingleExtension', [extensionLocation, isBuiltin, language]);
                if (extension !== null) {
                    extension.extensionLocation = URI.revive(extension.extensionLocation);
                }
                return extension;
            }, null);
        }
        catch (error) {
            this.logService.error(error);
            return null;
        }
    }
    withChannel(callback, fallback) {
        const connection = this.remoteAgentService.getConnection();
        if (!connection) {
            return Promise.resolve(fallback);
        }
        return connection.withChannel(RemoteExtensionsScannerChannelName, (channel) => callback(channel));
    }
};
RemoteExtensionsScannerService = ( __decorate([
    ( __param(0, IRemoteAgentService)),
    ( __param(1, IWorkbenchEnvironmentService)),
    ( __param(2, IUserDataProfileService)),
    ( __param(3, IRemoteUserDataProfilesService)),
    ( __param(4, ILogService)),
    ( __param(5, IActiveLanguagePackService))
], RemoteExtensionsScannerService));
export { RemoteExtensionsScannerService };
